// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections.Generic;

namespace DotCommunities.Framework.Collections
{
    /// <summary>
    /// Represents a collection of <see cref="Section"/> objects containing section data.
    /// </summary>
    public class SectionCollection : Dictionary<string, Section>
    {
        private Section homeSection;
        private Dictionary<int, Section> sectionsByID;

        /// <summary>
        /// Create a new collection ignoring the case.
        /// </summary>
        public SectionCollection()
            : base(StringComparer.OrdinalIgnoreCase)
        {
            sectionsByID = new Dictionary<int, Section>();
        }

        /// <summary>
        /// Adds a <see cref="Section"/> to the collection.
        /// </summary>
        /// <param name="key">The section path.</param>
        /// <param name="value">The <see cref="Section"/> to add.</param>
        public new void Add(string key, Section value)
        {
            // Don't allow duplicate values
            if (ContainsKey(key))
                return;

            if (value.ParentID == -1)
                homeSection = value;

            sectionsByID.Add(value.SectionID, value);
            base.Add(key, value);
        }

        /// <summary>
        /// Gets or sets a <see cref="Section"/> object with the required path.
        /// </summary>
        /// <param name="key">The section path.</param>
        /// <returns>The <see cref="Section"/> corresponding to the path.</returns>
        public new Section this[string key]
        {
            get { return base[key]; }
            set
            {
                // Don't allow duplicate values
                if (ContainsKey(key))
                    return;

                sectionsByID.Add(value.SectionID, value);
                base[key] = value;
            }
        }

        /// <summary>
        /// Gets the home section.
        /// </summary>
        public Section DefaultSection
        {
            get { return homeSection; }
        }

        /// <summary>
        /// Gets the required section based on the section ID.
        /// </summary>
        /// <param name="sectionID">The ID of the section to retrieve.</param>
        /// <returns>A <see cref="Section"/> with the required section ID.</returns>
        public Section GetSectionByID(int sectionID)
        {
            return sectionsByID[sectionID];
        }
    }
}
