// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;
using System.Collections.Generic;

using DotCommunities.Framework.Managers;

namespace DotCommunities.Framework
{
    /// <summary>
    /// Represents all information about a particular content item.
    /// This is the base class for the module classes. 
    /// </summary>
    [Serializable]
    public class CommunityContent
    {
        int _contentPageID = -1;
        string _title = String.Empty;
        string _description = String.Empty;
        
        string _body = String.Empty;
        string _formattedBody = String.Empty;
        string _searchText = String.Empty;
        DateTime _dateCreated;
        DateTime _dateUpdated;
        DateTime _dateCommented;
        int _sectionID = -1;
        string _author = String.Empty;
        string _ipaddress = String.Empty;
        int _viewCount = -1;
        int _commentCount = -1;
        int _aggregatedViewCount = -1;
        bool _hasRead = false;
        bool _isLocked = false;
        bool _isIndexed = false;
        string _remoteAuthor = String.Empty;
        string _remoteipaddress = String.Empty;

        Dictionary<int, Category> categories = new Dictionary<int,Category>();

        /// <summary>
        /// Initializes content information. 
        /// </summary>
        /// <param name="dr">An IDataReader containing the content information.</param>
        public CommunityContent(IDataReader dr)
        {
            // Required fields
            _contentPageID = (int)dr["ContentPage_ID"];
            _title = dr["ContentPage_Title"].ToString();
            _description = dr["ContentPage_Description"].ToString();
            //_body = dr["Body"].ToString();
            //_formattedBody = dr["FormattedBody"].ToString();
            //_searchText = dr["SearchText"].ToString();
            _dateCreated = (DateTime)dr["ContentPage_DateCreated"];
            _sectionID = (int)dr["ContentPage_SectionID"];
            _viewCount = (int)dr["ContentPage_ViewCount"];
            _commentCount = (int)dr["CommentCount"];
            //_aggregatedViewCount = (int)dr["AggregatedViews"];
            _hasRead = (bool)dr["HasRead"];
            //_isLocked = (bool)dr["IsLocked"];
            //_isIndexed = (bool)dr["IsIndexed"];
            //_ipaddress = dr["IPAddress"].ToString();
            _author = dr["UserName"].ToString();
            //_remoteipaddress = dr["RemoteIPAddress"].ToString();
            //_remoteAuthor = dr["RemoteUserName"].ToString();

            // Optional fields
            if (dr["ContentPage_DateUpdated"] != DBNull.Value)
                _dateUpdated = (DateTime)dr["ContentPage_DateUpdated"];
            if (dr["ContentPage_DateCommented"] != DBNull.Value)
                _dateCommented = (DateTime)dr["ContentPage_DateCommented"];

            // Build up the categories dictionary from the supplied category string
            if (dr["Categories"] != DBNull.Value)
            {
                Dictionary<int, Category> allCategories = FrameworkManager.GetCategories();
                string[] categoryIDs = dr["Categories"].ToString().Split('|');
                foreach (string s in categoryIDs)
                {
                    int categoryID = 0;
                    if (int.TryParse(s, out categoryID) && allCategories.ContainsKey(categoryID))
                        categories.Add(categoryID, allCategories[categoryID]);
                }
            }
        }

        /// <summary>
        /// Constructor used to index search content.
        /// </summary>
        /// <param name="dr">The search information as a data row.</param>
        public CommunityContent(DataRow dr)
        {
            // Required fields
            _contentPageID = (int)dr["ContentPage_ID"];
            _title = dr["ContentPage_Title"].ToString();
            _description = dr["ContentPage_Description"].ToString();
            //_searchText = dr["ContentPage_SearchText"].ToString();
            _dateCreated = (DateTime)dr["ContentPage_DateCreated"];
            _sectionID = (int)dr["ContentPage_SectionID"];
            //_isLocked = (bool)dr["ContentPage_IsLocked"];
            //_isIndexed = (bool)dr["ContentPage_IsIndexed"];
            _viewCount = (int)dr["ContentPage_ViewCount"];
            _commentCount = (int)dr["CommentCount"];
           // _aggregatedViewCount = (int)dr["ContentPage_AggregatedViews"];
           // _ipaddress = dr["ContentPage_IPAddress"].ToString();
            _author = dr["UserName"].ToString();
           // _remoteipaddress = dr["ContentPage_RemoteIPAddress"].ToString();
            //_remoteAuthor = dr["ContentPage_RemoteUserName"].ToString();

            // Optional fields
            if (dr["ContentPage_DateUpdated"] != DBNull.Value)
                _dateUpdated = (DateTime)dr["ContentPage_DateUpdated"];
            if (dr["ContentPage_DateCommented"] != DBNull.Value)
                _dateCommented = (DateTime)dr["ContentPage_DateCommented"];
        }

        /// <summary>
        /// Constructor used for creating new content.
        /// </summary>
        public CommunityContent() { }

        /// <summary>
        /// Represents the content page ID. 
        /// </summary>
        public int ContentPageID
        {
            get { return _contentPageID; }
            set { _contentPageID = value; }
        }

        /// <summary>
        /// Represents the content page Title. 
        /// </summary>
        public string Title
        {
            get { return _title; }
            set { _title = value; }
        }

        /// <summary>
        /// Represents the content page description. 
        /// </summary>
        public string BriefDescription
        {
            get { return _description; }
            set { _description = value; }
        }

        /// <summary>
        /// Represents the original body of the page.
        /// </summary>
        public string Body
        {
            get { return _body; }
            set { _body = value; }
        }

        /// <summary>
        /// Represents the formatted body of the page.
        /// </summary>
        public string FormattedBody
        {
            get { return _formattedBody; }
            set { _formattedBody = value; }
        }

        /// <summary>
        /// Represents the search text used to index the content page.
        /// </summary>
        public string SearchText
        {
            get { return _searchText; }
            set { _searchText = value; }
        }

        /// <summary>
        /// Represents the date that a content page was added to
        /// the database.
        /// </summary>
        public DateTime DateCreated
        {
            get { return _dateCreated; }
            set { _dateCreated = value; }
        }

        /// <summary>
        /// Represents the date that a content page was last updated.
        /// </summary>
        public DateTime DateUpdated
        {
            get { return _dateUpdated; }
            set { _dateUpdated = value; }
        }

        /// <summary>
        /// Represents the date that a content page last had a comment added.
        /// </summary>
        public DateTime DateCommented
        {
            get { return _dateCommented; }
            set { _dateCommented = value; }
        }

        /// <summary>
        /// Represents the section that contains this content page.
        /// </summary>
        public int SectionID
        {
            get { return _sectionID; }
            set { _sectionID = value; }
        }

        /// <summary>
        /// Represents the username of the author of this content page.
        /// </summary>
        public string Author
        {
            get { return _author; }
            set { _author = value; }
        }

        /// <summary>
        /// Represents the IP address of the user.
        /// </summary>
        public string IPAddress
        {
            get { return _ipaddress; }
            set { _ipaddress = value; }
        }

        /// <summary>
        /// Represents the number of times the content page
        /// has been requested.
        /// </summary>
        public int ViewCount
        {
            get { return _viewCount; }
            set { _viewCount = value; }
        }

        /// <summary>
        /// Represents the number of comments that have been added to
        /// the content page.
        /// </summary>
        public int CommentCount
        {
            get { return _commentCount; }
            set { _commentCount = value; }
        }

        /// <summary>
        /// Represents the number of aggregated views of the content page.
        /// </summary>
        public int AggregatedViewCount
        {
            get { return _aggregatedViewCount; }
            set { _aggregatedViewCount = value; }
        }

        /// <summary>
        /// True, if the user has previously viewed content.  
        /// </summary>
        public bool HasRead
        {
            get { return _hasRead; }
            set { _hasRead = value; }
        }

        /// <summary>
        /// Enables the content page to be locked.
        /// </summary>
        public bool IsLocked
        {
            get { return _isLocked; }
            set { _isLocked = value; }
        }

        /// <summary>
        /// True, if the content page has been indexed, false if it has been
        /// changed and not re-indexed yet.
        /// </summary>
        public bool IsIndexed
        {
            get { return _isIndexed; }
        }

        /// <summary>
        /// The name of the name remote author of this content. 
        /// </summary>
        public string RemoteAuthor
        {
            get { return _remoteAuthor; }
            set { _remoteAuthor = value; }
        }

        /// <summary>
        /// Represents the IP address of the remote user.
        /// </summary>
        public string RemoteIPAddress
        {
            get { return _remoteipaddress; }
            set { _remoteipaddress = value; }
        }

        /// <summary>
        /// Gets the categories that are referenced by this content.
        /// </summary>
        public Dictionary<int, Category> Categories
        {
            get { return categories; }
        }
    }
}
