// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.Profile;
using System.Web.Security;
using System.Collections.Generic;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Managers;
using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework
{
    /// <summary>
    /// Represents all the information about a user that
    /// can be retrieved from a cookie. It represents the username
    /// and the user roles.
    /// </summary>
    public sealed class CommunityUser
    {
        bool _mayView = false;
        bool _mayAdd = false;
        bool _mayEdit = false;
        bool _mayDelete = false;
        bool _mayComment = false;
        bool _mayModerate = false;
        bool _mayRate = false;

        string[] roleList = null;
        string userRoles = string.Empty;

        LocalUser localUser;
        MembershipUser user;
        CommunityProfile profile;

        /// <summary>
        /// Indicates whether the user is anonymous or authenticated. 
        /// </summary>
        public bool IsAuthenticated
        {
            get { return CommunityContext.Request.IsAuthenticated; }
        }

        /// <summary>
        /// Indicates whether the current user is a member of the 
        /// Administrators role. 
        /// </summary>
        public bool IsAdministrator
        {
            get { return Roles.IsUserInRole(CommunityRole.Administrators.ToString()); }
        }

        /// <summary>
        /// This property is calculated relative to the current section.
        /// When true, user can view pages in this section. 
        /// </summary>
        public bool MayView
        {
            get { return _mayView; }
        }

        /// <summary>
        /// This property is calculated relative to the current section.
        /// When true, user can add pages to this section. 
        /// </summary>
        public bool MayAdd
        {
            get { return _mayAdd; }
        }

        /// <summary>
        /// This property is calculated relative to the current section.
        /// When true, user can edit pages in this section. 
        /// </summary>
        public bool MayEdit
        {
            get { return _mayEdit; }
        }

        /// <summary>
        /// This property is calculated relative to the current section.
        /// When true, user can delete pages from this section. 
        /// </summary>
        public bool MayDelete
        {
            get { return _mayDelete; }
        }

        /// <summary>
        /// This property is calculated relative to the current section.
        /// When true, user can comment on pages in this section. 
        /// </summary>
        public bool MayComment
        {
            get { return _mayComment; }
        }

        /// <summary>
        /// This property is calculated relative to the current section.
        /// When true, user can moderate pages in this section. 
        /// </summary>
        public bool MayModerate
        {
            get { return _mayModerate; }
        }

        /// <summary>
        /// This property is calculated relative to the current section.
        /// When true, user can rate pages in this section. 
        /// </summary>
        public bool MayRate
        {
            get { return _mayRate; }
        }

        public string[] RoleList
        {
            get { return roleList; }
        }

        /// <summary>
        /// Returns a list of roles for the currently logged-on user.
        /// </summary>
        /// <remarks>
        /// Role names are separated by a comma.
        /// </remarks>
        public string UserRoles
        {
            get { return userRoles; }
        }

        /// <summary>
        /// Gets the user name from the Membership provider.
        /// </summary>
        public string UserName
        {
            get 
            {
                if (User != null && (!string.IsNullOrEmpty(User.UserName)))
                    return User.UserName;
                return "?";
            }
        }

        /// <summary>
        /// Gets an instance of the <see cref="LocalUser"/> object.
        /// </summary>
        public LocalUser LocalUser
        {
            get { return localUser; }
        }

        /// <summary>
        /// Gets the <see cref="MembershipUser"/> object from the Membership provider.
        /// </summary>
        public MembershipUser User
        {
            get { return user; }
        }

        /// <summary>
        /// Gets an instance of the <see cref="CommunityProfile"/> object.
        /// </summary>
        public CommunityProfile Profile
        {
            get { return profile; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CommunityUser"/> class.
        /// </summary>
        public CommunityUser()
            : base()
        {
            // If we are authenticated, we can get the membership and profile info
            if (IsAuthenticated)
            {
                // Get the current user
                user = Membership.GetUser();

                // Get the users roles
                roleList = Roles.GetRolesForUser();

                // Setup the roles string
                foreach (string s in roleList)
                    userRoles += s + ",";
                userRoles.TrimEnd(',');
                
                // Get the local user information
                localUser = FrameworkManager.GetUser(UserName.ToLowerInvariant());

                // Create a new local user if we do not have one already
                if (localUser == null)
                {
                    localUser = new LocalUser(UserName.ToLowerInvariant());
                    localUser.Save();
                }

                // Get the current users profile
                profile = CommunityUser.GetProfile(UserName, IsAuthenticated);
            }

            // Finally, update the last activity of the user
            UpdateActivity();
        }

        /// <summary>
        /// Initializes a new <see cref="CommunityUser"/> object given a section. All
        /// user permissions are calculated relative to a section. 
        /// </summary>
        /// <param name="objSectionInfo">The current section object.</param>
        public CommunityUser(Section objSectionInfo) 
            : this()
        {
            // Determine permissions based on page object
            if (!IsAuthenticated)
            {
                // if anonymous, then user can view if everyone can
                if (objSectionInfo.ViewRoles.IndexOf(CommunityRole.Everyone.ToString()) != -1)
                    _mayView = true;
                // if anonymous, then user can add if everyone can
                if (objSectionInfo.AddRoles.IndexOf(CommunityRole.Everyone.ToString()) != -1)
                    _mayAdd = true;
                // if anonymous, then user can edit if everyone can
                if (objSectionInfo.EditRoles.IndexOf(CommunityRole.Everyone.ToString()) != -1)
                    _mayEdit = true;
                // if anonymous, then user can delete if everyone can
                if (objSectionInfo.DeleteRoles.IndexOf(CommunityRole.Everyone.ToString()) != -1)
                    _mayDelete = true;
                // if anonymous, then user can comment if everyone can
                if (objSectionInfo.CommentRoles.IndexOf(CommunityRole.Everyone.ToString()) != -1)
                    _mayComment = true;
            }
            else
            {
                // Check View Permissions
                _mayView = CheckAuthenticatedPermission(objSectionInfo.ViewRoles);

                // Check Add Permissions
                _mayAdd = CheckAuthenticatedPermission(objSectionInfo.AddRoles);

                // Check Edit Permissions
                _mayEdit = CheckAuthenticatedPermission(objSectionInfo.EditRoles);

                // Check Delete Permissions
                _mayDelete = CheckAuthenticatedPermission(objSectionInfo.DeleteRoles);

                // Check Comment Permissions
                _mayComment = CheckAuthenticatedPermission(objSectionInfo.CommentRoles);

                // Check Moderate Permissions
                _mayModerate = CheckAuthenticatedPermission(objSectionInfo.ModerateRoles);

                // Check Rate Permissions
                _mayRate = CheckAuthenticatedPermission(objSectionInfo.RateRoles);
            }
        }

        /// <summary>
        /// Checks whether a user is in a particular role. 
        /// </summary>
        /// <param name="roleName">The role name to check.</param>
        /// <returns>True, if the user is in the role.</returns>
        public bool IsInRole(string roleName)
        {
            if (roleName == CommunityRole.Everyone.ToString())
                return true;
            if (roleName == CommunityRole.Authenticated.ToString() && IsAuthenticated)
                return true;
            if (!IsAuthenticated)
                return false;
            return (Roles.IsUserInRole(this.UserName, roleName));
        }

        /// <summary>
        /// Checks if the user has any roles from a list.
        /// </summary>
        /// <param name="roles">The <see cref="List<string>"/> of roles to check.</param>
        /// <returns><c>true</c> if the user has any of the roles in the list; otherwise <c>false</c>.</returns>
        public bool HasRole(List<string> roles)
        {
            // Check each role
            foreach (string s in roles)
                if (IsInRole(s))
                    return true;
            // No roles available for this user
            return false;
        }

        /// <summary>
        /// Updates the last activity timestamp of the user, if authenticated.
        /// </summary>
        public void UpdateActivity()
        {
            // Check that we have a user object before updating it            
            if (User != null)
            {
                User.LastActivityDate = DateTime.Now;
                Membership.UpdateUser(User);
            }
        }

        /// <summary>
        /// Checks whether a user is in the necessary role. 
        /// </summary>
        /// <param name="pageRoles">The list of roles for the page.</param>
        /// <returns>True, if the user has permission.</returns>
        private bool CheckAuthenticatedPermission(List<string> pageRoles)
        {
            // If Everyone has permission, return true        
            if (pageRoles.IndexOf(CommunityRole.Everyone.ToString()) != -1)
                return true;

            // If Authenticated has permission, return true
            if (pageRoles.IndexOf(CommunityRole.Authenticated.ToString()) != -1)
                return true;

            // Otherwise, check each role
            foreach (string role in pageRoles)
                if (Roles.IsUserInRole(role))
                    return true;

            // Nope, doesn't have permissions
            return false;
        }

        #region Static Membership Helper Methods
        /// <summary>
        /// Returns a list of all the roles from the role provider.
        /// </summary>
        public static string[] CommunityRoles
        {
            get { return Roles.GetAllRoles(); }
        }

        /// <summary>
        /// Returns a list of all the roles for this community.
        /// </summary>
        public static string[] AllRoles
        {
            get 
            { 
                // Get all the roles from the role provider
                List<string> roles = new List<string>(Roles.GetAllRoles());

                // Add the built-in roles to the list
                roles.Add(CommunityRole.Everyone.ToString());
                roles.Add(CommunityRole.Authenticated.ToString());

                return roles.ToArray();
            }
        }

        /// <summary>
        /// Returns a list of roles for a specific user.
        /// </summary>
        /// <param name="userName">The name of the user.</param>
        /// <returns>The users roles.</returns>
        public static string[] GetUserRoles(string userName)
        {
            return Roles.GetRolesForUser(userName);
        }

        /// <summary>
        /// Logs the user into the site using <see cref="FormsAuthentication" />
        /// </summary>
        /// <param name="userName">The user name to sign in.</param>
        /// <param name="password">The users password.</param>
        /// <param name="rememberUser">true if the user is to be remembered between browser restarts.</param>
        /// <returns>false if the user is not authenticated.</returns>
        public static bool Login(string userName, string password, bool rememberUser)
        {
            if (Membership.ValidateUser(userName, password))
                FormsAuthentication.RedirectFromLoginPage(userName, rememberUser);
            else
                return false;
            // We return true here but we have already redirected.
            return true;
        }

        /// <summary>
        /// Signs the user out of the site using <see cref="FormsAuthentication"/>.
        /// </summary>
        public static void Logout()
        {
            FormsAuthentication.SignOut();
        }

        public static bool SendPasswordReminder(string email)
        {
            string userName = Membership.GetUserNameByEmail(email);

            if (!string.IsNullOrEmpty(userName))
            {
                //// Get the reminder message
                //MessageInfo reminderEmail = MessageUtility.GetMessage("Password Reminder");

                //// Send formatted email
                //EmailUtility.SendFormattedEmail
                //(
                //    String.Format("reminder@{0}", CommunityGlobals.PrimaryDomain),
                //    reminderEmail,
                //    _CommunityProfileInfo,
                //    CommunityGlobals.SmtpServer
                //);

                return true;
            }

            return false;
        }

        /// <summary>
        /// Registers a new user with the <see cref="Membership"/> data store.
        /// </summary>
        /// <param name="userName">The user name.</param>
        /// <param name="password">The users password.</param>
        /// <param name="email">The users e-mail address.</param>
        /// <param name="question">The users security question.</param>
        /// <param name="answer">The users security answer.</param>
        /// <returns>
        /// -1 if registration fails, 1 if the user already exists, 2 if the 
        /// e-mail address is already registered and 0 if sucessful.
        /// </returns>
        public static int RegisterUser(string userName, string password, string email, string question, string answer)
        {
            MembershipCreateStatus status = new MembershipCreateStatus();

            Membership.CreateUser(
                            userName, 
                            password, 
                            email, 
                            question, 
                            answer, 
                            CommunityContext.CurrentCommunitySettings.RequireNewUserApproval,
                            null, 
                            out status);

            if (status == MembershipCreateStatus.DuplicateUserName)
                return 1;
            else if (status == MembershipCreateStatus.DuplicateEmail)
                return 2;
            else if (status == MembershipCreateStatus.Success)
                return 0;

            // If we get here then registration failed
            return -1;
        }

        /// <summary>
        /// Returns the <see cref="MembershipUser"/> object for a specific user.
        /// </summary>
        /// <param name="userName">The user name.</param>
        /// <returns>The <see cref="MembershipUser"/> object for the requested user name.</returns>
        public static MembershipUser GetUser(string userName)
        {
            return GetUser(userName, false);
        }

        /// <summary>
        /// Returns the <see cref="MembershipUser"/> object for a specific user.
        /// </summary>
        /// <param name="userName">The user name.</param>
        /// <param name="userIsOnline"><c>true</c>, if the activity should be updated for this user.</param>
        /// <returns>The <see cref="MembershipUser"/> object for the requested user name.</returns>
        public static MembershipUser GetUser(string userName, bool userIsOnline)
        {
            return Membership.GetUser(userName, userIsOnline);
        }

        /// <summary>
        /// Returns the <see cref="CommunityProfile"/> object for a specific user.
        /// </summary>
        /// <param name="userName">The user name.</param>
        /// <returns>The <see cref="CommunityProfile"/> object for the requested user name.</returns>
        public static CommunityProfile GetProfile(string userName)
        {
            return GetProfile(userName, true);
        }

        /// <summary>
        /// Returns the <see cref="CommunityProfile"/> object for a specific user.
        /// </summary>
        /// <param name="userName">The user name.</param>
        /// <param name="isAuthenticated"><c>true</c> if the user is authenicated; otherwise <c>false</c>.</param>
        /// <returns>The <see cref="CommunityProfile"/> object for the requested user name.</returns>
        public static CommunityProfile GetProfile(string userName, bool isAuthenticated)
        {
            // Get the current users profile
            CommunityProfile profile = new CommunityProfile();

            // Initialise the profile
            profile.Initialize(userName.ToLowerInvariant(), isAuthenticated);

            // Return the profile instance
            return profile;
        }

        /// <summary>
        /// Changes the specified users password.
        /// </summary>
        /// <param name="userName">The user name.</param>
        /// <param name="oldPassword">The existing password.</param>
        /// <param name="newPassword">The new password.</param>
        /// <returns><c>true</c> if the password has been changed; otherwise <c>false</c>.</returns>
        public static bool UpdatePassword(string userName, string oldPassword, string newPassword)
        {
            // Get the user details
            MembershipUser user = GetUser(userName);

            // Update the password
            if (user != null)
                return user.ChangePassword(oldPassword, newPassword);

            // We cannot change the password
            return false;
        }

        /// <summary>
        /// Changes the specified users security question/answer.
        /// </summary>
        /// <param name="userName">The user name.</param>
        /// <param name="password">The users password.</param>
        /// <param name="question">The new security question.</param>
        /// <param name="answer">The new security answer.</param>
        /// <returns><c>true</c> if the security question/answer has been changed; otherwise <c>false</c>.</returns>
        public static bool UpdatePasswordQuestionAnswer(string userName, string password, string question, string answer)
        {
            // Get the user details
            MembershipUser user = GetUser(userName);

            // Update the password question and answer
            if (user != null)
                return user.ChangePasswordQuestionAndAnswer(password, question, answer);

            // We cannot change the password question and answer
            return false;
        }

        /// <summary>
        /// Updates the specified users primary e-mail address.
        /// </summary>
        /// <param name="userName">The user name.</param>
        /// <param name="email">The new primary e-mail address.</param>
        public static void UpdateEMailAddress(string userName, string email)
        {
            // Get the user details
            MembershipUser user = GetUser(userName);

            // Update the e-mail address if changed
            if (user != null && user.Email != email.Trim())
            {
                user.Email = email.Trim();
                Membership.UpdateUser(user);
            }
        }
        #endregion
    }
}
