// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.IO;
using System.Data;
using System.Web;
using System.Text;
using System.Text.RegularExpressions;
using System.Reflection;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;

using DotCommunities.Framework.Enums;
using DotCommunities.Framework.Managers;
using DotCommunities.Framework.Collections;
using DotCommunities.Framework.Properties;

namespace DotCommunities.Framework.Components
{
    /// <summary>
    /// Contains static references to the current HTTP context information created during a HTTP request.
    /// </summary>
    public sealed class CommunityContext
    {
        #region Build Information
        /// <summary>
        /// Retrieves the version number from this assembly.
        /// </summary>
        public static Version BuildVersion
        {
            get { return Assembly.GetExecutingAssembly().GetName().Version; }
        }

        /// <summary>
        /// Retrieves the build type (debug/production).
        /// </summary>
        public static string BuildType
        {
            get
            {
#if DEBUG
                return Resources.DevelopmentVersionString;
#else
                return Resources.RetailVersionString;
#endif
            }
        }
        #endregion

        #region General Context information from HttpContext
        /// <summary>
        /// Returns the current <see cref="HttpContext" />.
        /// </summary>
        public static HttpContext Context
        {
            get { return HttpContext.Current; }
        }

        /// <summary>
        /// Returns true if the current HTTP context is available.
        /// </summary>
        public static bool IsContextAvailable
        {
            get { return Context != null; }
        }

        /// <summary>
        /// Returns the current HTTP context <see cref="HttpServerUtility" />.
        /// </summary>
        public static HttpServerUtility Server
        {
            get { return Context.Server; }
        }

        /// <summary>
        /// Returns the current HTTP context <see cref="HttpRequest" />.
        /// </summary>
        public static HttpRequest Request
        {
            get 
            {
                if (Context != null)
                    return Context.Request;
                return null;
            }
        }
        /// <summary>
        /// Returns the current HTTP context <see cref="HttpResponse" />.
        /// </summary>
        public static HttpResponse Response
        {
            get
            {
                if (Context != null)
                    return Context.Response;
                return null;
            }
        }

        /// <summary>
        /// Returns true if the current HTTP request uses secure sockets (that is HTTPS).
        /// </summary>
        public static bool IsSecureConnection
        {
            get
            {
                if (Request != null)
                    return Request.IsSecureConnection;
                return false;
            }
        }

        /// <summary>
        /// Gets the current <see cref="Uri"/> of the request.
        /// </summary>
        public static Uri RequestUrl
        {
            get
            {
                if (Request != null && Request.Url != null)
                    return Request.Url;
                return null;
            }
        }

        /// <summary>
        /// Gets the current host name of the request (e.g. www.dotcommunities.org).
        /// </summary>
        public static string Host
        {
            get
            {
                if (Request != null && Request.Url != null)
                    return Request.Url.Host;
                return null;
            }
        }

        /// <summary>
        /// Gets the port number of the request or the default port (80).
        /// </summary>
        public static int Port
        {
            get
            {
                if (Request != null && Request.Url != null)
                    return Request.Url.Port;
                return 80;
            }
        }

        /// <summary>
        /// Gets the current application path of the request.
        /// </summary>
        public static string ApplicationPath
        {
            get
            {
                if (Request != null && Request.ApplicationPath != "/")
                    return Request.ApplicationPath + "/";
                return "/";
            }
        }
        /// <summary>
        /// Gets the IP address of the web server.
        /// </summary>
        public static string ServerIPAddress
        {
            get
            {
                if (Request != null && Request.ServerVariables != null)
                    return Request.ServerVariables[Resources.LocalAddrServerVariable];
                return null;
            }
        }

        /// <summary>
        /// Returns the current logged on user name from the HTTP context.
        /// </summary>
        public static string UserName
        {
            get 
            {
                if (Context != null && Context.User != null && Context.User.Identity != null)
                    return Context.User.Identity.Name;
                return string.Empty;
            }
        }
        #endregion

        #region Community setup information
        /// <summary>
        /// Returns the configured time zone abbreviation for the community.
        /// </summary>
        public static string HostTimeZone
        {
            get 
            {
                if (CurrentCommunitySettings != null)
                    return CurrentCommunitySettings.TimeZone;
                return "GMT"; 
            }
        }

        /// <summary>
        /// Returns the configured time zone offset for the community.
        /// </summary>
        public static double HostTimeZoneOffset
        {
            get
            {
                if (CurrentCommunitySettings != null && (!string.IsNullOrEmpty(CurrentCommunitySettings.TimeZone)))
                    return TimeZoneManager.GetTimeZone(CurrentCommunitySettings.TimeZone).Offset;
                return 0;
            }
        }

        /// <summary>
        /// Returns the configured time zone abbreviation for the current user.
        /// </summary>
        public static string UserTimeZone
        {
            get
            {
                if (CurrentUser.IsAuthenticated && CurrentUser.Profile != null && !string.IsNullOrEmpty(CurrentUser.Profile.TimeZone))
                    return TimeZoneManager.GetTimeZone(CurrentUser.Profile.TimeZone).ID;
                return HostTimeZone;
            }
        }

        /// <summary>
        /// Returns the configured time zone offset for the current user.
        /// </summary>
        public static double UserTimeZoneOffset
        {
            get
            {
                if (CurrentUser.IsAuthenticated && CurrentUser.Profile != null && !string.IsNullOrEmpty(CurrentUser.Profile.TimeZone))
                    return TimeZoneManager.GetTimeZone(CurrentUser.Profile.TimeZone).Offset;
                return HostTimeZoneOffset;
            }
        }

        /// <summary>
        /// Gets the current configured time for the community or the user if authenticated.
        /// </summary>
        public static DateTime HostDateTime
        {
            get
            {
                return DateTime.Now.AddHours(UserTimeZoneOffset);
            }
        }

        /// <summary>
        /// Returns a value indicating if generic handlers are to be used.
        /// </summary>
        public static bool UseGenericHandlers
        {
            get
            {
                bool useGenericHandlers = false;
                if (!string.IsNullOrEmpty(ConfigurationManager.AppSettings[Resources.UseGenericHandlersName]))
                    bool.TryParse(ConfigurationManager.AppSettings[Resources.UseGenericHandlersName], out useGenericHandlers);
                return useGenericHandlers;
            }
        }

        /// <summary>
        /// Returns a value indicating if web parts are to be used.
        /// </summary>
        public static bool UseCommunityPageHandler
        {
            get
            {
                bool useCommunityPageHandler = false;
                if (!string.IsNullOrEmpty(ConfigurationManager.AppSettings[Resources.UseCommunityPageHandlerName]))
                    bool.TryParse(ConfigurationManager.AppSettings[Resources.UseCommunityPageHandlerName], out useCommunityPageHandler);
                return useCommunityPageHandler;
            }
        }

        /// <summary>
        /// Gets or sets the value indicating if web parts are enabled
        /// for this community.
        /// </summary>
        public static bool WebPartsEnabled
        {
            get
            {
                // Check if web parts are enabled in settings
                if (CurrentCommunitySettings != null && CurrentCommunitySettings.WebPartsEnabled)
                    // If they are enabled, check that we have a page handler
                    return UseCommunityPageHandler;
                // Web parts are disabled by the community settings
                return false;
            }
        }

        /// <summary>
        /// Returns the offline page for the community including the application path (e.g. /Web/index.htm).
        /// </summary>
        public static string CommunityOfflinePage
        {
            get
            {
                return ApplicationPath + Resources.CommunityOfflinePageName;
            }
        }

        /// <summary>
        /// Returns the default page for the community including the application path (e.g. /Web/Default.aspx).
        /// </summary>
        public static string CommunityDefaultPage
        {
            get
            {
                return ApplicationPath + Resources.CommunityDefaultPageName;
            }
        }

        /// <summary>
        /// Returns the default page path used to load community pages (e.g. /Web/Handlers/DefaultPage.aspx).
        /// </summary>
        public static string CommunityBasePage
        {
            get
            {
                return HandlersFolderPath + Resources.CommunityBasePageName;
            }
        }

        /// <summary>
        /// Returns the directory path to the communities folder.
        /// </summary>
        public static string CommunitiesFolderPath
        {
            get
            {
                return ApplicationPath + Resources.CommunitiesFolder;
            }
        }

        /// <summary>
        /// Returns the directory path to the services folder.
        /// </summary>
        public static string ServicesFolderPath
        {
            get
            {
                return ApplicationPath + Resources.ServicesFolder;
            }
        }

        /// <summary>
        /// Returns the directory path to the handlers folder.
        /// </summary>
        public static string HandlersFolderPath
        {
            get
            {
                return ApplicationPath + Resources.HandlersFolder;
            }
        }

        /// <summary>
        /// Returns the url to the administration section of the site.
        /// </summary>
        public static string AdministrationUrl
        {
            get 
            {
                return ApplicationPath + Resources.AdministrationFolder;
            }
        }

        /// <summary>
        /// Returns the full path to the App_Data\ directory.
        /// </summary>
        public static string AppDataPath
        {
            get { return AppDomain.CurrentDomain.BaseDirectory + "App_Data\\"; }
        }

        /// <summary>
        /// Returns the full path to the Settings folder in the App_Data directory.
        /// </summary>
        public static string AppSettingsPath
        {
            get { return AppDataPath + "Settings\\"; }
        }

        /// <summary>
        /// Returns the default community settings type name.
        /// </summary>
        public static string DefaultCommunitySettingsType
        {
            get 
            { 
                if (!string.IsNullOrEmpty(ConfigurationManager.AppSettings[Resources.DefaultCommunitySettingsTypeName]))
                    return ConfigurationManager.AppSettings[Resources.DefaultCommunitySettingsTypeName];
                return null;
            }
        }

        /// <summary>
        /// Returns the default community settings XML path.
        /// </summary>
        public static string DefaultCommunitySettingsPath
        {
            get 
            {
                if (!string.IsNullOrEmpty(ConfigurationManager.AppSettings[Resources.DefaultCommunitySettingsFileName]))
                    return AppSettingsPath + ConfigurationManager.AppSettings[Resources.DefaultCommunitySettingsFileName];
                return null;
            }
        }  
        
        /// <summary>
        /// Returns all the enabled sections for this community.
        /// </summary>
        public static SectionCollection EnabledSections
        {
            get { return FrameworkManager.GetAllEnabledSections(); }
        }

        /// <summary>
        /// Returns all the community named pages.
        /// </summary>
        public static GenericPageCollection NamedPages
        {
            get { return ContentTypesManager.GetNamedPages(); }
        }

        /// <summary>
        /// Returns all the community module pages.
        /// </summary>
        public static GenericPageCollection ModulePages
        {
            get { return ContentTypesManager.GetModulePages(); }
        }

        /// <summary>
        /// Returns a generic dictionary of all <see cref="PageTheme"/> objects.
        /// </summary>
        public static Dictionary<string, PageTheme> PageThemes
        {
            get { return ThemeManager.GetThemes(); }
        }

        /// <summary>
        /// Returns a generic dictionary of all <see cref="PageStyle"/> objects.
        /// </summary>
        public static Dictionary<string, PageStyle> PageStyles
        {
            get { return ThemeManager.GetStyles(); }
        }

        /// <summary>
        /// Returns all the generic handlers.
        /// </summary>
        public static GenericHandlerCollection GenericHandlers
        {
            get { return ContentTypesManager.GetGenericHandlers(); }
        }
        #endregion

        #region Community object reference information from the current context
        /// <summary>
        /// Gets or sets the <see cref="Community" /> object reference from the current context.
        /// </summary>
        public static Community CurrentCommunity
        {
            get
            {
                object community = Context.Items[Resources.CommunityContextName];
                if (community != null && community is Community)
                    return community as Community;
                return null;
            }
            set 
            { 
                if (value != null && value is Community)
                    Context.Items[Resources.CommunityContextName] = value; 
            }
        }

        /// <summary>
        /// Gets the <see cref="CommunitySettings" /> object reference from the current context
        /// or attempts to retrieve the object from the <see cref="CommunityContext.CurrentCommunity"/> 
        /// if not found in the current context.
        /// </summary>
        public static CommunitySettings CurrentCommunitySettings
        {
            get
            {
                object communitySettings = Context.Items[Resources.CommunitySettingsContextName];
                if (communitySettings != null)
                    return communitySettings as CommunitySettings;
                else
                {
                    communitySettings = CurrentCommunity.GetCommunitySettings();
                    if (communitySettings != null)
                        Context.Items[Resources.CommunitySettingsContextName] = communitySettings;
                    return (CommunitySettings)communitySettings;
                }
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="Section" /> object reference from the current context.
        /// </summary>
        public static Section CurrentSection
        {
            get
            {
                object section = Context.Items[Resources.SectionContextName];
                if (Context != null && section != null && section is Section)
                    return section as Section;
                return null;
            }
            set 
            {
                if (value != null && value is Section)
                    Context.Items[Resources.SectionContextName] = value; 
            }            
        }

        /// <summary>
        /// Gets or sets the <see cref="GenericPage" /> object reference from the current context.
        /// </summary>
        public static GenericPage CurrentPage
        {
            get
            {
                // See if we have a generic page already stored in the context
                object genericPage = Context.Items[Resources.GenericPageContextName];
                if (Context != null && genericPage != null && genericPage is GenericPage)
                    return genericPage as GenericPage;
                // If the page is not in the context, grab it and attempt to set
                // it in the context
                GenericPage page = GetCommunityPage();
                if (page != null)
                    Context.Items[Resources.GenericPageContextName] = page;
                return page;
            }
            set 
            {
                if (value != null && value is GenericPage)
                    Context.Items[Resources.GenericPageContextName] = value; 
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="CommunityUser" /> object reference from the current context.
        /// </summary>
        public static CommunityUser CurrentUser
        {
            get
            {
                object user = Context.Items[Resources.CommunityUserContextName];
                if (Context != null && user != null && user is CommunityUser)
                    return user as CommunityUser;
                return null;
            }
            set
            {
                if (value != null && value is CommunityUser)
                    Context.Items[Resources.CommunityUserContextName] = value;
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="CommunityContent" /> object reference from the current context.
        /// </summary>
        public static CommunityContent CurrentContent
        {
            get
            {
                object content = Context.Items[Resources.CommunityContentContextName];
                if (Context != null && content != null && content is CommunityContent)
                    return content as CommunityContent;
                return null;
            }
            set
            {
                if (value != null && value is CommunityContent)
                    Context.Items[Resources.CommunityContentContextName] = value;
            }
        }

        /// <summary>
        /// Gets or sets the status for the community using the <see cref="CommunityStatus"/> enum values.
        /// </summary>
        public static CommunityStatus CommunityStatus
        {
            get
            {
                // Get the key name for the status
                string statusKey = string.Format(Resources.CommunityStatusApplicationKey, CurrentCommunity.ID);
                // See if we have a status already stored in the application settings
                object applicationStatus = Context.Application[statusKey];
                if (applicationStatus != null && applicationStatus is CommunityStatus)
                    return (CommunityStatus)applicationStatus;
                // If the status is not in the application state, grab it and attempt to set
                // it in the application state
                CommunityStatus status = FrameworkManager.GetCommunityStatus();
                Context.Application[statusKey] = status;
                // Return the status
                return status;
            }
            set
            {
                // Add the status to the data source
                FrameworkManager.SetCommunityStatus(value);
                // Get the key name for the status
                string statusKey = string.Format(Resources.CommunityStatusApplicationKey, CurrentCommunity.ID);
                // Remove the status from the application state
                Context.Application.Remove(statusKey);
            }
        }
        #endregion

        #region Path Manipulation Methods
        /// <summary>
        /// Redirects the response to the login page.
        /// </summary>
        public static void RedirectToLoginPage()
        {
            string returnUrl = string.Empty;
            if (!string.IsNullOrEmpty(RequestUrl.PathAndQuery))
                returnUrl = string.Format("?ReturnUrl={0}", Server.UrlEncode(RequestUrl.PathAndQuery));
            Response.Redirect(ApplicationPath + Resources.LoginPageName + returnUrl, false);
        }

        /// <summary>
        /// Given a block of text, this method will convert all relative Urls contained
        /// in the text with absolute Urls.  This is useful for text editors that will
        /// be used for creating HTML e-mails for example.
        /// </summary>
        /// <param name="text">A block of text to convert.</param>
        /// <returns>The converted text.</returns>
        public static string ConvertRelativeUrlsToAbsoluteUrls(string text)
        {
            // Get the absolute url
            string host = ResolveAbsoluteUrl("/");
            // Convert the instances of relative urls
            text = Regex.Replace(text, "href=\"/", "href=\"" + host, RegexOptions.IgnoreCase);
            text = Regex.Replace(text, "src=\"/", "src=\"" + host, RegexOptions.IgnoreCase);
            // Return the modified text
            return text;
        }

        /// <summary>
        /// Given a path, this method will extract the page name from it (e.g. Default.aspx).
        /// </summary>
        /// <param name="path">The path to parse (e.g. /Section/Default.aspx).</param>
        /// <returns>The extracted page name.</returns>
        public static string GetPageName(string path)
        {
            string page = null;   

            // Path may already have a page name obscured by a "/"
            if (path.IndexOf('.') > 0)
            {
                string pathTail = path.Substring(path.IndexOf('.'));
                if ((!string.IsNullOrEmpty(pathTail)) && pathTail.Contains("/"))
                    path = path.Substring(0, path.LastIndexOf("/"));
                // Trim the "/" so we are left with the page name
                page = path.Substring(path.LastIndexOf('/')).TrimStart('/');
            }

            // We may have a default page - check for this here
            if (string.IsNullOrEmpty(page) || page == "/" || (!page.Contains(".")))
                return "Default.aspx";
            return page;
        }

        /// <summary>
        /// Returns the page number if the page name is numeric (e.g. 1.aspx).
        /// </summary>
        /// <param name="pageName">The page name to convert.</param>
        /// <returns>The extracted page number.</returns>
        public static int GetPageNumber(string pageName)
        {
            int result = -1;
            string pageNumber = pageName.Substring(0, pageName.IndexOf('.'));
            if (int.TryParse(pageNumber, out result))
                return result;
            return 0;
        }

        /// <summary>
        /// Gets the current page name of the request (e.g. Default.aspx).
        /// </summary>
        public static string CurrentPageName
        {
            get
            {
                if (Request != null)
                    return GetPageName(Request.Path);
                return null;
            }
        }

        /// <summary>
        /// Returns current page number of the request, if the page name is numeric (e.g. 1.aspx).
        /// </summary>
        public static int CurrentPageNumber
        {
            get
            {
                if (!string.IsNullOrEmpty(CurrentPageName))
                    return GetPageNumber(CurrentPageName);
                return 0;
            }
        }

        /// <summary>
        /// Returns <c>true</c> if the current page is a section page (e.g. Default.aspx).
        /// </summary>
        public static bool IsSectionPage
        {
            get
            {
                if (!string.IsNullOrEmpty(CurrentPageName))
                    return CurrentPageName.ToLowerInvariant() == "default.aspx";
                return false;
            }
        }

        /// <summary>
        /// Calculates the complete Url including the host name, then adding the path (e.g. http://www.dotcommunities.org/ + <paramref name="path"/>).
        /// </summary>
        /// <param name="path">The local path to resolve.</param>
        /// <returns>The complete Url.</returns>
        public static string ResolveAbsoluteUrl(string path)
        {
            string host = Host;
            if (Port != 80)
                host = host + string.Format(":{0}", Port);

            if (IsSecureConnection)
                return "https://" + host + path;
            else
                return "http://" + host + path;
        }

        /// <summary>
        /// Calculates the path, given the page name.
        /// </summary>
        /// <param name="pageName">Name of the page.</param>
        /// <returns>The full path to the page.</returns>
        public static string CalculatePath(string pageName)
        {
            string _rawUrl = CurrentSection.Path;
            string basePath = _rawUrl.Remove(_rawUrl.LastIndexOf("/"), _rawUrl.Length - _rawUrl.LastIndexOf("/"));
            return basePath + "/" + pageName;
        }

        /// <summary>
        /// This method returns the relative path of a content page 
        /// (including the section path). This method is used, for example,
        /// by the Categories page to generate the path to a content page.
        /// </summary>
        /// <param name="sectionID">The section ID of the content.</param>
        /// <param name="contentID">The contetnt ID.</param>
        /// <returns>The path to the content page.</returns>
        public static string CalculateContentPath(int sectionID, int contentID)
        {
            string path = FrameworkManager.GetSectionPath(sectionID);
            path = path.Remove(path.LastIndexOf("/"), path.Length - path.LastIndexOf("/"));
            return String.Format("{0}/{1}.aspx", path, contentID);
        }

        /// <summary>
        /// Converts a file into a generic handler file if generic handlers
        /// are enabled.
        /// </summary>
        /// <remarks>
        /// The file name is Url encoded to prevent confusion with parsing. Ensure
        /// that the file name is Url decoded when retrieved for processing.
        /// </remarks>
        /// <param name="fileName">The filename to convert (e.g. Image.jpg).</param>
        /// <returns>The converted file (e.g. Default.ashx?file=Image.jpg).</returns>
        public static string CalculateGenericHandlersPath(string fileName)
        {
            if (UseGenericHandlers)
                return string.Format("Default.ashx?file={0}", Server.UrlEncode(fileName));
            return fileName;
        }

        /// <summary>
        /// Gets the <see cref="GenericHandler"/> for this page if it is assigned to
        /// the supplied extension name.
        /// </summary>
        /// <param name="ext">The extension name for the handler.</param>
        public static GenericHandler GetCurrentHandler(string ext)
        {
            if (GenericHandlers != null)
                foreach (string extensionName in GenericHandlers.ExtensionNames)
                    if (ext.ToLowerInvariant() == extensionName.ToLowerInvariant())
                        foreach (GenericHandler handler in GenericHandlers.Values)
                            foreach (string s in handler.ExtensionNames)
                                if (s.ToLowerInvariant() == ext.ToLowerInvariant())
                                    return handler;
                return null;
        }


        /// <summary>
        /// Given a path, this method will extract the section path from it (e.g. /Section/).
        /// </summary>
        /// <param name="path">The path to parse (e.g. /Section/Default.aspx).</param>
        /// <returns>The extracted section path.</returns>
        public static string GetSectionPath(string path)
        {
            // Path may already have a page name obscured by a "/"
            if (path.Contains(CommunityContext.CurrentPageName))
                path = path.Substring(0, path.IndexOf(CommunityContext.CurrentPageName) + CommunityContext.CurrentPageName.Length);
            // Check that we do not have a slash missing from the end of the url
            if (!(path.EndsWith("/") || path.Contains(".")))
                path = path + "/";
            // Get the index of the last slash
            int pathIndex = path.LastIndexOf('/');
            // Check if we have anything to trim from the end of the path (such as page name)
            if (pathIndex < path.Length)
                return path.Remove(pathIndex + 1, path.Length - (pathIndex + 1));
            // Path is already a section path
            return path;
        }

        /// <summary>
        /// Returns the path for a section with a particular ID. 
        /// </summary>
        /// <param name="sectionID">Section ID to return</param>
        /// <returns>The section path.</returns>
        public static string GetSectionPath(int sectionID)
        {
            Section objSectionInfo = EnabledSections.GetSectionByID(sectionID);
            string lsSectionPath = null;
            if (objSectionInfo != null)
                lsSectionPath = objSectionInfo.Path;
            return lsSectionPath;
        }

        /// <summary>
        /// Gets the current section path from the request.
        /// </summary>
        public static string CurrentSectionPath
        {
            get
            {
                if (Request != null)
                    return GetSectionPath(Request.Path);
                return null;
            }
        }

        /// <summary>
        /// Builds a URL to a section image given the image ID and image name. 
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="fileName">Name of the file.</param>
        /// <returns></returns>
        public static string BuildImagePath(int id, string fileName)
        {
            string extension = Path.GetExtension(fileName).ToLower();
            return id.ToString() + extension;
        }

        /// <summary>
        /// Builds a URL to a section image that includes the section path.
        /// </summary>
        /// <param name="sectionID">The section ID.</param>
        /// <param name="imageID">The image ID.</param>
        /// <param name="fileName">Name of the file.</param>
        /// <returns>The full image path.</returns>
        public static string BuildFullImagePath(int sectionID, int imageID, string fileName)
        {
            string path = GetSectionPath(sectionID);
            path = path.Remove(path.LastIndexOf("/"), path.Length - path.LastIndexOf("/"));
            string extension = Path.GetExtension(fileName).ToLower();
            return String.Format("{0}/{1}{2}", path, imageID, extension);
        }

        /// <summary>
        /// Checks that a path is part of a root path.
        /// </summary>
        /// <param name="path">The path to check.</param>
        /// <returns><c>true</c> if the path is part of the root path.</returns>
        public static bool IsRootPath(string path)
        {
            return IsInPath(ApplicationPath.ToLowerInvariant() + path.ToLowerInvariant());
        }

        /// <summary>
        /// Checks that a path is part of the request path.
        /// </summary>
        /// <param name="path">The path to check.</param>
        /// <returns><c>true</c> if the path is part of the request path.</returns>
        public static bool IsInPath(string path)
        {
            if (Request != null)
                return Request.Path.ToLowerInvariant().Contains(path.ToLowerInvariant());
            return false;
        }

        /// <summary>
        /// Checks that the current path is not the installer path.
        /// </summary>
        public static bool IsInstallPath
        {
            get { return IsRootPath("install/"); }
        }

        /// <summary>
        /// Checks that the current path is not the services path.
        /// </summary>
        public static bool IsServicesPath
        {
            get { return IsRootPath("services/"); }
        }

        /// <summary>
        /// Checks that the current path is not the administration path.
        /// </summary>
        public static bool IsAdministrationPath
        {
            get { return IsRootPath("admin/"); }
        }

        /// <summary>
        /// Checks that the current path is not the communities path.
        /// </summary>
        public static bool IsCommunitiesPath
        {
            get { return IsRootPath("communities/"); }
        }

        /// <summary>
        /// Checks that the current path is not the handlers path.
        /// </summary>
        public static bool IsHandlersPath
        {
            get { return IsRootPath("handlers/"); }
        }

        /// <summary>
        /// Checks that the current page is not the login page.
        /// </summary>
        public static bool IsLoginPage
        {
            get { return IsInPath(Resources.LoginPageName); }
        }

        #endregion

        #region Text Manipulation Methods
        /// <summary>
        /// Transforms a string depending on the <see cref="AllowHtml"/> options.
        /// </summary>
        /// <param name="allowHtml">The <see cref="AllowHtml"/> option.</param>
        /// <param name="sectionInfo">The section to apply transformations with.</param>
        /// <param name="text">The text to format.</param>
        /// <returns>The formatted text.</returns>
        public static string FormatText(AllowHtml allowHtml, Section sectionInfo, string text)
        {
            // TODO Perform regex operations
            return text;
        }

        /// <summary>
        /// Returns a string of whole words, truncated to the required length plus the ellipsis ("...").
        /// </summary>
        /// <param name="text">The text to truncate.</param>
        /// <param name="length">The length to truncate the string to.</param>
        /// <returns>The truncated string of whole words plus the ellipsis.</returns>
        public static string TruncateWithEllipsis(string text, int length)
        {
            if (text.Length > length)
                return string.Format("{0}...", Truncate(text, length - 3));
            // We do not need to truncate if not required
            return text;
        }

        /// <summary>
        /// Returns a string of whole words, truncated to the required length.
        /// </summary>
        /// <param name="text">The text to truncate.</param>
        /// <param name="length">The length to truncate the string to.</param>
        /// <returns>The truncated string of whole words.</returns>
        public static string Truncate(string text, int length)
        {
            string s = string.Empty;
            // Check that the first word is not longer than the length
            int index = text.IndexOf(' ');
            if (index > length)
                s = text.Substring(0, length);
            else
            {
                // Loop through each word until we reach the length
                foreach (Match word in Regex.Matches(text, @"\w+"))
                    if (s.Length + word.Length < length)
                        s += string.Format("{0} ", word.Value);
                    else
                        break;
            }
            // Return the truncated string, trimming the remaining spaces
            return s.Trim();
        }

        /// <summary>
        /// Parses the meta keys from the supplied text.
        /// </summary>
        /// <param name="metaData">The text to extract meta keys from.</param>
        /// <returns>The formatted meta keys string.</returns>
        public static string ParseMetaKeys(string metaData)
        {
            return ParseMetaData(HttpUtility.HtmlEncode(metaData), 250, ',', true);
        }

        /// <summary>
        /// Parses the meta description from the supplied text.
        /// </summary>
        /// <param name="metaData">The text to extract meta description from.</param>
        /// <returns>The formatted meta description string.</returns>
        public static string ParseMetaDescription(string metaData)
        {
            return ParseMetaData(HttpUtility.HtmlEncode(metaData), 250, ' ', false);
        }

        /// <summary>
        /// Parses the meta data from the supplied text.
        /// </summary>
        /// <param name="metaData">The text to extract meta data from.</param>
        /// <param name="length">The required length of the meta data.</param>
        /// <param name="seperator">The text to place between words in the meta data.</param>
        /// <param name="uniqueOnly">If <c>true</c>, will only return unique words; Otherwise, all words are rerturned.</param>
        /// <returns>The formatted meta data string.</returns>
        public static string ParseMetaData(string metaData, int length, char seperator, bool uniqueOnly)
        {
            string metaKeys = string.Empty;
            // Create a list of added words to check for uniqueness
            List<string> words = new List<string>();
            // Loop through all words in the meta data
            foreach (Match word in Regex.Matches(metaData, @"\w+"))
                if (uniqueOnly && (words.Contains(word.Value) || IsNoiseWord(word.Value)))
                    continue; // word is invalid
                else if (metaKeys.Length < length)
                {
                    // Add the word
                    metaKeys += string.Format("{0}{1}", word.Value, seperator);
                    if (uniqueOnly)
                        words.Add(word.Value);
                }
                else if (metaKeys.Length > length)
                {
                    // Remove the last word
                    int index = metaKeys.LastIndexOf(seperator);
                    if (index > 0 && index > metaKeys.Length)
                        metaKeys = metaKeys.Substring(0, index);
                    break;
                }
                else // Same length as max length
                    break;
            // Return the formatted string with the last separator removed
            return metaKeys.TrimEnd(seperator);
        }
        #endregion

        #region Content Retrieval Methods
        /// <summary>
        /// Retrieves the <see cref="GenericPage"/> associated with the page being requested.
        /// </summary>
        /// <returns>A <see cref="GenericPage"/> containing page information.</returns>
        private static GenericPage GetCommunityPage()
        {
            GenericPage page = null;

            // Get the name of the requested page
            string requestFile = CurrentPageName.ToLowerInvariant();

            // Check for section default page
            if ((!string.IsNullOrEmpty(requestFile)) && requestFile != "default.aspx")
            {
                // Check for named page
                if (NamedPages.ContainsKey(requestFile))
                {
                    page = NamedPages[requestFile];
                    if (page != null)
                        return page;
                }

                // Check for content page
                page = FrameworkManager.GetContentPageFromPageName(requestFile);
                if (page != null)
                {
                    // Get the content type info with the details from the web configuration file (if defined)
                    if (ModulePages.ContainsKey(page.Name))
                        page.ContentType = ModulePages[page.Name].ContentType;
                }
            }
            else if (CurrentSection != null)
            {
                // Get section module information
                string moduleName = CurrentSection.PageTypeName;

                if (ModulePages.ContainsKey(moduleName))
                {
                    GenericPage modulePage = ModulePages[moduleName];

                    // Configure the page if module exists
                    if (modulePage != null)
                        page = new GenericPage(-1,
                            CurrentSection.ParentID,
                            modulePage.PageType,
                            modulePage.ContentType,
                            modulePage.Name,
                            CurrentSection.Title,
                            CurrentSection.Description,
                            CurrentSection.PageMetaDescription,
                            CurrentSection.PageMetaKeys
                        );
                }
            }

            return page;
        }

        /// <summary>
        /// Checks the path for a system section because system sections configure
        /// and handle their own path information.
        /// </summary>
        /// <param name="currentSectionPath">The path to check for a system section.</param>
        /// <returns>A <see cref="T:Section"/> object containing system section data if found; 
        /// otherwise <c>null</c>.</returns>
        public static Section GetSystemSection(string currentSectionPath)
        {
            string path = currentSectionPath;
            // Walk up the path and check if there is a parent
            while ((!string.IsNullOrEmpty(path)) &&
                   (!CommunityContext.EnabledSections.ContainsKey(path)) &&
                   path.LastIndexOf('/') > 0)
            {
                // Remove /default.aspx from the string
                path = path.Substring(0, path.LastIndexOf('/'));
                // Get the index of the last slash
                int pathIndex = path.LastIndexOf('/');
                // Set the path to the parent section
                path = path.Remove(pathIndex + 1, path.Length - (pathIndex + 1)) + "default.aspx";
                // Check that the section exists
                if (CommunityContext.EnabledSections.ContainsKey(path) && CommunityContext.EnabledSections[path].IsSystem)
                    return CommunityContext.EnabledSections[path];
            }
            // Return null if we cannot find a system section
            return null;
        }

        /// <summary>
        /// Checks if a word exists in the list of noise words in the configuration file.
        /// </summary>
        /// <param name="word">The word to check.</param>
        /// <returns>Returns <c>true</c> if the word is a noise word; otherwise <c>false</c>.</returns>
        public static bool IsNoiseWord(string word)
        {
            // TODO Get noise words from the web.config file
            return false;
        }
        #endregion
    }
}
