// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

// Uncomment this line or add the definition to the build options to disable the cache (not recommended)
// #define CACHE_DISABLED

// Uncomment this line or add the definition to the build options to use web caching only
// #define USE_WEB_CACHE

using System;
using System.Web;
using System.Collections;
using System.Text.RegularExpressions;

using DotCommunities.Framework.Managers;
using DotCommunities.Framework.Enums;

using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;

namespace DotCommunities.Framework.Components
{
    /// <summary>
    /// Abstract base class that is inherited from all managers that need the ability
    /// to cache their data. As a rule, data that is to be cached for a short time (60 seconds
    /// or less) should be added to the web cache; data that is added for longer and
    /// is serializable should be added to the Enterprise Library cache.
    /// </summary>
    /// <remarks>
    /// If you want to cache items which you will later need to remove with a regular expression
    /// pattern, you can only do this with the web cache as the Enterprise Library cache does not
    /// allow you to enummerate through the cached items.
    /// </remarks>
    public abstract class CommunityManagerBase
    {
        private static System.Web.Caching.Cache webCache;
        private static CacheManager cache;

        /// <summary>
        /// Gets a value indicating if caching is disabled (set with CACHE_DISABLED build flag).
        /// </summary>
        private static bool CachingDisabled
        {
            get
            {
#if CACHE_DISABLED
                return true;
#else
                return false;
#endif
            }
        }

        /// <summary>
        /// Gets a value indicating if caching is only done via the web cache and not through 
        /// Enterprise Library Caching Application Block (set with USE_WEB_CACHE build flag).
        /// </summary>
        public static bool UseWebCache
        {
            get
            {
#if USE_WEB_CACHE
                return true;
#else
                return false;
#endif
            }
        }

        /// <summary>
        /// Static constructor that loads the providers and creates the cache manager.
        /// </summary>
        static CommunityManagerBase()
        {
            // Create the cache
            if (!CachingDisabled)
            {
                webCache = HttpContext.Current.Cache;
                if (!UseWebCache)
                    cache = CacheFactory.GetCacheManager();
            }
        }

        /// <summary>
        /// Returns the number of items accross all caches.
        /// </summary>
        public static int CacheCount
        {
            get 
            {
                if (!CachingDisabled)
                {
                    if (cache != null)
                        return cache.Count + webCache.Count;
                    else
                        return webCache.Count;
                }
                return 0;
            }
        }

        /// <summary>
        /// Purges the cache of all stored data.
        /// </summary>
        public static void FlushCache()
        {
            if (!CachingDisabled)
            {
                // Clear the caching block
                if (cache != null)
                    cache.Flush();
                // Clear the web caching data
                foreach (DictionaryEntry cacheItem in webCache)
                    webCache.Remove(cacheItem.Key.ToString());
            }
        }

        /// <summary>
        /// Retrieves an object from the cache.
        /// </summary>
        /// <param name="cacheKey">The key name of the object.</param>
        /// <returns>The cached object if found, otherwise null.</returns>
        public static object GetCachedItem(string cacheKey)
        {
            if (!CachingDisabled)
            {
                object item = null;
                // Check if the item is in the cache application block
                if (cache != null)
                    item = cache.GetData(cacheKey);
                // If not in the application block then it might be in the web cache
                if (item == null)
                    item = webCache.Get(cacheKey);
                // Return the cached item
                return item;
            }

            // Caching disabled - return null
            return null;
        }

        /// <summary>
        /// Adds an object to the cache for the lifetime of the application.
        /// </summary>
        /// <param name="cacheKey">The key name of the object.</param>
        /// <param name="item">The object to cache.</param>
        /// <returns>Always returns the cached object, even if not cached.</returns>
        public static object AddCachedItemForLifetime(string cacheKey, object item)
        {
            // Check if caching is enabled (if not we just return the item)
            if (CachingDisabled)
                return item;

            // Check that the item to cache is not null
            if (item != null)
            {
                LogManager.WriteDebug(EventCode.Cache, string.Format("Adding object to cache {0}", cacheKey), string.Format("Adding object of type: {0} for lifetime", item.GetType().AssemblyQualifiedName));
                // We only add to the web cache if we are using it
                if (UseWebCache)
                    webCache.Add(cacheKey, item, null, DateTime.MaxValue, TimeSpan.Zero, System.Web.Caching.CacheItemPriority.NotRemovable, null);
                else if (cache != null)
                    cache.Add(cacheKey, item, CacheItemPriority.NotRemovable, null, null);
            }

            // We always return the item if it was cached or not
            return item;
        }

        /// <summary>
        /// Adds an object to the cache for upto 24 hours.
        /// </summary>
        /// <param name="cacheKey">The key name of the object.</param>
        /// <param name="item">The object ot cache.</param>
        /// <returns>Always returns the cached object, even if not cached.</returns>
        protected static object AddCachedItemFor24Hrs(string cacheKey, object item)
        {
            return AddCachedItem(cacheKey, item, 60 * 24, CacheItemPriority.High, false);
        }

        /// <summary>
        /// Adds an object to the web cache for upto 60 seconds.
        /// </summary>
        /// <param name="cacheKey">The key name of the object.</param>
        /// <param name="item">The object ot cache.</param>
        /// <returns>Always returns the cached object, even if not cached.</returns>
        protected static object AddCachedItemFor60Seconds(string cacheKey, object item)
        {
            return AddCachedItem(cacheKey, item, 1, CacheItemPriority.High, true);
        }

        /// <summary>
        /// Adds an object to the cache for upto 30 minutes (default).
        /// </summary>
        /// <param name="cacheKey">The key name of the object.</param>
        /// <param name="item">The object ot cache.</param>
        /// <returns>Always returns the cached object, even if not cached.</returns>
        protected static object AddCachedItem(string cacheKey, object item)
        {
            return AddCachedItem(cacheKey, item, 30, CacheItemPriority.Normal, false);
        }

        /// <summary>
        /// Adds an object to the cache for upto 30 minutes (default).
        /// </summary>
        /// <param name="cacheKey">The key name of the object.</param>
        /// <param name="item">The object ot cache.</param>
        /// <param name="cacheInContext">If <c>true</c>, the object will be cached to the web based cache rather than
        /// the Enterprise Library Caching Block.</param>
        /// <returns>Always returns the cached object, even if not cached.</returns>
        protected static object AddCachedItem(string cacheKey, object item, bool cacheInContext)
        {
            return AddCachedItem(cacheKey, item, 30, CacheItemPriority.Normal, cacheInContext);
        }

        /// <summary>
        /// Adds an object to the cache for a specified number of minutes.
        /// </summary>
        /// <param name="cacheKey">The key name of the object.</param>
        /// <param name="item">The object ot cache.</param>
        /// <param name="timeout">The number of minutes the item should be cached for.</param>
        /// <param name="cacheInContext">If <c>true</c>, the object will be cached to the web based cache rather than
        /// the Enterprise Library Caching Block.</param>
        /// <returns>Always returns the cached object, even if not cached.</returns>
        protected static object AddCachedItem(string cacheKey, object item, int timeout, bool cacheInContext)
        {
            return AddCachedItem(cacheKey, item, timeout, CacheItemPriority.Normal, cacheInContext);
        }

        /// <summary>
        /// Adds an object to the cache for a specified number of minutes and a specified <see cref="CacheItemPriority"/>.
        /// </summary>
        /// <param name="cacheKey">The key name of the object.</param>
        /// <param name="item">The object ot cache.</param>
        /// <param name="timeout">The number of minutes the item should be cached for.</param>
        /// <param name="priority">The <see cref="CacheItemPriority"/> of the item.</param>
        /// <param name="cacheInContext">If <c>true</c>, the object will be cached to the web based cache rather than
        /// the Enterprise Library Caching Block.</param>
        /// <returns>Always returns the cached object, even if not cached.</returns>
        protected static object AddCachedItem(string cacheKey, object item, int timeout, CacheItemPriority priority, bool cacheInContext)
        {
            // Check if caching is enabled (if not we just return the item)
            if (CachingDisabled)
                return item;

            // Check that the item to cache is not null
            if (item != null)
            {
                LogManager.WriteDebug(EventCode.Cache, string.Format("Adding object to cache {0}", cacheKey), string.Format("Adding object of type: {0} with timeout of {1}", item.GetType().AssemblyQualifiedName, timeout));
                // We only add to the web cache if we are using it or have asked for the data to be cahced there specifically
                if (cacheInContext || UseWebCache)
                    webCache.Add(cacheKey, item, null, DateTime.Now.AddMinutes(timeout), TimeSpan.Zero, ConvertPriorityToWebCachingPriority(priority), null);
                else if (cache != null)
                    cache.Add(cacheKey, item, priority, null, new AbsoluteTime(DateTime.Now.AddMinutes(timeout)));
            }

            // We always return the item if it was cached or not
            return item;
        }

        /// <summary>
        /// Given an Enterprise Library <see cref="CacheItemPriority"/>, converts it to a <see cref="System.Web.Caching.CacheItemPriority"/>.
        /// </summary>
        /// <param name="priority">The <see cref="CacheItemPriority"/> to convert.</param>
        /// <returns>The equivalent <see cref="System.Web.Caching.CacheItemPriority"/>.</returns>
        private static System.Web.Caching.CacheItemPriority ConvertPriorityToWebCachingPriority(CacheItemPriority priority)
        {
            // Switch between supplied priority
            switch (priority)
            {
                case CacheItemPriority.High:
                    return System.Web.Caching.CacheItemPriority.High;
                case CacheItemPriority.Low:
                    return System.Web.Caching.CacheItemPriority.Low;
                case CacheItemPriority.Normal:
                    return System.Web.Caching.CacheItemPriority.Normal;
                case CacheItemPriority.NotRemovable:
                    return System.Web.Caching.CacheItemPriority.NotRemovable;
                default:
                    return System.Web.Caching.CacheItemPriority.Default;
            }
        }

        /// <summary>
        /// Removes a specific item from both caches.
        /// </summary>
        /// <param name="cacheKey">The name of the object to remove.</param>
        public static void RemoveCachedItem(string cacheKey)
        {
            // Make sure caching is not disabled first
            if (!CachingDisabled)
            {
                LogManager.WriteDebug(EventCode.Cache, string.Format("Removing object from cache {0}", cacheKey), "Object is being removed from the cache");
                // Try to remove the item from the caching block
                if (cache != null && cache.Contains(cacheKey))
                    cache.Remove(cacheKey);
                // We always try to remove the item from the web cache
                webCache.Remove(cacheKey);
            }
        }

        /// <summary>
        /// Removes items from the web based cache by a regular expression.
        /// </summary>
        /// <param name="pattern">The regular expression to match cache keys with.</param>
        public static void RemoveCachedItemByPattern(string pattern)
        {
            // Make sure caching is not disabled first
            if (!CachingDisabled)
            {
                // Get the Regex
                Regex regex = new Regex(pattern, RegexOptions.IgnoreCase | RegexOptions.Singleline | RegexOptions.Compiled);

                // Get the enumerator for the web cache
                IDictionaryEnumerator cacheItems = webCache.GetEnumerator();

                // Apply it to the cache items
                while (cacheItems.MoveNext())
                    if (regex.IsMatch(cacheItems.Key.ToString()))
                        webCache.Remove(cacheItems.Key.ToString());
            }
        }
    }
}
