// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections.Specialized;
using System.Web.Configuration;
using System.Configuration.Provider;

using DotCommunities.Framework.Properties;

namespace DotCommunities.Framework.Components
{
    /// <summary>
    /// Abstract base class used by all configuration providers.
    /// </summary>
    public abstract class CommunityProviderBase : ProviderBase
    {
        private string connectionString;
        private string connectionStringName;

        private string name;
        private string providerType;

        /// <summary>
        /// The name of the provider.
        /// </summary>
        public override string Name
        {
            get { return name; }
        }

        /// <summary>
        /// The type of the provider.
        /// </summary>
        public string ProviderType
        {
            get { return providerType; }
        }

        /// <summary>
        /// The provider connection string.
        /// </summary>
        public string ConnectionString
        {
            get { return connectionString; }
            set { connectionString = value; }
        }

        /// <summary>
        /// The name of the connection string.
        /// </summary>
        public string ConnectionStringName
        {
            get { return connectionStringName; }
            set { connectionStringName = value; }
        }

        /// <summary>
        /// Initialises the provider. Overloaded from <see cref="ProviderBase"/>
        /// </summary>
        /// <param name="name">The name of the provider.</param>
        /// <param name="config">A collection of <see cref="NameValueCollection"/> configureation values.</param>
        public override void Initialize(string name, NameValueCollection config)
        {
            // Verify that config isn't null
            if (config == null)
                throw new ArgumentNullException("config");

            // Assign the provider a default name if it doesn't have one
            if (String.IsNullOrEmpty(name))
                name = "SqlSiteProvider";
            this.name = name;

            // Add a default "description" attribute to config if the
            // attribute doesn't exist or is empty
            if (string.IsNullOrEmpty(config["description"]))
            {
                config.Remove("description");
                config.Add("description", Resources.SqlCommunityProviderDescription);
            }

            // Call the base class's Initialize method
            base.Initialize(name, config);

            // Initialize providerType
            providerType = config["providerType"];

            if (string.IsNullOrEmpty(providerType))
                providerType = "Database";

            config.Remove("providerType");

            // Initialize connectionString
            connectionStringName = config["connectionStringName"];

            if (String.IsNullOrEmpty(connectionStringName))
                throw new ProviderException(Resources.MissingProviderConnectionStringNameException);

            config.Remove("connectionStringName");

            if (WebConfigurationManager.ConnectionStrings[connectionStringName] == null)
                throw new ProviderException(Resources.MissingProviderConnectionStringException);

            connectionString = WebConfigurationManager.ConnectionStrings[connectionStringName].ConnectionString;

            if (String.IsNullOrEmpty(connectionString) && providerType == "Database")
                throw new ProviderException(Resources.EmptyProviderConnectionStringException);

            // Throw an exception if unrecognized attributes remain
            if (config.Count > 0)
            {
                string attr = config.GetKey(0);
                if (!String.IsNullOrEmpty(attr))
                    throw new ProviderException(string.Format(Resources.UnrecognisedProviderAttributeException, attr));
            }
        }

        /// <summary>
        /// Converts a string into a <see cref="DBNull" /> value if it is empty.
        /// </summary>
        /// <param name="text">The string to check.</param>
        /// <returns>The passed string if not empty, otherwise a <see cref="DBNull" /> value.</returns>
        protected static object DBValue(string text)
        {
            if (string.IsNullOrEmpty(text))
                return DBNull.Value;
            // String is not empty so return it
            return text;
        }
    }
}