// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections.Generic;
using System.Xml.Serialization;
using DotCommunities.Framework.Managers;

namespace DotCommunities.Framework.Components
{
    /// <summary>
    /// Holds the settings for each community.
    /// </summary>
    [Serializable]
    [XmlRoot(ElementName = "communitySettings", Namespace = "http://www.dotcommunities.org/1.0/", IsNullable = false)]
    public class CommunitySettings : SerializableObject
    {
        // Email settings
        private string smtpServer;
        private string smtpUserName;
        private string smtpPassword;
        private bool useSmtpAuthentication;
        private short smtpConnectionLimit;
        private int maxEmailProcessingSize;

        // User settings
        private bool requireNewUserApproval;
        private int userDatabaseQuota;

        // Editor settings
        private string defaultTextEditor;

        private List<string> userEditors;
        private List<string> editorRoles;

        // Configuration Settings
        private string timeZone;
        private bool webPartsEnabled;

        // General settings
        private bool enableCategoryMenu;

        #region SMTP & E-mail Settings
        /// <summary>
        /// The SMTP server name for e-mail.
        /// </summary>
        [XmlElement(ElementName = "smtpServer", IsNullable = false)] 
        public string SmtpServer
        {
            get { return smtpServer; }
            set { smtpServer = value; }
        }

        /// <summary>
        /// <c>true</c>, if we need to use a username and password with the SMTP server.
        /// </summary>
        [XmlElement(ElementName = "useSmtpAuthentication", IsNullable = false)]
        public bool UseSmtpAuthentication
        {
            get { return useSmtpAuthentication; }
            set { useSmtpAuthentication = value; }
        }

        /// <summary>
        /// The SMTP user name (if required).
        /// </summary>
        [XmlElement(ElementName = "smtpUserName", IsNullable = true)]
        public string SmtpUserName
        {
            get { return smtpUserName; }
            set { smtpUserName = value; }
        }

        /// <summary>
        /// The SMTP password (if required).
        /// </summary>
        [XmlElement(ElementName = "smtpPassword", IsNullable = true)]
        public string SmtpPassword
        {
            get { return smtpPassword; }
            set { smtpPassword = value; }
        }

        /// <summary>
        /// The SMTP connection limit.
        /// </summary>
        [XmlElement(ElementName = "smtpConnectionLimit", IsNullable = false)]
        public short SmtpConnectionLimit
        {
            get { return smtpConnectionLimit; }
            set { smtpConnectionLimit = value; }
        }

        /// <summary>
        /// The total number of e-mails to process during an e-mail task cycle.
        /// </summary>
        [XmlElement(ElementName = "maxEmailProcessingSize", IsNullable = false)]
        public int MaxEmailProcessingSize
        {
            get { return maxEmailProcessingSize; }
            set { maxEmailProcessingSize = value; }
        }
        #endregion

        #region User Settings
        /// <summary>
        /// If <c>true</c>, users must wait to have their account approved 
        /// by an administrator before it can be accessed.
        /// </summary>
        [XmlElement(ElementName = "requireNewUserApproval", IsNullable = false)]
        public bool RequireNewUserApproval
        {
            get { return requireNewUserApproval; }
            set { requireNewUserApproval = value; }
        }

        /// <summary>
        /// The initial database quota for a new user.
        /// </summary> 
        [XmlElement(ElementName = "userDatabaseQuota", IsNullable = false)]
        public int UserDatabaseQuota
        {
            get { return userDatabaseQuota; }
            set { userDatabaseQuota = value; }
        }
        /// <summary>
        /// The provider name of the default text editor for this community.
        /// </summary>
        [XmlElement(ElementName = "defaultTextEditor", IsNullable = false)]
        public string DefaultTextEditor
        {
            get { return defaultTextEditor; }
            set { defaultTextEditor = value; }
        }

        /// <summary>
        /// The text editors that can be selected by users.
        /// </summary>
        [XmlElement(ElementName = "userEditors", IsNullable = false)]
        public List<string> UserEditors
        {
            get { return userEditors; }
            set { userEditors = value; }
        }

        /// <summary>
        /// The roles of users who can select editors.
        /// </summary>
        [XmlElement(ElementName = "editorRoles", IsNullable = false)]
        public List<string> EditorRoles
        {
            get { return editorRoles; }
            set { editorRoles = value; }
        }

        #endregion

        #region Configuration Settings
        /// <summary>
        /// The server time zone.
        /// </summary> 
        [XmlElement(ElementName = "timeZone", IsNullable = false)]
        public string TimeZone
        {
            get { return timeZone; }
            set { timeZone = value; }
        }

        /// <summary>
        /// If <c>true</c>, web parts are enabled throughout the community.
        /// This is overriden if the site is not configured for web parts 
        /// functionality.
        /// </summary>
        [XmlElement(ElementName = "webPartsEnabled", IsNullable = false)]
        public bool WebPartsEnabled
        {
            get { return webPartsEnabled; }
            set { webPartsEnabled = value; }
        }
        #endregion

        #region General Settings
        /// <summary>
        /// If <c>true</c>, categories can be configured and assigned to content items.
        /// </summary>
        [XmlElement(ElementName = "enableCategoryMenu", IsNullable = false)]
        public bool EnableCategoryMenu
        {
            get { return enableCategoryMenu; }
            set { enableCategoryMenu = value; }
        }
        #endregion

        /// <summary>
        /// Creates a new instance of the <see cref="CommunitySettings"/> class.
        /// </summary>
        public CommunitySettings() 
        {
            userEditors = new List<string>();
            editorRoles = new List<string>();
        }

        /// <summary>
        /// Saves the serialised XML of this instance to the database.
        /// </summary>
        public void Update()
        {
            FrameworkManager.UpdateCommunitySettings(
                CommunityContext.CurrentCommunity.ID,
                this.GetType().AssemblyQualifiedName,
                this.ToXmlString());
        }
    }
}
