// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Collections.Generic;
using System.Text;

namespace DotCommunities.Framework.Components
{
    /// <summary>
    /// The syndication writer for a generic RSS feed.
    /// </summary>
    public class RssWriter : SyndicationWriterBase
    {
        private string baseGuid = null;

        /// <summary>
        /// Constructs a writer for an RSS feed.
        /// </summary>
        /// <param name="content">The entries to write as part of the feed.</param>
        /// <param name="baseUrl">The base Url of the feed.</param>
        public RssWriter(Syndication content, string baseUrl)
            : base(content, baseUrl)
        {
            baseGuid = CommunityContext.CurrentCommunity.ApplicationID.ToString();
        }

        /// <summary>
        /// Build the feed.
        /// </summary>
        protected override void Build()
        {
            StartDocument();
            StartChannel();
            WriteChannel();
            foreach (SyndicationItem si in this.content.Items)
            {
                StartItem();
                WriteItem(si);
                EndItem();
            }
            EndChannel();
            EndDocument();
        }

        /// <summary>
        /// Output the rss base element.
        /// </summary>
        protected virtual void StartDocument()
        {
            this.WriteStartElement("rss");
            this.WriteAttributeString("version", "2.0");
            this.WriteAttributeString("xmlns:slash", "http://purl.org/rss/1.0/modules/slash/");
        }

        /// <summary>
        /// Output the channel element.
        /// </summary>
        protected void StartChannel()
        {
            this.WriteStartElement("channel");
        }

        /// <summary>
        /// Output the header contents of the channel element.
        /// </summary>
        protected void WriteChannel()
        {
            BuildChannel(content.Title, content.Link, content.Description);
        }

        /// <summary>
        /// Actually build the contents of the channel element.
        /// </summary>
        /// <param name="title">The title element of the channel.</param>
        /// <param name="link">The link element of the channel.</param>
        /// <param name="description">The description element of the channel.</param>
        protected void BuildChannel(string title, string link, string description)
        {
            this.WriteElementString("title", title);
            this.WriteElementString("link", link);
            this.WriteElementString("description", description);
            this.WriteElementString("generator", ".communities (http://www.dotcommunities.org/)");
        }

        /// <summary>
        /// Output an item element.
        /// </summary>
        protected void StartItem()
        {
            this.WriteStartElement("item");
        }

        /// <summary>
        /// Actually write the contents of an item element.
        /// </summary>
        protected virtual void WriteItem(SyndicationItem si)
        {
            // Item Info.
            this.WriteElementString("title", si.Title);
            this.WriteElementString("link", CommunityContext.ResolveAbsoluteUrl(si.Link));
            this.WriteElementString("pubDate", si.DateCreated.AddHours(CommunityContext.HostTimeZoneOffset).ToUniversalTime().ToString("r"));

            this.WriteStartElement("guid");
            this.WriteAttributeString("isPermaLink", "false");
            this.WriteString(string.Format("{0}:{1}", baseGuid, si.ContentPageID));
            this.WriteEndElement();

            this.WriteElementString("description", si.Description);

            // Category info.
            if (si.EnableCategories)
                foreach (string category in si.Categories)
                    if (!string.IsNullOrEmpty(category))
                    {
                        this.WriteStartElement("category");
                        this.WriteAttributeString("domain", CommunityContext.ResolveAbsoluteUrl(CommunityContext.ApplicationPath + "/Categories_Category.aspx?Name=" + HttpUtility.UrlEncode(category)));
                        this.WriteString(HttpUtility.UrlEncode(category));
                        this.WriteEndElement();
                    }

            // Comment info.
            if (si.EnableComments)
            {
                this.WriteElementString("comments", CommunityContext.ResolveAbsoluteUrl(si.Link) + "#comments");
                this.WriteElementString("slash:comments", si.CommentCount.ToString());
            }
        }

        /// <summary>
        /// Write the end tag for an item element.
        /// </summary>
        protected void EndItem()
        {
            this.WriteEndElement();
        }

        /// <summary>
        /// Write the end tag for a channel element.
        /// </summary>
        protected void EndChannel()
        {
            this.WriteEndElement();
        }

        /// <summary>
        /// Write the end tag for a rss element.
        /// </summary>
        protected void EndDocument()
        {
            this.WriteEndElement();
        }
    }
}
