// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Configuration;

using DotCommunities.Framework.Properties;

namespace DotCommunities.Framework.Configuration
{
    /// <summary>
    /// A collection of generic handler elements from the configuration file.
    /// </summary>
    public class GenericHandlerElementCollection : ConfigurationElementCollection
    {
        /// <summary>
        /// Returns a generic handler element from the collection.
        /// </summary>
        /// <param name="index">The index of the element.</param>
        /// <returns>A generic handler element.</returns>
        public GenericHandlerElement this[int index]
        {
            get { return (GenericHandlerElement)BaseGet(index); }
        }

        /// <summary>
        /// Property that returns a <see cref="GenericHandlerElement"/> from the collection.
        /// </summary>
        /// <param name="name">The name of the generic handler element to return from the collection.</param>
        /// <returns>The <see cref="GenericHandlerElement"/> from the collection.</returns>
        public new GenericHandlerElement this[string name]
        {
            get { return (GenericHandlerElement)base[name]; }
        }

        /// <summary>
        /// Adds a generic handler to the collection, checking that the object is of 
        /// the <see cref="GenericHandlerElement"/> type.
        /// </summary>
        /// <param name="handler">The <see cref="GenericHandlerElement"/> to add.</param>
        public void Add(GenericHandlerElement handler)
        {
            if (handler == null)
                throw new ArgumentNullException("generichandler");

            if (!(handler is GenericHandlerElement))
                throw new ArgumentException(Resources.InvalidGenericHandlerElementTypeException, "generichandler");

            BaseAdd(handler);
        }

        /// <summary>
        /// (Overridden) Returns the <see cref="ConfigurationElementCollectionType.AddRemoveClearMap" /> setting.
        /// </summary>
        public override ConfigurationElementCollectionType CollectionType
        {
            get { return ConfigurationElementCollectionType.AddRemoveClearMap; }
        }

        /// <summary>
        /// Returns the index of the current element.
        /// </summary>
        /// <param name="handler">The handler element to return the index of.</param>
        /// <returns>The handler element index.</returns>
        public int IndexOf(GenericHandlerElement handler)
        {
            return BaseIndexOf(handler);
        }

        /// <summary>
        /// (Overridden) Creates a new <see cref="GenericHandlerElement"/>.
        /// </summary>
        /// <returns>A new instance of <see cref="GenericHandlerElement"/>.</returns>
        protected override ConfigurationElement CreateNewElement()
        {
            return new GenericHandlerElement();
        }

        /// <summary>
        /// (Overridden) Creates a new <see cref="GenericHandlerElement"/>.
        /// </summary>
        /// <param name="elementName">The name of the element to create.</param>
        /// <returns>A new instance of <see cref="GenericHandlerElement"/>.</returns>
        protected override ConfigurationElement CreateNewElement(string elementName)
        {
            return new GenericHandlerElement(elementName);
        }

        /// <summary>
        /// (Overridden) Gets the key name of the provided element.
        /// </summary>
        /// <param name="element">The <see cref="ConfigurationElement"/> to return the key of.</param>
        /// <returns>The key name of the <see cref="ConfigurationElement"/>.</returns>
        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((GenericHandlerElement)element).Name;
        }
    }
}
