// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;
using System.Collections.Generic;

using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework
{
    /// <summary>
    /// Represents all information about a particular content page. This
    /// class differs from the <see cref="CommunityContent"/> class in that 
    /// it does not represent information about the content of a content page. 
    /// It represents only the information needed to create the page shell
    /// in the site default page.
    /// </summary>
    [Serializable]
    public sealed class ContentPage
    {
        private int _contentPageID;
        private int _parentID;
        private int _sectionID;
        private string _title;
        private string _description;
        private string _metaDescription;
        private string _metaKeys;
        private string _body;
        private string _formattedBody;
        private string _contentType;

        private ModerationStatus _moderationStatus;

        /// <summary>
        /// Represents the content page ID.
        /// </summary>
        public int ContentPageID
        {
            get { return _contentPageID; }
            set { _contentPageID = value; }
        }

        /// <summary>
        /// Represents the parent page of this page.
        /// </summary>
        public int ParentID
        {
            get { return _parentID; }
            set { _parentID = value; }
        }

        /// <summary>
        /// Represents the section associated with this page.
        /// </summary>
        public int SectionID
        {
            get { return _sectionID; }
            set { _sectionID = value; }
        }

        /// <summary>
        /// Represents the title of this page.
        /// </summary>
        public string Title
        {
            get { return _title; }
            set { _title = value; }
        }

        /// <summary>
        /// Represents the description of this page.
        /// </summary>
        public string Description
        {
            get { return _description; }
            set { _description = value; }
        }

        /// <summary>
        /// Represents the content page's meta description. 
        /// </summary>
        public string MetaDescription
        {
            get { return _metaDescription; }
            set { _metaDescription = value; }
        }

        /// <summary>
        /// Represents the content page's meta keys.
        /// </summary>
        public string MetaKeys
        {
            get { return _metaKeys; }
            set { _metaKeys = value; }
        }

        /// <summary>
        /// Represents the content page's body text.
        /// </summary>
        public string Body
        {
            get { return _body; }
            set { _body = value; }
        }

        /// <summary>
        /// Represents the content page's formatted body text.
        /// </summary>
        public string FormattedBody
        {
            get { return _formattedBody; }
            set { _formattedBody = value; }
        }

        /// <summary>
        /// Represents the type of content contained in the content page.
        /// </summary>
        public string ContentType
        {
            get { return _contentType; }
            set { _contentType = value; }
        }

        /// <summary>
        /// Represents the current <see cref="ModerationStatus" /> of the page. 
        /// </summary>
        public ModerationStatus ModerationStatus
        {
            get { return _moderationStatus; }
            set { _moderationStatus = value; }
        }

        /// <summary>
        /// Initializes the <see cref="ContentPage" /> class.
        /// </summary>
        public ContentPage() { }

        /// <summary>
        /// Initializes the <see cref="ContentPage" /> class.
        /// </summary>
        /// <param name="dr">An <see cref="IDataReader"/> containing content row data.</param>
        public ContentPage(IDataReader dr)
        {
            if (dr["ContentPage_ParentID"] != DBNull.Value)
                this.ParentID = (int)dr["ContentPage_ParentID"];
            else
                this.ParentID = -1;

            this.ContentPageID = (int)dr["ContentPage_ID"];            
            this.SectionID = (int)dr["ContentPage_SectionID"];
            this.ContentType = (string)dr["ContentPage_PageTypeName"];
            this.Title = (string)dr["ContentPage_Title"];
            this.Description = (string)dr["ContentPage_Description"];
            this.MetaDescription = (string)dr["ContentPage_MetaDescription"];
            this.MetaKeys = (string)dr["ContentPage_MetaKeys"];
            this.ModerationStatus = (ModerationStatus)dr["contentPage_moderationStatus"];
        }
    }
}
