// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Security;

using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// All administration pages need to be inherited from this abstract class.
    /// </summary>
    public abstract class AdministrationPage : Page
    {
        /// <summary>
        /// When overloaded, is used to update the page title.
        /// </summary>
        protected abstract string PageTitle { get; }

        /// <summary>
        /// When overloaded, is used to update the page sub title.
        /// </summary>
        protected abstract string PageSubTitle { get; }

        /// <summary>
        /// When overloaded, is used to update the side menu item names.
        /// </summary>
        protected abstract string[] MenuItems { get; }

        /// <summary>
        /// When overloaded, is used to update the side menu item links.
        /// </summary>
        protected abstract string[] MenuLinks { get; }

        /// <summary>
        /// When overloaded, is used to update the action list item names.
        /// </summary>
        protected abstract string[] ActionItems { get; }

        /// <summary>
        /// When overloaded, is used to update the action list item links.
        /// </summary>
        protected abstract string[] ActionLinks { get; }

        /// <summary>
        /// Adds page titles to base master page if assigned, if no master is
        /// assigned, checks user security.
        /// </summary>
        /// <param name="e">Initialisation event arguments.</param>
        protected override void OnInit(EventArgs e)
        {
            // Check to see if we have a master page defined, if not we have to check security
            if (Master != null)
            {
                // Setup the page titles
                (Master as AdministrationMasterPage).PageTitle = this.PageTitle;
                (Master as AdministrationMasterPage).PageSubTitle = this.PageSubTitle;
                // Setup the sidebar menus
                (Master as AdministrationMasterPage).InitialiseMenuItems(MenuItems, MenuLinks);
                (Master as AdministrationMasterPage).InitialiseActions(ActionItems, ActionLinks);
            }
            else
            {
                // Redirect to the login page if the user does not have access to the
                // administration section of the site
                if (!(Roles.IsUserInRole(CommunityRole.System.ToString()) || Roles.IsUserInRole(CommunityRole.Administrators.ToString()) || Roles.IsUserInRole(CommunityRole.Demo.ToString())))
                    Response.Redirect("~/Admin/Default.aspx?ReturnUrl=" + HttpUtility.UrlEncode(Request.Path));
            }

            // If there are no problems then pass flow up to base objects
            base.OnInit(e);
        }

        /// <summary>
        /// Returns information on the master page.
        /// </summary>
        /// <param name="path">The page path to redirect to with the message.</param>
        /// <param name="text">The text to display on the master page.</param>
        protected void ReturnInformation(string path, string text)
        {
            (Master as AdministrationMasterPage).ReturnInformation(path, text);
        }

        /// <summary>
        /// Returns a warning on the master page.
        /// </summary>
        /// <param name="path">The page path to redirect to with the message.</param>
        /// <param name="text">The text to display on the master page.</param>
        protected void ReturnWarning(string path, string text)
        {
            (Master as AdministrationMasterPage).ReturnWarning(path, text);
        }

        /// <summary>
        /// Returns an error on the master page.
        /// </summary>
        /// <param name="path">The page path to redirect to with the message.</param>
        /// <param name="text">The text to display on the master page.</param>
        protected void ReturnError(string path, string text)
        {
            (Master as AdministrationMasterPage).ReturnError(path, text);
        }
        
        /// <summary>
        /// Shows information on the master page.
        /// </summary>
        /// <param name="text">The text to display on the master page.</param>
        protected void ShowInformation(string text)
        {
            (Master as AdministrationMasterPage).ShowInformation(text);
        }

        /// <summary>
        /// Shows a warning on the master page.
        /// </summary>
        /// <param name="text">The text to display on the master page.</param>
        protected void ShowWarning(string text)
        {
            (Master as AdministrationMasterPage).ShowWarning(text);
        }

        /// <summary>
        /// Shows an error on the master page.
        /// </summary>
        /// <param name="text">The text to display on the master page.</param>
        protected void ShowError(string text)
        {
            (Master as AdministrationMasterPage).ShowError(text);
        }
    }
}
