// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections.Generic;
using System.ComponentModel;

using DotCommunities.Framework.Designers;
using DotCommunities.Framework.Managers;
using DotCommunities.Framework.Components;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// A check box list that displays categories.
    /// </summary>
    [Designer(typeof(CommunityDesigner))]
    public class CategoryCheckBoxList : CheckBoxList
    {
        private Dictionary<int, Category> categories;

        /// <summary>
        /// Creates a new instance of the <see cref="CategoryCheckBoxList"/> class.
        /// </summary>
        public CategoryCheckBoxList()
            : base()
        {
            // Setup the default properties for the base control
            base.RepeatLayout = RepeatLayout.Table;
            base.RepeatDirection = RepeatDirection.Horizontal;
            base.RepeatColumns = 5;
        }

        /// <summary>
        /// Gets or sets the selected category from the check boxes.
        /// </summary>
        public Dictionary<int, Category> SelectedCategories
        {
            get { return GetSelectedCategories(); }
            set { SetSelectedCategories(value); }
        }

        /// <summary>
        /// Adds the categories to the check down list items.
        /// </summary>
        /// <param name="e">The arguments for this event.</param>
        protected override void OnInit(EventArgs e)
        {
            // Call the base class
            base.OnInit(e);
            // Get the category data
            categories = FrameworkManager.GetCategories();
            // Load the values into the list
            if (categories != null)
                foreach (KeyValuePair<int, Category> category in categories)
                    if (category.Value.SectionID < 0 || 
                       (CommunityContext.CurrentSection != null && 
                       category.Value.SectionID == CommunityContext.CurrentSection.SectionID))
                        this.Items.Add(new ListItem(string.Format("{0}", category.Value.Name), category.Key.ToString()));                        
        }

        /// <summary>
        /// Selects the check boxes that match the supplied categories.
        /// </summary>
        /// <param name="categories">A dictionary list of <see cref="Category"/> objects to select.</param>
        private void SetSelectedCategories(Dictionary<int, Category> categories)
        {
            // Reset the selected items
            ClearSelection();
            // Select the items from the category dictionary
            foreach (KeyValuePair<int, Category> category in categories)
                this.Items.FindByValue(category.Key.ToString()).Selected = true;
        }

        /// <summary>
        /// Retrieves the categories based on the selected check boxes.
        /// </summary>
        /// <returns>A dictionary list of selected <see cref="Category"/> objects.</returns>
        private Dictionary<int, Category> GetSelectedCategories()
        {
            Dictionary<int, Category> selectedCategories = new Dictionary<int, Category>();
            // Loop through the list of categories
            int categoryID = 0;
            foreach (ListItem li in this.Items)
                if (li.Selected && int.TryParse(li.Value, out categoryID))
                    selectedCategories.Add(categoryID, categories[categoryID]);

            return selectedCategories;
        }
    }
}
