// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Managers;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Represents a category page. Category pages list all content that
    /// is associated with the selected category.
    /// </summary>
    public class CategoryListPage : ContentListPage
    {
        string _skinFileName = "Categories_Category.ascx";

        GetContentItemsDelegate _getContentItems = 
            new GetContentItemsDelegate(FrameworkManager.GetCategoryContent);

        Image imgCategoryImage;
        Label lblCategoryTitle;
        PlaceHolder plhCategoryDescription;

        private Category categoryInfo;

        /// <summary>
        /// For categories, the skin is in the ContentSkins directory.
        /// </summary>
        protected override string SkinType
        {
            get { return "ContentSkins"; }
        }

        /// <summary>
        /// Stores the ID of the current category in view state.
        /// </summary>
        /// <value>The category ID.</value>
        int CategoryID
        {
            get { return (int)ViewState["CategoryID"]; }
            set { ViewState["CategoryID"] = value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:CategoryListPage"/> class and
        /// assigns skin and contentItems method to base ContentListPage class.
        /// </summary>
        public CategoryListPage()
            : base()
        {
            // Setup the control
            SkinFileName = _skinFileName;
            GetContentItems = _getContentItems;
            // Wire-up event handlers
            this.SkinLoad += new SkinLoadEventHandler(SkinLoadCategory);
        }

        /// <summary>
        /// Retrieves all the controls from the Page Skin.
        /// </summary>
        /// <param name="s">The sender pf this event.</param>
        /// <param name="e">The <see cref="T:SkinLoadEventArgs"/> instance containing the event data.</param>
        void SkinLoadCategory(Object s, SkinLoadEventArgs e)
        {
            // Find Topic Image
            imgCategoryImage = (Image)GetControl(e.Skin, "imgCategoryImage");

            // Find Topic Title
            lblCategoryTitle = (Label)GetControl(e.Skin, "lblCategoryTitle");

            // Find Topic Description
            plhCategoryDescription = (PlaceHolder)GetControl(e.Skin, "plhCategoryDescription");
        } 

        /// <summary>
        /// Raises the <see cref="E:Control.Init" /> event. Here we check whether a category was passed in a query string.
        /// </summary>
        /// <param name="e">An <see cref="EventArgs" /> object that contains the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            // If no topic, go to home page
            if (!Page.IsPostBack)
                if (Context.Request.QueryString["Category"] == null)
                    Context.Response.Redirect(CommunityContext.ApplicationPath + "Default.aspx");
        }

        /// <summary>
        /// Raises the <see cref="Control.Load" /> event. Handles the Page Load event in order 
        /// to initialize the current page by assigning values to the controls.
        /// </summary>
        /// <param name="e">The <see cref="EventArgs" /> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            string _categoryName = Context.Server.UrlDecode(Context.Request.QueryString["Category"]);

            // Get the category, redirecting if not found
            categoryInfo = FrameworkManager.GetCategory(_categoryName);
            if (categoryInfo == null)
                Context.Response.Redirect(CommunityContext.ApplicationPath + "Default.aspx");

            // Store categoryID in view state
            CategoryID = categoryInfo.CategoryID;

            // Create the controls
            EnsureChildControls();

            if (!Page.IsPostBack)
            {
                // Get total records
                TotalRecords = FrameworkManager.GetTotalCategoryRecords(CategoryID);

                // Display the category image (if specified)
                if (!string.IsNullOrEmpty(categoryInfo.ImagePath))
                    imgCategoryImage.ImageUrl = ResolveUrl("~/" + CommunityContext.CalculateGenericHandlersPath(categoryInfo.ImagePath));
                else
                    imgCategoryImage.Visible = false;

                // Display category title
                lblCategoryTitle.Text = categoryInfo.Name;
            }

            // Display category description
            plhCategoryDescription.Controls.Add(new LiteralControl(categoryInfo.Description));
        }

        /// <summary>
        /// Binds the categories data source to the ContentList control.
        /// </summary>
        override protected void BindContent()
        {
            objContentList.DataSource = GetContentItems(
                                            CategoryID,
                                            CurrentSection.RecordsPerPage, 
                                            objContentList.CurrentPage, 
                                            SortOrder);
            objContentList.DataBind();
        }
    }
}
