// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Enums;
using DotCommunities.Framework.Managers;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Represents the Add Comment page. Enables users to add comments to content.
    /// </summary>
    public class CommentAddPage : CommunityControl
    {
        string _skinFileName = "Comments_AddComment.ascx";

        Panel pnlForm;
        Panel pnlPreview;
        TextBox txtTitle;
        CommentTextEditor txtBody;
        Button btnAdd;
        Button btnEdit;
        Button btnPreview;
        Title lblPreviewTitle;
        CommentText lblPreviewBody;
        Button btnContinue;

        /// <summary>
        /// The type of skin. For example Page, Control, Template
        /// </summary>
        protected override string SkinType
        {
            get { return "ContentSkins"; }
        }

        /// <summary>
        /// Gets or sets the Content Page ID of the resource being commented on.
        /// </summary>
        /// <value>The current content page ID.</value>
        public int ContentPageID
        {
            get { return (int)ViewState["ContentPageID"]; }
            set { ViewState["ContentPageID"] = value; }
        }

        /// <summary>
        /// Gets or sets the Content Page ID of the comment.
        /// </summary>
        /// <value>The comment content page ID.</value>
        public int CommentPageID
        {
            get 
            {
                if (ViewState["CommentPageID"] != null)
                    return (int)ViewState["CommentPageID"];
                return -1;
            }
            set { ViewState["CommentPageID"] = value; }
        }

        /// <summary>
        /// Gets or sets the URL that the user is redirected back to after adding a comment.
        /// </summary>
        /// <value>The return URL.</value>
        public string ReturnUrl
        {
            get
            {
                if (ViewState["ReturnUrl"] == null)
                    return String.Format("{0}.aspx", ContentPageID);
                else
                    return (string)ViewState["ReturnUrl"];
            }
            set { ViewState["ReturnUrl"] = value; }

        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:CommentAddPage"/> class. Also checks whether
        // current user has permission to add a comment.
        /// </summary>
        public CommentAddPage()
            : base()
        {
            // Check security
            if (!CurrentUser.MayComment)
               CommunityContext.RedirectToLoginPage();

            // Assign a default skin file name
            if (SkinFileName == null)
                SkinFileName = _skinFileName;
        }

        /// <summary>
        /// Raises the <see cref="E:Control.Load" /> event.
        /// </summary>
        /// <param name="e">The <see cref="T:EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                // Determine content page
                ContentPageID = Int32.Parse(Context.Request.QueryString["id"]);

                int commentPageID = -1;
                if (Int32.TryParse(Context.Request.QueryString["cid"], out commentPageID))
                    CommentPageID = commentPageID;

                ReturnUrl = Context.Request.QueryString["ReturnUrl"];

                // Ensure the controls are loaded
                EnsureChildControls();

                // Hide preview panel
                pnlPreview.Visible = false;
            }
        }        

        /// <summary>
        /// This method must be implemented by all derived controls. The
        /// inherited method retrieves all the controls that it needs from
        /// the skin.
        /// </summary>
        /// <param name="skin">The skin to retrieve controls from.</param>
        override protected void InitializeSkin(Control skin)
        {
            // Find the Form Panel
            pnlForm = (Panel)GetControl(skin, "pnlForm");
            pnlForm.Visible = true;

            // Find the Preview Panel
            pnlPreview = (Panel)GetControl(skin, "pnlPreview");

            // Find the Title TextBox
            txtTitle = (TextBox)GetControl(skin, "txtTitle");

            // Find the Body TextBox
            txtBody = (CommentTextEditor)GetControl(skin, "txtBody");

            // Find Add Button
            btnAdd = (Button)GetControl(skin, "btnAdd");
            btnAdd.Click += new EventHandler(btnAdd_Click);

            // Find Edit Button
            btnEdit = (Button)GetControl(skin, "btnEdit");
            btnEdit.Visible = false;

            // Find Preview Button
            btnPreview = (Button)GetControl(skin, "btnPreview");
            btnPreview.Click += new EventHandler(btnPreview_Click);

            // Find Preview Title
            lblPreviewTitle = (Title)GetControl(skin, "lblPreviewTitle");

            // Find Preview Body
            lblPreviewBody = (CommentText)GetControl(skin, "lblPreviewBody");

            // Find Continue Button
            btnContinue = (Button)GetControl(skin, "btnContinue");
            btnContinue.Click += new EventHandler(btnContinue_Click);

            btnContinue.CausesValidation = false;
        }

        /// <summary>
        /// Handles the Click event of the btnAdd control.
        /// </summary>
        /// <param name="s">The source of the event.</param>
        /// <param name="e">The <see cref="T:EventArgs"/> instance containing the event data.</param>
        void btnAdd_Click(Object s, EventArgs e)
        {
            ModerationStatus moderationStatus = ModerationStatus.Approved;

            if (Page.IsValid)
            {
                // Determine moderation status
                if (CurrentSection.EnableModeration && !CurrentUser.MayModerate)
                    moderationStatus = ModerationStatus.Pending;

                // Add the comment
                int contentPageID = FrameworkManager.AddComment(                                
                                        ContentPageID, 
                                        CommentPageID,
                                        moderationStatus, 
                                        CurrentUser.UserName, 
                                        txtTitle.Text, 
                                        txtBody.Text);

                // Show warning message if moderation enabled
                if (CurrentSection.EnableModeration && !CurrentUser.MayModerate)
                    Context.Response.Redirect(CommunityContext.CalculatePath(String.Format("Messages_Message.aspx?message=moderation&ReturnUrl={0}", HttpUtility.UrlEncode(ReturnUrl))));

                // Otherwise, redirect back and send notifications
                FrameworkManager.SendNotifications(CurrentSection.SectionID, ContentPageID, CurrentUser.UserName, txtTitle.Text, txtBody.Text, CommunityContext.CalculatePath(ReturnUrl));
            }
        }

        /// <summary>
        /// Handles the Click event of the btnPreview control.
        /// </summary>
        /// <param name="s">The source of the event.</param>
        /// <param name="e">The <see cref="T:EventArgs"/> instance containing the event data.</param>
        void btnPreview_Click(Object s, EventArgs e)
        {
            EnsureChildControls();

            // Display preview panel
            pnlPreview.Visible = true;
            pnlForm.Visible = false;

            // Display preview content
            lblPreviewTitle.Text = txtTitle.Text;
            lblPreviewBody.Text = txtBody.Text;
        }

        /// <summary>
        /// Handles the Click event of the btnContinue control.
        /// </summary>
        /// <param name="s">The source of the event.</param>
        /// <param name="e">The <see cref="T:EventArgs"/> instance containing the event data.</param>
        void btnContinue_Click(Object s, EventArgs e)
        {
            EnsureChildControls();

            // Hide preview panel
            pnlPreview.Visible = false;
            pnlForm.Visible = true;
        }
    }
}
