// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Designers;
using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Represents a link to a page to add a comment.
    /// This control will also check comment permissions before displaying the link.
    /// </summary>
    [Designer(typeof(CommunityDesigner))]
    public class CommentReply : WebControl
    {
        private string _text = String.Empty;
        private string _imageUrl = String.Empty;
        private string _replyUrl = "Comments_AddComment.aspx?id={0}&cid={1}";

        /// <summary>
        /// Gets or sets the text to display in the Hyperlink.
        /// </summary>
        /// <value>The text.</value>
        public string Text
        {
            get { return _text; }
            set { _text = value; }
        }

        /// <summary>
        /// Gets or sets the path for the (optional) picture to display in the hyperlink.
        /// </summary>
        /// <value>The image URL.</value>
        public string ImageUrl
        {
            get { return _imageUrl; }
            set { _imageUrl = value; }
        }

        /// <summary>
        /// Gets or sets the reply URL for the comment page (defaults to the standard one).
        /// </summary>
        /// <value>The reply URL.</value>
        public string ReplyUrl
        {
            get { return _replyUrl; }
            set { _replyUrl = value; }
        }

        /// <summary>
        /// Adds HTML attributes and styles that need to be rendered to the specified <see cref="HtmlTextWriterTag"/>. This method is used primarily by control developers.
        /// </summary>
        /// <param name="writer">A <see cref="HtmlTextWriter"/> that represents the output stream to render HTML content on the client.</param>
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            CommunityContent objContentInfo = CommunityContext.CurrentContent;
            string replyLink = String.Format(_replyUrl, CommunityContext.CurrentPage.PageID, objContentInfo.ContentPageID);
            writer.AddAttribute(HtmlTextWriterAttribute.Href, CommunityContext.CalculatePath(replyLink));
            base.AddAttributesToRender(writer);
        }

        /// <summary>
        /// Renders the control to the specified HTML writer.
        /// </summary>
        /// <param name="writer">The <see cref="HtmlTextWriter"/> object that receives the control content.</param>
        override protected void Render(HtmlTextWriter writer)
        {
            CommunityUser objUserInfo = CommunityContext.CurrentUser;
            Section objSectionInfo = CommunityContext.CurrentSection;
            if (objUserInfo.MayComment || objSectionInfo.CommentRoles.IndexOf(CommunityRole.Authenticated.ToString()) != -1)
                base.Render(writer);
        }

        /// <summary>
        /// Renders the contents of the control to the specified writer. This method is used primarily by control developers.
        /// </summary>
        /// <param name="writer">A <see cref="HtmlTextWriter"/> that represents the output stream to render HTML content on the client.</param>
        override protected void RenderContents(HtmlTextWriter writer)
        {
            if (_imageUrl == String.Empty)
                writer.Write(_text);
            else
                writer.Write(String.Format("<img src=\"{0}\" alt=\"{1}\" border=\"0\" />", Page.ResolveUrl(_imageUrl), _text));
        }

        /// <summary>
        /// Gets the <see cref="HtmlTextWriterTag"/> value that corresponds to this Web server control. This property is used primarily by control developers.
        /// </summary>
        /// <value>An &lt;A&gt; tag around the content of this control.</value>
        /// <returns>One of the <see cref="HtmlTextWriterTag"/> enumeration values.</returns>
        override protected HtmlTextWriterTag TagKey
        {
            get { return HtmlTextWriterTag.A; }
        }
    }
}
