// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using DotCommunities.Framework.Components;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Delegate used for skin loading events.
    /// </summary>
    /// <param name="s">The sender of the event.</param>
    /// <param name="e">The <see cref="SkinLoadEventArgs"/> passed to the event.</param>
    public delegate void SkinLoadEventHandler(Object s, SkinLoadEventArgs e);

    /// <summary>
    /// Delegate used for page submission events.
    /// </summary>
    /// <param name="s">The sender of the event.</param>
    /// <param name="e">The <see cref="EventArgs"/> passed to the event.</param>
    public delegate void SubmitEventHandler(Object s, EventArgs e);

    /// <summary>
    /// Delegate used for page preview events.
    /// </summary>
    /// <param name="s">The sender of the event.</param>
    /// <param name="e">The <see cref="EventArgs"/> passed to the event.</param>
    public delegate void PreviewEventHandler(Object s, EventArgs e);

    /// <summary>
    /// Delegate used for page continue events.
    /// </summary>
    /// <param name="s">The sender of the event.</param>
    /// <param name="e">The <see cref="EventArgs"/> passed to the event.</param>
    public delegate void ContinueEventHandler(Object s, EventArgs e);

    /// <summary>
    /// Class used to add content pages.
    /// </summary>
    public abstract class ContentAddPage : CommunityControl 
    {
        /// <summary>
        /// The add button on the page skin.
        /// </summary>
        protected Button btnAdd;

        /// <summary>
        /// The edit button on the page skin.
        /// </summary>
        protected Button btnEdit;

        /// <summary>
        /// The preview button on the page skin.
        /// </summary>
        protected Button btnPreview;

        /// <summary>
        /// The continue button on the page skin.
        /// </summary>
        protected Button btnContinue;

        /// <summary>
        /// The preview panel on the page skin.
        /// </summary>
        protected Panel pnlPreview;

        /// <summary>
        /// The form panel on the page skin.
        /// </summary>
        protected Panel pnlForm;

        /// <summary>
        /// The categories panel on the page skin.
        /// </summary>
        protected Panel pnlCategories;

        /// <summary>
        /// Event handler for the skin load method.
        /// </summary>
        public event SkinLoadEventHandler SkinLoad; 

        /// <summary>
        /// Event handler for the submit method.
        /// </summary>
        public event SubmitEventHandler Submit;

        /// <summary>
        /// Event handler for the preview method.
        /// </summary>
        public event PreviewEventHandler Preview;

        /// <summary>
        /// Event handler for the continue method.
        /// </summary>
        public event ContinueEventHandler Continue;
        
        string _sectionContent = "Not Specified";


        /// <summary>
        /// Calls the base <see cref="CommunityControl" /> constructor
        /// and assigns the default page skin. Also checks whether
        /// current user has permissions to add content.
        /// </summary>
        public ContentAddPage() 
            : base() 
        {
            if (!CurrentUser.MayAdd)
                CommunityContext.RedirectToLoginPage();
        }
        
        /// <summary>
        /// Make sure that the content of the section matches the content of
        /// the page. Otherwise, the user is trying something sneaky.
        /// </summary>
        /// <param name="e">The <see cref="EventArgs"/> passed to the event.</param>
        protected override void OnInit(EventArgs e)
        {
            string contentName = _sectionContent;

            // Trim the namespace from the section content name
            string[] sectionContentList = _sectionContent.Split('.');
            if (sectionContentList.Length > 1)
                contentName = sectionContentList[sectionContentList.Length - 1];

            if (CurrentSection.Content != contentName)
                throw new Exception(String.Format("Section content {0} does not match {1}!", CurrentSection.Content, _sectionContent));
        }

        
         /// <summary>
        /// Specifies the skins directory where this page's skin file is located.
        /// </summary>
        override protected string SkinType 
        {
            get { return "ContentSkins"; }
        }

        /// <summary>
        /// Specifies the type of content contained in this section.
        /// </summary>
        protected string SectionContent 
        {
            get { return _sectionContent;}
            set { _sectionContent = value; }
        }

        /// <summary>
        /// Retrieves all the controls from the page skin.
        /// </summary>
        /// <param name="skin">The <see cref="Control" /> to initialise.</param>
        override protected void InitializeSkin(Control skin) 
        {
            // Find and hide Edit Button
            btnEdit = (Button)GetOptionalControl(skin, "btnEdit");
            if (btnEdit != null)
                btnEdit.Visible = false;
           
            // Find Add Button
            btnAdd = (Button)GetControl(skin, "btnAdd");
            btnAdd.Click += new EventHandler(OnSubmit);
  
            // Find Topics Panel
            pnlCategories = (Panel)GetOptionalControl(skin, "pnlCategories");
            if (pnlCategories != null)
                if (!CurrentSection.EnableCategories)
                    pnlCategories.Visible = false;
                
            // Find Preview Panel
            pnlPreview = (Panel)GetOptionalControl(skin, "pnlPreview");

            if (pnlPreview != null) 
            {
                // Find Form Panel
                pnlForm = (Panel)GetControl(skin, "pnlForm");

                // Find Preview Button
                btnPreview = (Button)GetControl(skin, "btnPreview");
                btnPreview.Click += new EventHandler(OnPreview);
    
                // Find Continue Button
                btnContinue = (Button)GetControl(skin, "btnContinue");
                if (btnContinue != null) 
                {
                    btnContinue.CausesValidation = false;
                    btnContinue.Click += new EventHandler(OnContinue);
                }
                
                // Hide the Preview Panel on first load
                if (!Page.IsPostBack)
                    pnlPreview.Visible = false;
            }  
  
            // Call Skin Load event 
            OnSkinLoad(this, new SkinLoadEventArgs(skin));
        }
        
        /// <summary>
        /// Method used for skin loading events.
        /// </summary>
        /// <param name="s">The sender of the event.</param>
        /// <param name="e">The <see cref="SkinLoadEventArgs"/> passed to the event.</param>
        protected virtual void OnSkinLoad(Object s, SkinLoadEventArgs e)
        {
            if (SkinLoad != null)
                SkinLoad(s, e);
        }

        /// <summary>
        /// Method used for page submission events.
        /// </summary>
        /// <param name="s">The sender of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> passed to the event.</param>
        protected virtual void OnSubmit(Object s, EventArgs e)
        {
            if (Submit != null)
                Submit(s, EventArgs.Empty);                
        }

        /// <summary>
        /// Method used for preview events.
        /// </summary>
        /// <param name="s">The sender of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> passed to the event.</param>
        protected virtual void OnPreview(Object s, EventArgs e)
        {
            if (Page.IsValid) 
            {
                pnlForm.Visible = false;
                pnlPreview.Visible = true;        
                
                if (Preview != null)
                    Preview(s, e);
            }        
        }

        /// <summary>
        /// Method used for page continue events.
        /// </summary>
        /// <param name="s">The sender of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> passed to the event.</param>
        protected virtual void OnContinue(Object s, EventArgs e)
        {
            pnlForm.Visible = true;
            pnlPreview.Visible = false;
            
            if (Continue != null)
                Continue(s, e);
        }
    }

    /// <summary>
    /// The <see cref="SkinLoadEventArgs"/> object used to store skin load
    /// event data.
    /// </summary>
    public class SkinLoadEventArgs 
    {
        Control _skin;

        /// <summary>
        /// The skin control of the event owner.
        /// </summary>
        public Control Skin 
        {
            get { return _skin; }
        }

        /// <summary>
        /// Initialises a <see cref="SkinLoadEventArgs"/> object.
        /// </summary>
        /// <param name="skin">The <see cref="Control" /> used to raise this event.</param>
        public SkinLoadEventArgs(Control skin) 
        {
            _skin = skin;
        }
    }
}
