// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections.Generic;
using System.ComponentModel;

using DotCommunities.Framework.Components;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Delegate used to handle <see cref="ContentItem"/> events.
    /// </summary>
    /// <param name="sender">An <see cref="Object" /> that represents the sender.</param>
    /// <param name="e">The <see cref="ContentListItemEventArgs"/> instance containing the event data.</param>
    public delegate void ContentListItemEventHandler(object sender, ContentListItemEventArgs e);

    /// <summary>
    /// This class is used when an ItemDataBound event is raised to pass
    /// information about the item that raised the event. 
    /// </summary>
    public sealed class ContentListItemEventArgs : EventArgs
    {
        private ContentItem item;

        /// <summary>
        /// Initializes a new instance of the <see cref="ContentListItemEventArgs"/> class.
        /// </summary>
        /// <param name="item">The <see cref="ContentItem"/>.</param>
        public ContentListItemEventArgs(ContentItem item)
        {
            this.item = item;
        }

        /// <summary>
        /// Gets the <see cref="ContentItem"/>.
        /// </summary>
        /// <value>The content item.</value>
        public ContentItem Item
        {
            get { return item; }
        }
    }

    /// <summary>
    /// The container control we use with the templates. 
    /// </summary>
    public class ContentItem : Control, INamingContainer
    {
        private Object _dataItem;
        private int _itemIndex;
        private ListItemType _itemType;

        /// <summary>
        /// Initializes a new instance of the <see cref="ContentItem"/> class.
        /// </summary>
        public ContentItem() { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ContentItem"/> class.
        /// </summary>
        /// <param name="itemIndex">Index of the item.</param>
        /// <param name="dataItem">The data item.</param>
        /// <param name="itemType">Type of the item.</param>
        public ContentItem(int itemIndex, Object dataItem, ListItemType itemType)
        {
            _dataItem = dataItem;
            _itemIndex = itemIndex;
            _itemType = itemType;
        }

        /// <summary>
        /// Gets the index of the item.
        /// </summary>
        /// <value>The index of the item.</value>
        public Object ItemIndex
        {
            get { return _itemIndex; }
        }

        /// <summary>
        /// Gets the data item.
        /// </summary>
        /// <value>The data item.</value>
        public Object DataItem
        {
            get { return _dataItem; }
        }

        /// <summary>
        /// Gets the type of the item.
        /// </summary>
        /// <value>The type of the item.</value>
        public ListItemType ItemType
        {
            get { return _itemType; }
        }
    }

    /// <summary>
    /// The ContentList control is used to display all the content items
    /// in .communities. The ContentList control has support for paging 
    /// using the database custom paging mechanism.
    /// </summary>
    public class ContentList : WebPartControlBase, INamingContainer, IPostBackEventHandler
    {
        private ITemplate _itemTemplate = null;
        private ITemplate _separatorTemplate = null;
        private ITemplate _noContentTemplate = null;
        private ITemplate _headerTemplate = null;
        private ITemplate _footerTemplate = null;
        private ITemplate _alternatingItemTemplate = null;
        private List<ContentItem> _colContentItems = new List<ContentItem>();
        private Section _sectionInfo;
        private CommunityUser _userInfo;
        private int _totalPages;
        private List<object> _dataSource = null;
        private RepeatLayout _repeatLayout = RepeatLayout.Flow;
        private int _repeatColumns = 1;
        private string _pagerText = String.Empty;
        private string _returnFromEvent = String.Empty;

        private static readonly object EventItemDataBound = new object();

        #region Public events
        /// <summary>
        /// An <see cref="EventHandler"/> which executes when the selected index is chsnged.
        /// </summary>
        public event EventHandler SelectedIndexChanged;

        /// <summary>
        /// An <see cref="EventHandler"/> which executes when an item command is fired.
        /// </summary>
        public event EventHandler ItemCommand;

        /// <summary>
        /// We need this event to perform custom actions during databinding. 
        /// </summary>
        public event ContentListItemEventHandler ItemDataBound
        {
            add { Events.AddHandler(EventItemDataBound, value); }
            remove { Events.RemoveHandler(EventItemDataBound, value); }
        }
        #endregion

        #region Control Templates
        /// <summary>
        /// Gets or sets the item template.
        /// </summary>
        /// <value>The item template.</value>
        [
            PersistenceMode(PersistenceMode.InnerProperty),
            TemplateContainer(typeof(ContentItem))
        ]
        public ITemplate ItemTemplate
        {
            get { return _itemTemplate; }
            set { _itemTemplate = value; }
        }

        /// <summary>
        /// Gets or sets the separator template.
        /// </summary>
        /// <value>The separator template.</value>
        [
            PersistenceMode(PersistenceMode.InnerProperty),
            TemplateContainer(typeof(ContentItem))
        ]
        public ITemplate SeparatorTemplate
        {
            get { return _separatorTemplate; }
            set { _separatorTemplate = value; }
        }

        /// <summary>
        /// Gets or sets the no content template.
        /// </summary>
        /// <remarks>
        /// Whatever you add to the NoContentTemplate is displayed when
        /// there are no records.
        /// </remarks>
        /// <value>The no content template.</value>
        [
            PersistenceMode(PersistenceMode.InnerProperty),
            TemplateContainer(typeof(ContentItem))
        ]
        public ITemplate NoContentTemplate
        {
            get { return _noContentTemplate; }
            set { _noContentTemplate = value; }
        }

        /// <summary>
        /// Gets or sets the header template.
        /// </summary>
        /// <value>The header template.</value>
        [
            PersistenceMode(PersistenceMode.InnerProperty),
            TemplateContainer(typeof(ContentItem))
        ]
        public ITemplate HeaderTemplate
        {
            get { return _headerTemplate; }
            set { _headerTemplate = value; }
        }

        /// <summary>
        /// Gets or sets the footer template.
        /// </summary>
        /// <value>The footer template.</value>
        [
            PersistenceMode(PersistenceMode.InnerProperty),
            TemplateContainer(typeof(ContentItem))
        ]
        public ITemplate FooterTemplate
        {
            get { return _footerTemplate; }
            set { _footerTemplate = value; }
        }

        /// <summary>
        /// Gets or sets the alternating item template.
        /// </summary>
        /// <value>The alternating item template.</value>
        [
            PersistenceMode(PersistenceMode.InnerProperty),
            TemplateContainer(typeof(ContentItem))
        ]
        public ITemplate AlternatingItemTemplate
        {
            get { return _alternatingItemTemplate; }
            set { _alternatingItemTemplate = value; }
        }
        #endregion

        #region Public Properties
        /// <summary>
        /// Gets or sets the data source.
        /// </summary>
        /// <remarks>
        /// You can only use <see cref="List<object>"/> as a Data Source.
        /// </remarks>
        /// <value>The data source.</value>
        public List<object> DataSource
        {
            get { return _dataSource; }
            set { _dataSource = value; }
        }

        /// <summary>
        /// Gets or sets the repeat layout.
        /// </summary>
        /// <remarks>
        /// Like the DataList control, the ContentList supports both flow
        /// and table layouts.
        /// </remarks>
        /// <value>The <see cref="RepeatLayout"/>.</value>
        public RepeatLayout RepeatLayout
        {
            get { return _repeatLayout; }
            set { _repeatLayout = value; }
        }

        /// <summary>
        /// Gets or sets the repeat columns.
        /// </summary>
        /// <remarks>
        /// When using a table layout, determines the number of columns to display.
        /// This property is used in the PhotoGallery to display photo thumbnails.
        /// </remarks>
        /// <value>The number of repeat columns.</value>
        public int RepeatColumns
        {
            get { return _repeatColumns; }
            set { _repeatColumns = value; }
        }

        /// <summary>
        /// Gets or sets the current page.
        /// </summary>
        /// <value>The current page.</value>
        public int CurrentPage
        {
            get
            {
                if (ViewState["CurrentPage"] == null)
                    return 1;
                else
                    return (int)ViewState["CurrentPage"];
            }
            set { ViewState["CurrentPage"] = value; }
        }

        /// <summary>
        /// Gets or sets the total records.
        /// </summary>
        /// <remarks>
        /// We need the total records to display the proper number of pages in the pager.
        /// </remarks>
        /// <value>The total records.</value>
        public int TotalRecords
        {
            get
            {
                if (ViewState["TotalRecords"] == null)
                {
                    return -1;
                }
                return (int)ViewState["TotalRecords"];
            }
            set { ViewState["TotalRecords"] = value; }
        }

        /// <summary>
        /// Gets or sets the pager text. A typical value is "Page {0} of {1}".
        /// </summary>
        /// <value>The pager text.</value>
        public string PagerText
        {
            get { return _pagerText; }
            set { _pagerText = value; }
        }
        #endregion

        /// <summary>
        /// Gets the <see cref="HtmlTextWriterTag" /> value that corresponds to this Web server control. 
        /// This property is used primarily by control developers.
        /// </summary>
        /// <remarks>
        /// If Flow Layout then use Span, otherwise use Table.
        /// </remarks>
        /// <returns>One of the <see cref="HtmlTextWriterTag" /> enumeration values.</returns>
        override protected HtmlTextWriterTag TagKey
        {
            get
            {
                if (_repeatLayout == RepeatLayout.Flow)
                    return HtmlTextWriterTag.Span;
                else
                    return HtmlTextWriterTag.Table;
            }
        }

        /// <summary>
        /// Gets or sets the number of content items that we displayed in the page.
        /// </summary>
        /// <remarks>
        /// This property is used internally to track the number of items to 
        /// display in postback scenarios.
        /// </remarks>
        /// <value>The number of content items.</value>
        private int NumItems
        {
            get
            {
                if (ViewState["NumItems"] == null)
                    return 0;
                else
                    return (int)ViewState["NumItems"];
            }
            set { ViewState["NumItems"] = value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ContentList"/> class.
        /// </summary>
        /// <remarks>
        /// Retrieves the <see cref="Section"/> and <see cref="CommunityUser"/> 
        /// objects from the current context.
        /// </remarks>
        public ContentList()
        {
            if (Context != null && CommunityContext.CurrentSection != null && CommunityContext.CurrentUser != null)
            {
                _sectionInfo = CommunityContext.CurrentSection;
                _userInfo = CommunityContext.CurrentUser;
            }
            else
                _sectionInfo = new Section();
        }

        /// <summary>
        /// Notifies the server control that an element, either XML or HTML, 
        /// was parsed, and adds the element to the server control's <see cref="ControlCollection" /> object.
        /// </summary>
        /// <remarks>
        /// Ignores everything except for content items between the opening
        /// and closing ContentList tags.
        /// </remarks>
        /// <param name="obj">An <see cref="Object" /> that represents the parsed element.</param>
        protected override void AddParsedSubObject(Object obj)
        {
            if (obj is ContentItem)
                Controls.Add((ContentItem)obj);
        }

        /// <summary>
        /// Raises the selected index changed event when a user clicks a page number. You can
        /// override this event to do something custom.
        /// </summary>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected virtual void OnSelectedIndexChanged(EventArgs e)
        {
            if (SelectedIndexChanged != null)
                SelectedIndexChanged(this, e);
        }

        /// <summary>
        /// When implemented by a class, enables a server control to process an event raised when a 
        /// form is posted to the server.
        /// </summary>
        /// <remarks>
        /// This method is called after the user clicks a page number.
        /// </remarks>
        /// <param name="eventArgument">A <see cref="String" /> that represents an optional 
        /// event argument to be passed to the event handler.</param>
        public void RaisePostBackEvent(String eventArgument)
        {
            if (eventArgument == "next")
            {
                CurrentPage++;
                OnSelectedIndexChanged(EventArgs.Empty);
                return;
            }

            if (eventArgument == "prev")
            {
                CurrentPage--;
                OnSelectedIndexChanged(EventArgs.Empty);
                return;
            }

            CurrentPage = Int32.Parse(eventArgument);
            OnSelectedIndexChanged(EventArgs.Empty);
        }

        /// <summary>
        /// Raises the item data bound event. We handle the event raised by this method 
        /// in the Comments control.
        /// </summary>
        /// <param name="e">The <see cref="ContentListItemEventArgs"/> instance containing 
        /// the event data.</param>
        protected virtual void OnItemDataBound(ContentListItemEventArgs e)
        {
            ContentListItemEventHandler onItemDataBoundHandler = (ContentListItemEventHandler)Events[EventItemDataBound];
            if (onItemDataBoundHandler != null)
                onItemDataBoundHandler(this, e);
        }

        /// <summary>
        /// This method is invoked when a child control raises an event.
        /// For example, the ItemEditContent control fires this method
        /// </summary>
        /// <param name="s">An <see cref="Object" /> that represents the sender.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected virtual void OnItemCommand(Object s, EventArgs e)
        {
            if (ItemCommand != null)
                ItemCommand(s, e);
        }

        /// <summary>
        /// This method is executed when a child control bubbles an event.
        /// </summary>
        /// <param name="s">An <see cref="Object" /> that represents the sender.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        /// <returns>This method always returns <c>false</c>.</returns>
        protected override bool OnBubbleEvent(object s, EventArgs e)
        {
            OnItemCommand(s, e);
            return false;
        }

        /// <summary>
        /// Binds a data source to the invoked server control and all its child controls.
        /// </summary>
        override public void DataBind()
        {
            CreateControlHierarchy(true);
            // Prevent CreateChildControls from executing            
            ChildControlsCreated = true;
        }

        /// <summary>
        /// Called by the ASP.NET page framework to notify server controls that use 
        /// composition-based implementation to create any child controls they contain 
        /// in preparation for posting back or rendering.
        /// </summary>
        override protected void CreateChildControls()
        {
            CreateControlHierarchy(false);
        }

        /// <summary>
        /// Creates the control hierarchy.
        /// </summary>
        /// <remarks>
        /// This is where all the work is done. Add all the content items
        /// into the templates to display the contents of the control.
        /// </remarks>
        /// <param name="useDataSource">If set to <c>true</c>, use the datasource.</param>
        void CreateControlHierarchy(bool useDataSource)
        {
            int itemCount;
            Object item = null;
            ContentItem ctlItem;

            // don't do anything if no ItemTemplate
            if (_itemTemplate == null)
                return;

            // Initialize content items collection
            _colContentItems = new List<ContentItem>();

            // if using data source, then populate arraylist
            if (useDataSource)
            {
                if (_dataSource == null)
                    return;

                // Clear any state
                Controls.Clear();
                ClearChildViewState();

                // retrieve content count
                itemCount = _dataSource.Count;
            }
            else
                itemCount = NumItems;

            // if no content, then show nocontent template
            if (itemCount == 0)
            {
                if (_noContentTemplate != null)
                {
                    ctlItem = new ContentItem(0, null, ListItemType.Header);
                    NoContentTemplate.InstantiateIn(ctlItem);
                    Controls.Add(ctlItem);
                    _colContentItems.Add(ctlItem);
                }
            }
            else
            {
                // Create header template
                if (_headerTemplate != null)
                {
                    ctlItem = new ContentItem(0, null, ListItemType.Header);
                    HeaderTemplate.InstantiateIn(ctlItem);
                    Controls.Add(ctlItem);
                    _colContentItems.Add(ctlItem);
                }

                // loop through content, creating templates
                for (int itemIndex = 0; itemIndex < itemCount; itemIndex++)
                {
                    // if dataSource, then grab item
                    if (useDataSource)
                        item = _dataSource[itemIndex];

                    // Return if transformations for the page contains {nocontent}
                    if (_sectionInfo.Transformations.Contains("{nocontent}"))
                        if (item is ChildSection)
                            return;

                    // Add the SeparatorTemplate
                    if (itemIndex > 0 && _separatorTemplate != null)
                    {
                        ctlItem = new ContentItem(itemIndex, null, ListItemType.Separator);
                        SeparatorTemplate.InstantiateIn(ctlItem);
                        Controls.Add(ctlItem);
                        _colContentItems.Add(ctlItem);
                    }

                    if (itemIndex % 2 != 0 && _alternatingItemTemplate != null)
                    {
                        // Add the AlternatingItemTemplate
                        ctlItem = new ContentItem(itemIndex, item, ListItemType.AlternatingItem);
                        AlternatingItemTemplate.InstantiateIn(ctlItem);
                        Controls.Add(ctlItem);
                        _colContentItems.Add(ctlItem);
                    }
                    else
                    {
                        // Add the ItemTemplate
                        ctlItem = new ContentItem(itemIndex, item, ListItemType.Item);
                        ItemTemplate.InstantiateIn(ctlItem);
                        Controls.Add(ctlItem);
                        _colContentItems.Add(ctlItem);
                    }

                    // Perform databinding
                    if (useDataSource)
                    {
                        ContentListItemEventArgs e = new ContentListItemEventArgs(ctlItem);
                        ctlItem.DataBind();
                        OnItemDataBound(e);
                    }
                }

                // Create footer template
                if (_footerTemplate != null)
                {
                    ctlItem = new ContentItem(0, null, ListItemType.Footer);
                    FooterTemplate.InstantiateIn(ctlItem);
                    Controls.Add(ctlItem);
                    _colContentItems.Add(ctlItem);
                }
            }

            // store the number of items created in viewstate for postback scenarios
            NumItems = itemCount;
        }

        /// <summary>
        /// Renders the contents of the control to the specified writer. This method 
        /// is used primarily by control developers.
        /// </summary>
        /// <param name="writer">A <see cref="HtmlTextWriter" /> that represents the 
        /// output stream to render HTML content on the client.</param>
        override protected void RenderContents(HtmlTextWriter writer)
        {
            int repeatCounter = 0;

            // open main table row
            if (_repeatLayout == RepeatLayout.Table)
                writer.RenderBeginTag(HtmlTextWriterTag.Tr);

            // display all the content items
            foreach (ContentItem item in _colContentItems)
            {
                repeatCounter++;

                // open table cell (and possibly row)
                if (_repeatLayout == RepeatLayout.Table)
                {
                    if (repeatCounter > _repeatColumns)
                    {
                        writer.RenderEndTag(); // for last open row
                        writer.RenderBeginTag(HtmlTextWriterTag.Tr); // for new open row
                        repeatCounter = 1;
                    }
                    if (_repeatColumns > 0)
                        writer.AddAttribute(HtmlTextWriterAttribute.Width, Convert.ToString((100 / _repeatColumns)) + "%");
                    writer.AddAttribute(HtmlTextWriterAttribute.Class, "ContentListItem");

                    writer.RenderBeginTag(HtmlTextWriterTag.Td);
                }

                item.RenderControl(writer);

                // close table cell
                if (_repeatLayout == RepeatLayout.Table)
                    writer.RenderEndTag();
            }

            // close main table row
            if (_repeatLayout == RepeatLayout.Table)
                writer.RenderEndTag();

            // add the pager ui
            RenderPager(writer);
        }

        /// <summary>
        /// Render the page numbers for the pager.
        /// </summary>
        /// <param name="writer">A <see cref="HtmlTextWriter" /> that represents the 
        /// output stream to render HTML content on the client.</param>
        protected void RenderPager(HtmlTextWriter writer)
        {
            int totalRecords = 0;

            // See if TotalRecords are defined
            if (TotalRecords > 0)
                totalRecords = TotalRecords;

            // Calculate total Pages
            _totalPages = totalRecords / _sectionInfo.RecordsPerPage;

            // Now do a mod for any remainder
            if ((totalRecords % _sectionInfo.RecordsPerPage) > 0)
                _totalPages++;

            // don't render anything if only 1 page
            if (_totalPages < 2)
                return;

            // Open main pager table
            if (_repeatLayout == RepeatLayout.Flow)
            {
                writer.AddAttribute(HtmlTextWriterAttribute.Width, "100%");
                writer.RenderBeginTag(HtmlTextWriterTag.Table);
            }

            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
            writer.AddAttribute(HtmlTextWriterAttribute.Valign, "bottom");
            writer.AddAttribute(HtmlTextWriterAttribute.Colspan, _repeatColumns.ToString());

            writer.RenderBeginTag(HtmlTextWriterTag.Td);

            writer.AddAttribute(HtmlTextWriterAttribute.Width, "100%");
            writer.RenderBeginTag(HtmlTextWriterTag.Table);
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
            writer.RenderBeginTag(HtmlTextWriterTag.Td);

            // Open page numbers table
            writer.RenderBeginTag(HtmlTextWriterTag.Table);
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);

            // show previous link
            if (CurrentPage > 1)
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Td);

                writer.AddAttribute(HtmlTextWriterAttribute.Href, "javascript:" +
                    Page.ClientScript.GetPostBackEventReference(this, "prev"));
                writer.RenderBeginTag(HtmlTextWriterTag.A);
                writer.Write("&lt;&lt;");
                writer.RenderEndTag();

                writer.RenderEndTag();
            }

            // if less than 11 pages, render all page numbers
            if (_totalPages < 11)
                RenderAllPages(writer);
            else
                // otherwise, do complicated stuff
                RenderPageRange(writer);

            // show next link
            if (CurrentPage < _totalPages)
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Td);

                writer.AddAttribute(HtmlTextWriterAttribute.Href, "javascript:" +
                    Page.ClientScript.GetPostBackEventReference(this, "next"));
                writer.RenderBeginTag(HtmlTextWriterTag.A);
                writer.Write("&gt;&gt;");
                writer.RenderEndTag();

                writer.RenderEndTag();
            }

            // close page numbers table       
            writer.RenderEndTag();
            writer.RenderEndTag();
            writer.RenderEndTag();

            // if pagerText exists, show it
            if (PagerText != String.Empty)
            {
                writer.AddAttribute(HtmlTextWriterAttribute.Align, "right");
                writer.RenderBeginTag(HtmlTextWriterTag.Td);

                writer.Write(PagerText, CurrentPage, _totalPages);

                writer.RenderEndTag();
            }

            // close table       
            writer.RenderEndTag();
            writer.RenderEndTag();

            writer.RenderEndTag();
            writer.RenderEndTag();

            if (_repeatLayout == RepeatLayout.Flow)
                writer.RenderEndTag();
        }

        /// <summary>
        /// Renders all pages.
        /// </summary>
        /// <remarks>
        /// We just loop through and display all the page numbers.
        /// </remarks>
        /// <param name="writer">A <see cref="HtmlTextWriter" /> that represents the 
        /// output stream to render HTML content on the client.</param>
        private void RenderAllPages(HtmlTextWriter writer)
        {
            // display page numbers
            for (int pageNumber = 1; pageNumber <= _totalPages; pageNumber++)
                AddPageNumber(writer, pageNumber);
        }

        /// <summary>
        /// Renders the page range.
        /// </summary>
        /// <remarks>
        /// Too many pages to display them all so instead we display ellipsis.
        /// </remarks>
        /// <param name="writer">A <see cref="HtmlTextWriter" /> that represents the 
        /// output stream to render HTML content on the client.</param>
        private void RenderPageRange(HtmlTextWriter writer)
        {
            int lowerbound = CurrentPage - 1;
            int upperbound = CurrentPage + 1;

            // range check
            if (lowerbound < 1)
                lowerbound = 1;

            if (upperbound > _totalPages)
                upperbound = _totalPages;

            // Show Lower Range
            // if the currentpage is closer to the beginning than to the end, 
            // then show all pages to the current page, otherwise, just
            // display first 3 pages
            if (CurrentPage < (_totalPages - CurrentPage))
                for (int i = 1; i < lowerbound; i++)
                    AddPageNumber(writer, i);
            else
            {
                for (int i = 1; i < 4 & i < lowerbound; i++)
                    AddPageNumber(writer, i);
                // Display ellipsis    
                if (lowerbound > 4)
                {
                    writer.RenderBeginTag(HtmlTextWriterTag.Td);
                    writer.Write("...");
                    writer.RenderEndTag();
                }
            }

            // show mid range
            for (int i = lowerbound; i <= upperbound; i++)
                AddPageNumber(writer, i);

            // Show Upper Range
            // if the currentpage is closer to the end than to the beginning, 
            // then show all pages from the end to the current page, otherwise, just
            // display first 3 pages
            if (upperbound < _totalPages)
            {
                if (CurrentPage > (_totalPages - CurrentPage))
                    for (int i = upperbound + 1; i <= _totalPages; i++)
                        AddPageNumber(writer, i);
                else
                {
                    // Display ellipsis    
                    if (upperbound < (_totalPages - 2))
                    {
                        writer.RenderBeginTag(HtmlTextWriterTag.Td);
                        writer.Write("...");
                        writer.RenderEndTag();
                    }

                    for (int i = _totalPages - 1; i <= _totalPages; i++)
                        AddPageNumber(writer, i);
                }
            }
        }

        /// <summary>
        /// Display an individual page number.
        /// </summary>
        /// <param name="writer">A <see cref="HtmlTextWriter" /> that represents the 
        /// output stream to render HTML content on the client.</param>
        /// <param name="pageNumber">The page number to display.</param>
        void AddPageNumber(HtmlTextWriter writer, int pageNumber)
        {
            writer.RenderBeginTag(HtmlTextWriterTag.Td);

            if (pageNumber == CurrentPage)
                writer.Write("[");
            writer.AddAttribute(HtmlTextWriterAttribute.Href, "javascript:" +
                Page.ClientScript.GetPostBackEventReference(this, pageNumber.ToString()));
            writer.AddAttribute(HtmlTextWriterAttribute.Name, UniqueID);
            writer.AddAttribute(HtmlTextWriterAttribute.Id, UniqueID + "_" + pageNumber.ToString());

            writer.RenderBeginTag(HtmlTextWriterTag.A);
            writer.Write(pageNumber);
            writer.RenderEndTag();
            if (pageNumber == CurrentPage)
                writer.Write("]");

            writer.RenderEndTag();
        }
    }
}
