// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

using DotCommunities.Framework.Designers;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Represents an author displayed in a template.
    /// </summary>
    [Designer(typeof(CommunityDesigner))]
    public class AuthorItem : WebControl 
    {
        private string _externalImage = "~/communities/common/images/external.gif";

        /// <summary>
        /// Initializes a new instance of the <see cref="AuthorItem"/> class.
        /// </summary>
        public AuthorItem()
            : base()
        {
            CssClass = "AuthorItemLink";
            EnableViewState = false;
		}

        /// <summary>
        /// Gets or sets the external image.
        /// </summary>
        /// <remarks>
        /// Represents the relative path to an image displayed for remote users. 
        /// </remarks>
        /// <value>The external image.</value>
        public string ExternalImage 
        {
            get {return _externalImage;}
            set {_externalImage = value; }
        }

        /// <summary>
        /// Raises the <see cref="DataBinding" /> event.
        /// </summary>
        /// <remarks>
        /// Grabs the author from the container's DataItem property.
        /// </remarks>
        /// <param name="e">An <see cref="EventArgs" /> object that contains the event data.</param>
        override protected void OnDataBinding(EventArgs e) {
            ContentItem item;

            if (NamingContainer is ContentItem)
                item = (ContentItem)NamingContainer;
            else
                item = (ContentItem)NamingContainer.NamingContainer;


            CommunityContent objContentInfo = (CommunityContent)item.DataItem;
            ViewState["Author"] = objContentInfo.Author;
            ViewState["RemoteAuthor"] = objContentInfo.RemoteAuthor;
        }

        /// <summary>
        /// Gets the <see cref="HtmlTextWriterTag" /> value that corresponds to this Web server control. This property is used primarily by control developers.
        /// </summary>
        /// <remarks>If local user display a link, otherwise display a span.</remarks>
        /// <returns>One of the <see cref="HtmlTextWriterTag" /> enumeration values.</returns>
        protected override HtmlTextWriterTag TagKey
        {
            get
            {
                if ((string)ViewState["RemoteAuthor"] == String.Empty)
                    return HtmlTextWriterTag.A;
                else
                    return HtmlTextWriterTag.Span;
            }
        }
        
        /// <summary>
        /// Adds HTML attributes and styles that need to be rendered to the specified <see cref="HtmlTextWriterTag" />. This method is used primarily by control developers.
        /// </summary>
        /// <remarks>
        /// Adds the HRef that links to the profile page. 
        /// </remarks>
        /// <param name="writer">A <see cref="HtmlTextWriter" /> that represents the output stream to render HTML content on the client.</param>
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            if ((string)ViewState["RemoteAuthor"] == String.Empty)
            {
                string encodedAuthor = String.Format("Users_ShowProfile.aspx?user={0}", Context.Server.UrlEncode((string)ViewState["Author"]));
                writer.AddAttribute(HtmlTextWriterAttribute.Href, encodedAuthor);
                base.AddAttributesToRender(writer);
            }
        }        

        /// <summary>
        /// Renders the contents of the control to the specified writer. 
        /// This method is used primarily by control developers.
        /// </summary>
        /// <remarks>
        /// Displays the link label with author name.
        /// </remarks>
        /// <param name="writer">A <see cref="HtmlTextWriter" /> that represents the output stream to render HTML content on the client.</param>
        override protected void RenderContents(HtmlTextWriter writer)
        {
            if ((string)ViewState["RemoteAuthor"] == String.Empty)
                writer.Write(Context.Server.HtmlEncode((string)ViewState["Author"]));
            else
            {
                writer.Write(String.Format("<img src=\"{0}\" />", Page.ResolveUrl(_externalImage)));
                writer.Write(Context.Server.HtmlEncode((string)ViewState["RemoteAuthor"]));
            }
        }
    }
}
