// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web.UI;
using System.Web.UI.WebControls;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// WebControl that displays different actions that can be taken
    /// with a content page such as adding, deleting, moving, and editing.
    /// This control checks the current users permissions before
    /// displaying options.
    /// </summary>
    public class EditContent : WebControl
    {

        string _addText = String.Empty;
        string _editText = String.Empty;
        string _deleteText = String.Empty;
        string _moveText = String.Empty;
        string _commentText = String.Empty;
        string _moderateText = String.Empty;
        string _rateText = String.Empty;

        string _addUrl;
        string _editUrl;
        string _deleteUrl;
        string _moveUrl;
        string _commentUrl;
        string _moderateUrl;
        string _rateUrl;

        /// <summary>
        /// Initializes a new instance of the <see cref="EditContent"/> class.
        /// </summary>
        /// <remarks>
        /// Creates a <see cref="WebControl"/> with a table tag as its containing tag.
        /// </remarks>
        public EditContent() : base(HtmlTextWriterTag.Table) { }

        /// <summary>
        /// Sets the text displayed for adding new content.
        /// </summary>
        /// <value>The add new content text.</value>
        public string AddText
        {
            set { _addText = value; }
        }

        /// <summary>
        /// Sets the text displayed for editing content.
        /// </summary>
        /// <value>The edit content text.</value>
        public string EditText
        {
            set { _editText = value; }
        }

        /// <summary>
        /// Sets the text displayed for deleting content.
        /// </summary>
        /// <value>The delete content text.</value>
        public string DeleteText
        {
            set { _deleteText = value; }
        }

        /// <summary>
        /// Sets the text displayed for moving content.
        /// </summary>
        /// <value>The move content text.</value>
        public string MoveText
        {
            set { _moveText = value; }
        }

        /// <summary>
        /// Sets the text displayed for moderating content.
        /// </summary>
        /// <value>The moderate content text.</value>
        public string ModerateText
        {
            set { _moderateText = value; }
        }

        /// <summary>
        /// Sets the text displayed for rating content.
        /// </summary>
        /// <value>The rate content text.</value>
        public string RateText
        {
            set { _rateText = value; }
        }

        /// <summary>
        /// Sets the text displayed for commenting on content.
        /// </summary>
        /// <value>The comment on content text.</value>
        public string CommentText
        {
            set { _commentText = value; }
        }

        /// <summary>
        /// Gets or sets the URL for adding new content.
        /// </summary>
        /// <value>The add content URL.</value>
        public string AddUrl
        {
            set { _addUrl = value; }
            get { return _addUrl; }
        }

        /// <summary>
        /// Gets or sets the URL for editing content.
        /// </summary>
        /// <value>The edit content URL.</value>
        public string EditUrl
        {
            set { _editUrl = value; }
            get { return _editUrl; }
        }

        /// <summary>
        /// Gets or sets the URL for deleting content.
        /// </summary>
        /// <value>The delete content URL.</value>
        public string DeleteUrl
        {
            set { _deleteUrl = value; }
            get { return _deleteUrl; }
        }

        /// <summary>
        /// Gets or sets the URL for moving content.
        /// </summary>
        /// <value>The move content URL.</value>
        public string MoveUrl
        {
            set { _moveUrl = value; }
            get { return _moveUrl; }
        }

        /// <summary>
        /// Gets or sets the URL for moderating content.
        /// </summary>
        /// <value>The moderate content URL.</value>
        public string ModerateUrl
        {
            set { _moderateUrl = value; }
            get { return _moderateUrl; }
        }

        /// <summary>
        /// Gets or sets the URL for rating content.
        /// </summary>
        /// <value>The rate content URL.</value>
        public string RateUrl
        {
            set { _rateUrl = value; }
            get { return _rateUrl; }
        }

        /// <summary>
        /// Gets or sets the URL for commenting on content.
        /// </summary>
        /// <value>The comment on content URL.</value>
        public string CommentUrl
        {
            set { _commentUrl = value; }
            get { return _commentUrl; }
        }

        /// <summary>
        /// Called by the ASP.NET page framework to notify server controls that 
        /// use composition-based implementation to create any child controls they 
        /// contain in preparation for posting back or rendering.
        /// </summary>
        /// <remarks>
        /// Adds the links to this control Controls collection. 
        /// </remarks>
        override protected void CreateChildControls()
        {
            HyperLink lnk;

            // Get UserInfo
            CommunityUser objUserInfo = CommunityContext.CurrentUser;

            // Get SectionInfo
            Section objSectionInfo = CommunityContext.CurrentSection;

            // Add Link
            if ((objUserInfo.MayAdd || objSectionInfo.AddRoles.IndexOf(CommunityRole.Authenticated.ToString()) != -1)
                && _addText != String.Empty)
            {
                lnk = new HyperLink();
                lnk.Text = _addText;
                lnk.NavigateUrl = CommunityContext.CalculatePath(_addUrl);
                Controls.Add(lnk);
            }

            // Edit Link
            if ((objUserInfo.MayEdit || objSectionInfo.EditRoles.IndexOf(CommunityRole.Authenticated.ToString()) != -1)
                && _editText != String.Empty)
            {
                lnk = new HyperLink();
                lnk.Text = _editText;
                lnk.NavigateUrl = CommunityContext.CalculatePath(_editUrl);
                Controls.Add(lnk);
            }

            // Delete Link
            if ((objUserInfo.MayDelete || objSectionInfo.DeleteRoles.IndexOf(CommunityRole.Authenticated.ToString()) != -1)
                && _deleteText != String.Empty)
            {
                lnk = new HyperLink();
                lnk.Text = _deleteText;
                lnk.NavigateUrl = CommunityContext.CalculatePath(_deleteUrl);
                Controls.Add(lnk);
            }

            // Move Link
            if
            (
                ((objUserInfo.MayModerate || objSectionInfo.ModerateRoles.IndexOf(CommunityRole.Authenticated.ToString()) != -1)
                || (objUserInfo.MayEdit || objSectionInfo.EditRoles.IndexOf(CommunityRole.Authenticated.ToString()) != -1))
                && _moveText != String.Empty)
            {
                lnk = new HyperLink();
                lnk.Text = _moveText;
                lnk.NavigateUrl = CommunityContext.CalculatePath(_moveUrl);
                Controls.Add(lnk);
            }

            // Comment Link
            if (objSectionInfo.EnableComments)
            {
                if ((objUserInfo.MayComment || objSectionInfo.CommentRoles.IndexOf(CommunityRole.Authenticated.ToString()) != -1)
                    && _commentText != String.Empty)
                {
                    lnk = new HyperLink();
                    lnk.Text = _commentText;
                    lnk.NavigateUrl = CommunityContext.CalculatePath(_commentUrl);
                    Controls.Add(lnk);
                }
            }

            // Moderate Link
            if (objSectionInfo.EnableModeration)
            {
                if ((objUserInfo.MayModerate || objSectionInfo.ModerateRoles.IndexOf(CommunityRole.Authenticated.ToString()) != -1)
                    && _moderateText != String.Empty)
                {
                    lnk = new HyperLink();
                    lnk.Text = _moderateText;
                    lnk.NavigateUrl = CommunityContext.CalculatePath(_moderateUrl);
                    Controls.Add(lnk);
                }
            }

            // Rate Link
            if (objSectionInfo.EnableRatings)
            {
                if ((objUserInfo.MayRate || objSectionInfo.RateRoles.IndexOf(CommunityRole.Authenticated.ToString()) != -1)
                    && _rateText != String.Empty)
                {
                    lnk = new HyperLink();
                    lnk.Text = _rateText;
                    lnk.NavigateUrl = CommunityContext.CalculatePath(_rateUrl);
                    Controls.Add(lnk);
                }
            }
        }

        /// <summary>
        /// Renders links to the browser by iterating through the Controls collection. 
        /// </summary>
        /// <param name="tw">The <see cref="HtmlTextWriter"/> to render content to.</param>
        protected override void Render(HtmlTextWriter tw)
        {
            if (Controls.Count > 0)
            {
                tw.RenderBeginTag(HtmlTextWriterTag.Table);
                tw.RenderBeginTag(HtmlTextWriterTag.Tr);

                int controlCount = Controls.Count;
                int counter = 0;

                foreach (Control control in Controls)
                {
                    counter++;

                    tw.RenderBeginTag(HtmlTextWriterTag.Td);
                    control.RenderControl(tw);
                    tw.RenderEndTag(); // close td

                    // Render Separator
                    if (counter < controlCount)
                    {
                        tw.RenderBeginTag(HtmlTextWriterTag.Td);
                        tw.Write("|");
                        tw.RenderEndTag(); // close td
                    }
                }

                tw.RenderEndTag(); // close tr
                tw.RenderEndTag(); // close table
            }
        }
    }
}
