// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web.UI;
using System.Web.UI.WebControls;

using DotCommunities.Framework.Managers;
using DotCommunities.Framework.Components;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Enables users with proper permissions to edit content displayed in
    /// a <see cref="ContentList"/> control. This control displays an edit, 
    /// delete, sort up, and sort down button.
    /// </summary>
    public class EditContentItem : WebControl, IPostBackEventHandler
    {
        /// <summary>
        /// The event handler fired when the sort order of the content changes.
        /// </summary>
        public event EventHandler Sorted;

        private CommunityUser objUserInfo;

        private string _editUrl = String.Empty;

        private string _editIcon = "~/Communities/Common/Images/Admin/Edit.gif";

        private string _deleteUrl = "ContentPages_DeleteContentPage.aspx?id={0}&ReturnUrl={1}";
        private string _deleteIcon = "~/Communities/Common/Images/Admin/Delete.gif";

        private string _upIcon = "~/Communities/Common/Images/Admin/up.gif";
        private string _downIcon = "~/Communities/Common/Images/Admin/dn.gif";

        /// <summary>
        /// Initializes a new instance of the <see cref="EditContentItem"/> class.
        /// </summary>
        public EditContentItem()
            : base(HtmlTextWriterTag.Table)
        {
            CssClass = "EditContentItem";
        }

        /// <summary>
        /// Gets or sets the URL for the page for editing this content.
        /// </summary>
        /// <value>The edit URL.</value>
        public string EditUrl
        {
            get { return _editUrl; }
            set { _editUrl = value; }
        }

        /// <summary>
        /// Raises the sorted event.
        /// </summary>
        /// <remarks>
        /// When the sort order changes, we need to tell the <see cref="ContentList"/> control
        /// which contains this control to rebind to its data source. So, we
        /// bubble an event to the containing control as a notification. 
        /// </remarks>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected virtual void OnSorted(EventArgs e)
        {
            if (Sorted != null)
                Sorted(this, e);

            RaiseBubbleEvent(this, e);
        }

        /// <summary>
        /// When implemented by a class, enables a server control to process an 
        /// event raised when a form is posted to the server.
        /// </summary>
        /// <remarks>
        /// When a postback event happens, we know that it must be a sort event,
        /// so we handle the sort event here.
        /// </remarks>
        /// <param name="eventArgument">A <see cref="String"/> that represents 
        /// an optional event argument to be passed to the event handler.</param>
        public void RaisePostBackEvent(string eventArgument)
        {
            if (eventArgument == "up")
                FrameworkManager.MoveContentUp((int)ViewState["ContentPageID"]);

            if (eventArgument == "down")
                FrameworkManager.MoveContentDown((int)ViewState["ContentPageID"]);

            OnSorted(EventArgs.Empty);
        }

        /// <summary>
        /// Returns the URL encoded path of the current page.
        /// </summary>
        /// <value>The return URL.</value>
        private string ReturnUrlHelper
        {
            get { return Context.Server.UrlEncode(Context.Request.RawUrl); }
        }

        /// <summary>
        /// Returns necessary javascript link for postback.
        /// </summary>
        /// <param name="action">The action.</param>
        /// <param name="actionImage">The action image.</param>
        /// <returns></returns>
        private string LinkHelper(string action, string actionImage)
        {
            return String.Format("<a id=\"{0}\" href=\"javascript:" +
                Page.ClientScript.GetPostBackEventReference(this, action) + "\">{1}</a>", UniqueID, actionImage);
        }

        /// <summary>
        /// Raises the <see cref="Control.DataBinding"/> event.
        /// </summary>
        /// <param name="e">An <see cref="EventArgs"/> object that contains 
        /// the event data.</param>
        override protected void OnDataBinding(EventArgs e)
        {
            ContentItem item;

            if (NamingContainer is ContentItem)
                item = (ContentItem)NamingContainer;
            else
                item = (ContentItem)NamingContainer.NamingContainer;


            CommunityContent objContentInfo = (CommunityContent)item.DataItem;
            ViewState["ContentPageID"] = objContentInfo.ContentPageID;
        }

        /// <summary>
        /// Renders the control to the specified HTML writer.
        /// </summary>
        /// <remarks>
        /// Only rendered if the user has edit or delete permissions.
        /// </remarks>
        /// <param name="writer">The <see cref="HtmlTextWriter"/> object that 
        /// receives the control content.</param>
        override protected void Render(HtmlTextWriter writer)
        {
            // Get UserInfo from context
            if (Context != null)
            {
                objUserInfo = CommunityContext.CurrentUser;

                if (objUserInfo.MayEdit || objUserInfo.MayDelete)
                    base.Render(writer);
            }
        }

        /// <summary>
        /// Renders the contents of the control to the specified writer. 
        /// This method is used primarily by control developers.
        /// </summary>
        /// <param name="writer">A <see cref="HtmlTextWriter"/> that represents the 
        /// output stream to render HTML content on the client.</param>
        override protected void RenderContents(HtmlTextWriter writer)
        {
            // Get contentpageID
            int _contentPageID = (int)ViewState["ContentPageID"];

            // open table row
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);

            // Edit cell
            if (objUserInfo.MayEdit)
            {
                _editUrl = String.Format(_editUrl, _contentPageID);
                writer.RenderBeginTag(HtmlTextWriterTag.Td);
                writer.AddAttribute(HtmlTextWriterAttribute.Href, CommunityContext.CalculatePath(_editUrl));
                writer.RenderBeginTag(HtmlTextWriterTag.A);
                writer.Write(String.Format("<img src=\"{0}\" border=\"0\" />", Page.ResolveUrl(_editIcon)));
                writer.RenderEndTag();
                writer.RenderEndTag();
            }

            // Delete cell
            if (objUserInfo.MayDelete)
            {
                _deleteUrl = String.Format(_deleteUrl, _contentPageID, CommunityContext.CurrentSectionPath + "Default.aspx");
                writer.RenderBeginTag(HtmlTextWriterTag.Td);
                writer.AddAttribute(HtmlTextWriterAttribute.Href, CommunityContext.CalculatePath(_deleteUrl));
                writer.RenderBeginTag(HtmlTextWriterTag.A);
                writer.Write(String.Format("<img src=\"{0}\" border=\"0\" />", Page.ResolveUrl(_deleteIcon)));
                writer.RenderEndTag();
                writer.RenderEndTag();
            }

            // Sort cells
            if (objUserInfo.MayEdit)
            {
                // up link
                _upIcon = String.Format("<img src=\"{0}\" border=\"0\" />", Page.ResolveUrl(_upIcon));
                writer.RenderBeginTag(HtmlTextWriterTag.Td);
                writer.Write(LinkHelper("up", _upIcon));
                writer.RenderEndTag();

                // down link
                _downIcon = String.Format("<img src=\"{0}\" border=\"0\" />", Page.ResolveUrl(_downIcon));
                writer.RenderBeginTag(HtmlTextWriterTag.Td);
                writer.Write(LinkHelper("down", _downIcon));
                writer.RenderEndTag();
            }

            writer.RenderEndTag();
        }
    }
}
