// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Designers;
using DotCommunities.Framework.Managers;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Web Control that enables users to rate content.
    /// </summary>
    public abstract class Ratings : WebControl, INamingContainer
    {
        const string DefaultRatingImageFull = "~/Communities/Common/Images/FilledStar.png";
        const string DefaultRatingImageEmpty = "~/Communities/Common/Images/EmptyStar.png";

        string _ratingImageFull = String.Empty;
        string _ratingImageEmpty = String.Empty;

        RadioButtonList rdlRating;
        LinkButton btnSubmit;

        Section objSectionInfo;
        CommunityUser objUserInfo;

        int _averageRating = -1;
        int _yourRating = -1;

        bool _displayOnly = false;

        string _averageRatingText = "Average Rating: ";
        string _yourRatingText = "Your Rating: ";

        string _goodText = "Good";
        string _badText = "Bad";

        string _submitText = "Submit Rating";

        /// <summary>
        /// Initializes a new instance of the <see cref="Ratings"/> class.
        /// </summary>
        public Ratings()
            : base(HtmlTextWriterTag.Table)
        {
            // Set default CSS class
            CssClass = "Rating";

            if (Context != null)
            {
                // Get UserInfo object
                objUserInfo = CommunityContext.CurrentUser;

                // Get SectionInfo object
                objSectionInfo = CommunityContext.CurrentSection;
            }
        }

        /// <summary>
        /// Gets or sets the Content ID for this item.
        /// </summary>
        /// <value>The content page ID.</value>
        public int ContentPageID
        {
            get
            {
                if (ViewState["ContentPageID"] == null)
                    return -1;
                else
                    return (int)ViewState["ContentPageID"];
            }
            set { ViewState["ContentPageID"] = value; }
        }

        /// <summary>
        /// Gets or sets the current user's rating.
        /// </summary>
        /// <value>Your rating.</value>
        public int YourRating
        {
            get { return _yourRating; }
            set { _yourRating = value; }
        }

        /// <summary>
        /// Gets or sets the Average Rating for this item.
        /// </summary>
        /// <value>The average rating.</value>
        public int AverageRating
        {
            get { return _averageRating; }
            set { _averageRating = value; }
        }

        /// <summary>
        /// When true, only displays ratings and does not enable user
        /// to update ratings.
        /// </summary>
        /// <value><c>true</c> if only display ratings; otherwise, <c>false</c>.</value>
        public bool DisplayOnly
        {
            get { return _displayOnly; }
            set { _displayOnly = value; }
        }

        /// <summary>
        /// Gets or sets the average rating text.
        /// </summary>
        /// <value>The average rating text.</value>
        public string AverageRatingText
        {
            get { return _averageRatingText; }
            set { _averageRatingText = value; }
        }

        /// <summary>
        /// Gets or sets your rating text.
        /// </summary>
        /// <value>Your rating text.</value>
        public string YourRatingText
        {
            get { return _yourRatingText; }
            set { _yourRatingText = value; }
        }

        /// <summary>
        /// Gets or sets the full rating image path.
        /// </summary>
        /// <value>The full rating image path.</value>
        public string RatingImageFull
        {
            get { return _ratingImageFull; }
            set { _ratingImageFull = value; }
        }

        /// <summary>
        /// Gets or sets the empty rating image path.
        /// </summary>
        /// <value>The empty rating image path.</value>
        public string RatingImageEmpty
        {
            get { return _ratingImageEmpty; }
            set { _ratingImageEmpty = value; }
        }

        /// <summary>
        /// Gets or sets the text that is displayed for highly rated items.
        /// </summary>
        /// <value>The good text.</value>
        public string GoodText
        {
            get { return _goodText; }
            set { _goodText = value; }
        }

        /// <summary>
        /// Gets or sets the text that is displayed for lowly rated items.
        /// </summary>
        /// <value>The bad text.</value>
        public string BadText
        {
            get { return _badText; }
            set { _badText = value; }
        }

        /// <summary>
        /// Gets or sets the text that is displayed for submitting a rating.
        /// </summary>
        /// <value>The submit text.</value>
        public string SubmitText
        {
            get { return _submitText; }
            set { _submitText = value; }
        }

        /// <summary>
        /// This method is executed when a user submits a new rating.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the rating event data.</param>
        void YourRating_Changed(object sender, EventArgs e)
        {
            int rating;
            if (rdlRating.SelectedIndex != -1)
            {
                rating = Int32.Parse(rdlRating.SelectedItem.Value);
                AverageRating = FrameworkManager.AddRating(objSectionInfo.SectionID, ContentPageID, objUserInfo.UserName, rating);
                YourRating = rating;
            }
        }

        /// <summary>
        /// Called by the ASP.NET page framework to notify server controls that use 
        /// composition-based implementation to create any child controls they contain 
        /// in preparation for posting back or rendering.
        /// </summary>
        override protected void CreateChildControls()
        {
            Controls.Clear();

            if (objUserInfo.MayRate)
            {
                // Create RadioButton List
                rdlRating = new RadioButtonList();
                rdlRating.RepeatDirection = RepeatDirection.Horizontal;
                rdlRating.CssClass = "RateBox";

                for (int i = 1; i < 6; i++)
                    rdlRating.Items.Add(i.ToString());
                
                // Create Submit Button
                btnSubmit = new LinkButton();
                btnSubmit.ID = "btnSubmit";
                btnSubmit.Text = SubmitText;
                btnSubmit.Click += new EventHandler(YourRating_Changed);

                Controls.Add(rdlRating);
                Controls.Add(btnSubmit);
            }
        }

        /// <summary>
        /// Renders the control using the specified text writer.
        /// </summary>
        /// <param name="tw">The <see cref="HtmlTextWriter"/> used to render the control.</param>
        override protected void Render(HtmlTextWriter tw)
        {
            // Check if ratings are enabled
            if (objSectionInfo.EnableRatings)
            {
                // Get the ratings object
                ContentRating rating = FrameworkManager.GetRatings(ContentPageID, objUserInfo.UserName);

                // Allocate the ratings data
                if (rating != null)
                {
                    YourRating = rating.YourRating;
                    AverageRating = rating.AverageRating;
                }
                
                base.Render(tw);
            }
        }

        /// <summary>
        /// Renders the contents of the control using the specified text writer.
        /// </summary>
        /// <param name="tw">The <see cref="HtmlTextWriter"/> used to render the control.</param>
        override protected void RenderContents(HtmlTextWriter tw)
        {
            // Display Checkbox List
            if (objUserInfo.MayRate && YourRating == -1 && !DisplayOnly)
            {
                tw.RenderBeginTag(HtmlTextWriterTag.Tr);

                tw.RenderBeginTag(HtmlTextWriterTag.Td);
                tw.Write(_badText);
                tw.RenderEndTag();

                tw.RenderBeginTag(HtmlTextWriterTag.Td);
                rdlRating.RenderControl(tw);
                tw.RenderEndTag();

                tw.RenderBeginTag(HtmlTextWriterTag.Td);
                tw.Write(_goodText);
                tw.RenderEndTag();

                tw.RenderBeginTag(HtmlTextWriterTag.Td);
                tw.Write("|");
                tw.RenderEndTag();

                tw.RenderBeginTag(HtmlTextWriterTag.Td);
                btnSubmit.RenderControl(tw);
                tw.RenderEndTag();

                tw.RenderEndTag(); // close tr

                // Display Average rating if available
                if (AverageRating != -1)
                {
                    tw.RenderBeginTag(HtmlTextWriterTag.Tr);

                    tw.AddAttribute(HtmlTextWriterAttribute.Colspan, "4");
                    tw.RenderBeginTag(HtmlTextWriterTag.Td);
                    tw.Write(_averageRatingText);
                    tw.Write(GetRatingImage(AverageRating));
                    tw.RenderEndTag();

                    tw.RenderEndTag(); // close tr
                }
            }
            else
            {

                // Do not render anything if there is nothing to render.
                // Prevents unwanted background and border in control.
                if ((AverageRating == -1) || (!objUserInfo.MayRate))
                {
                    this.Visible = false;
                    return;
                }

                tw.RenderBeginTag(HtmlTextWriterTag.Tr);

                // Display Your Rating
                if (YourRating != -1)
                {
                    tw.RenderBeginTag(HtmlTextWriterTag.Td);
                    tw.Write(_yourRatingText);
                    tw.Write(GetRatingImage(YourRating));
                    tw.RenderEndTag();
                }

                if (DisplayOnly)
                {
                    tw.RenderEndTag(); // close tr
                    tw.RenderBeginTag(HtmlTextWriterTag.Tr);
                }

                tw.RenderBeginTag(HtmlTextWriterTag.Td);

                // Display Average Rating
                if (AverageRating != -1)
                {
                    tw.Write(_averageRatingText);
                    tw.Write(GetRatingImage(AverageRating));
                }
                else
                    tw.Write("&nbsp;");

                tw.RenderEndTag();

                tw.RenderEndTag(); // close tr
            }
        }

        /// <summary>
        /// Returns the appropriate images for a particular rating.
        /// </summary>
        /// <param name="rating">The rating score.</param>
        /// <returns>A string containing the image HTML for a rating.</returns>
        private string GetRatingImage(int rating)
        {
            string ratingHtml = string.Empty;

            // Generate the full images
            for (int i = 1; i <= rating; i++)
                if (!string.IsNullOrEmpty(_ratingImageFull))
                    ratingHtml += String.Format("<img src=\"{0}\">", Page.ResolveUrl(_ratingImageFull));
                else
                    ratingHtml += String.Format("<img src=\"{0}\">", Page.ResolveUrl(DefaultRatingImageFull));

            // Generate the empty images
            for (int j = rating + 1; j <= 5; j++)
                if (!string.IsNullOrEmpty(_ratingImageEmpty))
                    ratingHtml += String.Format("<img src=\"{0}\">", Page.ResolveUrl(_ratingImageEmpty));
                else
                    ratingHtml += String.Format("<img src=\"{0}\">", Page.ResolveUrl(DefaultRatingImageEmpty));

            return ratingHtml;
        }
    }
}
