// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

using DotCommunities.Framework.Designers;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Creates a control that displays the letters A through Z as clickable controls.
    /// </summary>
    [ParseChildren(true)]
    [DefaultProperty("SelectedLetter")]
    [Designer(typeof(CommunityDesigner))]
    public class AlphaPicker : WebControl, INamingContainer
    {
        private string currentLetter = "All";

        /// <summary>
        /// Event raised when a letter has been clicked by the end user
        /// </summary>
        public event EventHandler LetterChanged;

        /// <summary>
        /// Property that returns the currently selected letter.
        /// </summary>
        public string SelectedLetter
        {
            get
            {
                if (ViewState["SelectedLetter"] == null)
                    return currentLetter;

                return (string)ViewState["SelectedLetter"];
            }
            set { ViewState["SelectedLetter"] = value; }
        }

        /// <summary>
        /// Creates the link buttons.
        /// </summary>
        /// <param name="e">The <see cref="EventArgs"/> of the OnLoad event.</param>
        protected override void OnLoad(EventArgs e)
        {
            // Perform the base OnLoad event
            base.OnLoad(e);
            // Only select the buttons when not posting back
            if (!Page.IsPostBack)
                SelectCurrentLinkButton();
        }

        /// <summary>
        /// Renders the alpha picker letter buttons.
        /// </summary>
        protected override void  CreateChildControls()
        {
            Label label;

            // Add the series of linkbuttons
            char chrStart = 'A';
            char chrStop = 'Z';

            // Loop through all the characters
            for (int iLoop = chrStart; iLoop <= chrStop; iLoop++)
            {
                Controls.Add(CreateLetteredLinkButton(((char)iLoop).ToString()));

                label = new Label();
                label.CssClass = "alphaSpacer";
                label.Text = " | ";
                Controls.Add(label);
            }

            Controls.Add(CreateLetteredLinkButton("All"));
        }

        /// <summary>
        /// Creates a new link button.
        /// </summary>
        /// <param name="buttonText">The text of the button.</param>
        /// <returns>A new <see cref="LinkButton"/> control.</returns>
        private LinkButton CreateLetteredLinkButton(String buttonText)
        {
            // Add a new link button
            LinkButton btnTmp = new LinkButton();

            btnTmp.ID = string.Format("btn{0}", buttonText);
            btnTmp.Text = buttonText;
            btnTmp.CommandArgument = buttonText;
            btnTmp.Click += new System.EventHandler(Letter_Clicked);
            btnTmp.Enabled = SelectedLetter != buttonText;
            
            if (btnTmp.Enabled)
                btnTmp.CssClass = "alphaLetter";
            else
                btnTmp.CssClass = "alphaLetterSelected";

            return btnTmp;
        }

        /// <summary>
        /// Updates the selected button properties.
        /// </summary>
        private void SelectCurrentLinkButton()
        {
            // Loop through the control collection
            foreach (Control c in Controls)
                if (c is LinkButton)
                {
                    LinkButton selectedButton = (LinkButton)c;

                    // Update the selected buttons
                    if (c.ID == string.Format("btn{0}", SelectedLetter))
                    {
                        selectedButton.Enabled = false;
                        selectedButton.CssClass = "alphaLetterSelected";
                    }
                    else
                    {
                        selectedButton.Enabled = true;
                        selectedButton.CssClass = "alphaLetter";
                    }
                }
        }

        /// <summary>
        /// Event raised when a letter is clicked upon.
        /// </summary>
        /// <param name="sender">The sender of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> for the event.</param>
        private void Letter_Clicked(Object sender, EventArgs e)
        {
            // Set the selected letter
            SelectedLetter = ((LinkButton)sender).CommandArgument;
            // Select the appropriate letter button
            SelectCurrentLinkButton();
            // Fire the LetterChanged event
            if (null != LetterChanged)
                LetterChanged(sender, e);
        }
    }
}