// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections;
using System.Collections.Specialized;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Designers;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Creates a server or client side list picker.
    /// </summary>
    [Designer(typeof(CommunityDesigner))]
    public class ListPicker : WebControl, IPostBackDataHandler, INamingContainer
    {
        /// <summary>
        /// Event handler used when the items in the list have changed.
        /// </summary>
        public event EventHandler ListChanged;

        private bool _isUplevel;
        private ArrayList _allItems = new ArrayList();
        private ArrayList _selectedItems = new ArrayList();
        private IEnumerable _dataSource = null;
        private string _dataField = String.Empty;


        // The following controls are used by downlevel browsers
        ListBox lstAllItems;
        ListBox lstSelectedItems;
        Button btnAddItem;
        Button btnRemoveItem;

        #region Public Properties
        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="ListPicker" /> has changed.
        /// </summary>
        /// <value><c>true</c> if changed; otherwise, <c>false</c>.</value>
        public bool Changed
        {
            get
            {
                if (ViewState["Changed"] == null)
                    return false;
                else
                    return (bool)ViewState["Changed"];
            }
            set { ViewState["Changed"] = value; }
        }

        /// <summary>
        /// Gets or sets the selected items text.
        /// </summary>
        /// <value>The selected items text.</value>
        public string SelectedItemsText
        {
            get
            {
                if (ViewState["SelectedItemsText"] == null)
                    return "Selected Items";
                else
                    return (string)ViewState["SelectedItemsText"];
            }
            set { ViewState["SelectedItemsText"] = value; }
        }

        /// <summary>
        /// Gets or sets all items text.
        /// </summary>
        /// <value>All items text.</value>
        public string AllItemsText
        {
            get
            {
                if (ViewState["AllItemsText"] == null)
                    return "All Items";
                else
                    return (string)ViewState["AllItemsText"];
            }
            set { ViewState["AllItemsText"] = value; }
        }

        /// <summary>
        /// Gets or sets the number of items displayed by the picker.
        /// </summary>
        /// <value>The number of items.</value>
        public int Size
        {
            get
            {
                if (ViewState["Size"] == null)
                    return 5;
                else
                    return (int)ViewState["Size"];
            }
            set { ViewState["Size"] = value; }
        }

        /// <summary>
        /// Gets or sets the state between postbacks so change notification can 
        /// be raised only when state changes. 
        /// </summary>
        /// <value>The current selected state.</value>
        private string SelectedState
        {
            get
            {
                if (ViewState["selectedState"] == null)
                    return String.Empty;
                else
                    return (string)ViewState["selectedState"];
            }
            set { ViewState["selectedState"] = value; }
        }

        /// <summary>
        /// Gets or sets the data source for the All Items listbox.
        /// </summary>
        /// <value>The data source.</value>
        public IEnumerable DataSource
        {
            get { return _dataSource; }
            set { _dataSource = value; }
        }

        /// <summary>
        /// Gets or sets the data field for the All Items Data Source.
        /// </summary>
        /// <value>The data field.</value>
        public string DataField
        {
            get { return _dataField; }
            set { _dataField = value; }
        }

        /// <summary>
        /// Gets or sets an array of selected items.
        /// </summary>
        /// <value>The selected items.</value>
        public string[] SelectedItems
        {
            get
            {
                if (!_isUplevel)
                {
                    EnsureChildControls();
                    _selectedItems.Clear();
                    foreach (ListItem item in lstSelectedItems.Items)
                        _selectedItems.Add(item.Text);
                }
                return (string[])_selectedItems.ToArray(typeof(String));
            }
            set
            {
                _selectedItems = new ArrayList(value);
                SelectedState = String.Join(",", value);
            }
        }

        /// <summary>
        /// Gets  an array of all items.
        /// </summary>
        /// <value>All items.</value>
        public string[] AllItems
        {
            get { return (string[])_allItems.ToArray(typeof(String)); }
        }
        #endregion

        /// <summary>
        /// Gets the selected helper ID of the hidden form field for the selected list box.
        /// </summary>
        /// <value>The selected helper ID.</value>
        protected string SelectedHelperID
        {
            get { return ClientID + "_SelectedState"; }
        }

        /// <summary>
        /// Gets the ID of the hidden form field for the all list box.
        /// </summary>
        /// <value>All helper ID.</value>
        protected string AllHelperID
        {
            get { return ClientID + "_AllState"; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ListPicker"/> class.
        /// </summary>
        public ListPicker()
            : base(HtmlTextWriterTag.Table)
        {
            if (Context != null)
            {
                if ((CommunityContext.Request.Browser.JScriptVersion.Major > 0))
                    _isUplevel = true;
                else
                    _isUplevel = false;
            }
        }

        /// <summary>
        /// Raises the <see cref="Control.Init"/> event.
        /// </summary>
        /// <remarks>
        /// Notifies framework that this control wants to be notified of data
        /// changes on postback.
        /// </remarks>
        /// <param name="e">An <see cref="EventArgs" /> object that contains 
        /// the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            if (Page != null)
                Page.RegisterRequiresPostBack(this);
        }

        /// <summary>
        /// This method retrieves the posted data and updates the control properties.
        /// </summary>
        /// <param name="postDataKey">The post data key.</param>
        /// <param name="values">The values.</param>
        /// <returns><c>true</c>, if the post data has been loaded; otherwise <c>false</c>.</returns>
        public bool LoadPostData(String postDataKey, NameValueCollection values)
        {
            string _allState;
            string _selectedState;

            // return if null
            if (values[AllHelperID] == null)
                return false;

            if (_isUplevel)
            {
                _allState = values[AllHelperID].Trim();
                _selectedState = values[SelectedHelperID].Trim();

                if (_allState == String.Empty)
                    _allItems.Clear();
                else
                    _allItems = new ArrayList(_allState.Split(','));

                if (_selectedState == String.Empty)
                    _selectedItems.Clear();
                else
                    _selectedItems = new ArrayList(_selectedState.Split(','));

                // No change, return false
                if (SelectedState == _selectedState.Trim())
                    return false;

                // Change, return true and update state
                Changed = true;
                SelectedState = _selectedState;
                return true;
            }
            else
            {
                if (Changed)
                    return true;
                else
                    return false;
            }
        }

        /// <summary>
        /// When implemented by a class, signals the server control to notify the 
        /// ASP.NET application that the state of the control has changed.
        /// </summary>
        /// <remarks>
        /// This method raises the OnListChanged event if posted data changed.
        /// </remarks>
        public void RaisePostDataChangedEvent()
        {
            OnListChanged(EventArgs.Empty);
        }

        /// <summary>
        /// Raises the list changed event for handling list control post data changes.
        /// </summary>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected virtual void OnListChanged(EventArgs e)
        {
            if (ListChanged != null)
                ListChanged(this, e);
        }

        /// <summary>
        /// Raises the <see cref="Control.DataBinding"/> event.
        /// </summary>
        /// <param name="e">An <see cref="EventArgs"/> object that contains the event data.</param>
        protected override void OnDataBinding(EventArgs e)
        {
            IEnumerator objDataEnum;

            // bind all items
            if (_dataSource != null)
            {
                // Populate all items
                objDataEnum = _dataSource.GetEnumerator();
                while (objDataEnum.MoveNext())
                    if (_dataField == String.Empty)
                        _allItems.Add((string)objDataEnum.Current);
                    else
                        _allItems.Add((string)DataBinder.Eval(objDataEnum.Current, _dataField));
            }

            // Remove selected items from all items
            foreach (string item in _selectedItems)
                _allItems.Remove(item);

            // Bind to ListBox for downlevel browsers
            if (!_isUplevel)
            {
                EnsureChildControls();

                lstAllItems.DataSource = _allItems;
                lstAllItems.DataBind();

                lstSelectedItems.DataSource = _selectedItems;
                lstSelectedItems.DataBind();
            }
        }

        /// <summary>
        /// Called by the ASP.NET page framework to notify server controls that use 
        /// composition-based implementation to create any child controls they 
        /// contain in preparation for posting back or rendering.
        /// </summary>
        override protected void CreateChildControls()
        {
            if (!_isUplevel)
            {
                lstAllItems = new ListBox();
                Controls.Add(lstAllItems);

                lstSelectedItems = new ListBox();
                Controls.Add(lstSelectedItems);

                btnAddItem = new Button();
                btnAddItem.Text = "->";
                btnAddItem.CausesValidation = false;
                btnAddItem.Click += new EventHandler(btnAddClick);
                Controls.Add(btnAddItem);

                btnRemoveItem = new Button();
                btnRemoveItem.Text = "<-";
                btnRemoveItem.CausesValidation = false;
                btnRemoveItem.Click += new EventHandler(btnRemoveClick);
                Controls.Add(btnRemoveItem);
            }
        }

        /// <summary>
        /// Raised by clicking add button on downlevel browser.
        /// </summary>
        /// <param name="s">The sender.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        void btnAddClick(Object s, EventArgs e)
        {
            if (lstAllItems.SelectedIndex != -1)
            {
                // Move the item
                lstSelectedItems.SelectedIndex = -1;
                lstSelectedItems.Items.Add(lstAllItems.SelectedItem);
                lstAllItems.Items.RemoveAt(lstAllItems.SelectedIndex);

                // update changed status items
                Changed = true;
            }
        }

        /// <summary>
        /// Raised by clicking remove button on downlevel browser.
        /// </summary>
        /// <param name="s">The sender.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        void btnRemoveClick(Object s, EventArgs e)
        {
            if (lstSelectedItems.SelectedIndex != -1)
            {
                // Move the item
                lstAllItems.SelectedIndex = -1;
                lstAllItems.Items.Add(lstSelectedItems.SelectedItem);
                lstSelectedItems.Items.RemoveAt(lstSelectedItems.SelectedIndex);

                // update changed status items
                Changed = true;
            }
        }

        /// <summary>
        /// Raises the <see cref="Control.PreRender" /> event.
        /// </summary>
        /// <param name="e">An <see cref="EventArgs"/> object that contains the event data.</param>
        override protected void OnPreRender(EventArgs e)
        {
            if (_isUplevel)
            {
                if (!Page.ClientScript.IsClientScriptBlockRegistered("listpicker"))
                    Page.ClientScript.RegisterClientScriptBlock(this.GetType(), "listpicker", String.Format("<script language=\"JScript\" Src=\"{0}\"></script>", CommunityContext.ApplicationPath + "Scripts/listpicker.js"));
                Page.ClientScript.RegisterHiddenField(SelectedHelperID, String.Join(",", SelectedItems));
                Page.ClientScript.RegisterHiddenField(AllHelperID, String.Join(",", AllItems));
            }
        }

        /// <summary>
        /// Renders the contents of the control to the specified writer. 
        /// This method is used primarily by control developers.
        /// </summary>
        /// <remarks>
        /// Displays either uplevel or downlevel content.
        /// </remarks>
        /// <param name="writer">A <see cref="HtmlTextWriter"/> that represents the 
        /// output stream to render HTML content on the client.</param>
        override protected void RenderContents(HtmlTextWriter writer)
        {
            if (_isUplevel)
                RenderUplevelContent(writer);
            else
                RenderDownlevelContent(writer);
        }

        /// <summary>
        /// Renders the uplevel content.
        /// </summary>
        /// <param name="writer">The writer.</param>
        void RenderUplevelContent(HtmlTextWriter writer)
        {
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);

            // Add Labels
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);

            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write(AllItemsText);
            writer.RenderEndTag();
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write("&nbsp;");
            writer.RenderEndTag();
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write(SelectedItemsText);
            writer.RenderEndTag();

            writer.RenderEndTag();

            // Add All listbox
            writer.RenderBeginTag(HtmlTextWriterTag.Td);

            writer.AddAttribute(HtmlTextWriterAttribute.Id, ClientID + "_SrcList");
            writer.AddAttribute(HtmlTextWriterAttribute.Size, Size.ToString());
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "listPicker");
            writer.RenderBeginTag(HtmlTextWriterTag.Select);

            foreach (string item in _allItems)
                writer.Write(String.Format("<option value=\"{0}\">{0}</option>", item));

            writer.RenderEndTag();
            writer.RenderEndTag();

            // Add Button
            writer.RenderBeginTag(HtmlTextWriterTag.Td);

            writer.AddAttribute(HtmlTextWriterAttribute.Onclick, String.Format("addItem('{0}')", ClientID));
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "listPickerArrow");
            writer.RenderBeginTag(HtmlTextWriterTag.Button);
            writer.Write("-&gt;");
            writer.RenderEndTag();

            // Delete Button
            writer.RenderBeginTag(HtmlTextWriterTag.P);
            writer.AddAttribute(HtmlTextWriterAttribute.Onclick, String.Format("removeItem('{0}')", ClientID));
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "listPickerArrow");
            writer.RenderBeginTag(HtmlTextWriterTag.Button);
            writer.Write("&lt;-");
            writer.RenderEndTag();
            writer.RenderEndTag();

            writer.RenderEndTag();

            // Add selected listbox
            writer.RenderBeginTag(HtmlTextWriterTag.Td);

            writer.AddAttribute(HtmlTextWriterAttribute.Id, ClientID + "_DstList");
            writer.AddAttribute(HtmlTextWriterAttribute.Size, Size.ToString());
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "listPicker");
            writer.RenderBeginTag(HtmlTextWriterTag.Select);

            foreach (string item in _selectedItems)
                writer.Write(String.Format("<option value=\"{0}\">{0}</option>", item));

            writer.RenderEndTag();
            writer.RenderEndTag();
            writer.RenderEndTag();
        }

        /// <summary>
        /// Renders the downlevel content.
        /// </summary>
        /// <param name="writer">The writer.</param>
        void RenderDownlevelContent(HtmlTextWriter writer)
        {
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);

            // Add Labels
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);

            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write(AllItemsText);
            writer.RenderEndTag();
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write("&nbsp;");
            writer.RenderEndTag();
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write(SelectedItemsText);
            writer.RenderEndTag();

            writer.RenderEndTag();

            // Add All listbox
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            lstAllItems.RenderControl(writer);
            writer.RenderEndTag();

            // Add Button
            writer.RenderBeginTag(HtmlTextWriterTag.Td);

            btnAddItem.RenderControl(writer);

            // Delete Button
            writer.RenderBeginTag(HtmlTextWriterTag.P);
            btnRemoveItem.RenderControl(writer);
            writer.RenderEndTag();

            writer.RenderEndTag();

            // Add selected listbox
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            lstSelectedItems.RenderControl(writer);
            writer.RenderEndTag();

            writer.RenderEndTag();
        }
    }
}
