// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Managers;
using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Creates a control that displays rotating RSS information from
    /// the current sections RSS feed.
    /// </summary>
    public class RssRotator : CommunityControl
    {
        string _skinFileName = "Syndication_RssRotator.ascx";

        UpdatePanel RssUpdater;
        HyperLink TitleLink;
        Label DescriptionLabel;

        /// <summary>
        /// Gets or sets the current RSS item index.
        /// </summary>
        protected int CurrentItem
        {
            get
            {
                if (ViewState["CurrentItem"] != null)
                    return (int)ViewState["CurrentItem"];
                return 0;
            }
            set { ViewState["CurrentItem"] = value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RssRotator"/> class.
        /// </summary>
        public RssRotator()
            : base()
        {
            if (Context != null)
            {
                // Assign a default skin file name
                if (SkinFileName == null)
                    SkinFileName = _skinFileName;
                // Set the controls visibility
                this.Visible = CommunityContext.IsSectionPage && CurrentSection.IsSectionSyndicated;
            }
        }

        /// <summary>
        /// Specifies the skins directory where this page's skin file is located.
        /// </summary>
        /// <value>The type of skin. For example Page, Control, Template.</value>
        override protected string SkinType
        {
            get { return "ControlSkins"; }
        }

        /// <summary>
        /// This method must be implemented by all derived controls. The
        /// inherited method retrieves all the controls that it needs from
        /// the skin.
        /// </summary>
        /// <param name="skin">The control skin.</param>
        override protected void InitializeSkin(Control skin)
        {
            RssUpdater = (UpdatePanel)GetControl(skin, "RssUpdater");
            TitleLink = (HyperLink)GetControl(skin, "TitleLink");
            DescriptionLabel = (Label)GetControl(skin, "DescriptionLabel");
        }

        /// <summary>
        /// Retrieves the RSS from the feed.
        /// </summary>
        /// <param name="e">The loading event arguments.</param>
        protected override void OnLoad(EventArgs e)
        {
            ServiceResponse serviceResponse = GetServiceResponse();

            if (serviceResponse != null && serviceResponse.Items != null && serviceResponse.Items.Count > 1)
            {
                ServiceResponseItem responseItem = serviceResponse.Items[CurrentItem];

                if (CurrentItem < serviceResponse.Items.Count - 1)
                    CurrentItem = CurrentItem + 1;
                else
                    CurrentItem = 0;

                if (responseItem != null)
                {
                    TitleLink.Text = CommunityContext.TruncateWithEllipsis(responseItem.Title, 35);
                    TitleLink.NavigateUrl = responseItem.Link;
                    DescriptionLabel.Text = CommunityContext.TruncateWithEllipsis(responseItem.Description, 50);
                }
            }
            else
                this.Visible = false;
        }

        /// <summary>
        /// Retrieves the RSS feed.
        /// </summary>
        /// <returns>A <see cref="ServiceResponse"/> object containing feed items.</returns>
        private ServiceResponse GetServiceResponse()
        {
            Service serviceInfo = new Service();
            serviceInfo.Url = GetResponseUrl();
            serviceInfo.ServiceType = ServiceType.RSS;
            serviceInfo.RefreshRate = 24 * 60;
            serviceInfo.MaximumItems = 10;
            serviceInfo.Name = CommunityContext.Server.UrlEncode(serviceInfo.Url);
            serviceInfo.ID = -1;

            return FrameworkManager.GetServiceResponse(serviceInfo.Name, ref serviceInfo);
        }

        /// <summary>
        /// Gets the absolute Url of the RSS feed, checking for generic RSS handlers.
        /// </summary>
        /// <returns>The Url to the RSS feed.</returns>
        private string GetResponseUrl()
        {
            string path = "default.rss";
            if (CommunityContext.UseGenericHandlers)
            {
                // Check if we have a handler for the extension
                string handler = CommunityContext.GenericHandlers["rss"].ControlName;
                // If we have a handler, make that the path
                if (!string.IsNullOrEmpty(handler))
                    path = handler;
            }
            return CommunityContext.ResolveAbsoluteUrl(CommunityContext.CurrentSectionPath + path);
        }
    }
}
