// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Drawing;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

using DotCommunities.Framework.Designers;
using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Displays quota statistics and validates whether or not user
    /// has passed either community or user quota.
    /// </summary>
    [Designer(typeof(CommunityDesigner))]
    public class DisplayQuota : WebControl, IValidator
    {
        private bool _isValid = false;
        private string _errorMessage = "You have exceeded your quota. Please contact the Administrator of this Web site.";
        private string _text = "You have exceeded your quota. Please contact the Administrator of this Web site.";
        private bool _displayStatistics = true;
        private Quota _quotaInfo = null;

        private string _userQuotaText = "User Quota: {0:n0} KB";
        private string _userQuotaUsedText = "User Quota Used: {0:n0} KB";

        private string _communityQuotaText = "Community Quota: {0:n0} KB";
        private string _communityQuotaUsedText = "Community Quota Used: {0:n0} KB";

        private Style _errorStyle = new Style();

        /// <summary>
        /// Gets or sets the user quota text.
        /// </summary>
        /// <value>The user quota text.</value>
        public string UserQuotaText
        {
            get { return _userQuotaText; }
            set { _userQuotaText = value; }
        }

        /// <summary>
        /// Gets or sets the user quota used text.
        /// </summary>
        /// <value>The user quota used text.</value>
        public string UserQuotaUsedText
        {
            get { return _userQuotaUsedText; }
            set { _userQuotaUsedText = value; }
        }

        /// <summary>
        /// Gets or sets the community quota text.
        /// </summary>
        /// <value>The community quota text.</value>
        public string CommunityQuotaText
        {
            get { return _communityQuotaText; }
            set { _communityQuotaText = value; }
        }

        /// <summary>
        /// Gets or sets the community quota used text.
        /// </summary>
        /// <value>The community quota used text.</value>
        public string CommunityQuotaUsedText
        {
            get { return _communityQuotaUsedText; }
            set { _communityQuotaUsedText = value; }
        }

        /// <summary>
        /// When implemented by a class, gets or sets the error message text generated when the condition being validated fails.
        /// </summary>
        /// <value>The error message to generate.</value>
        public string ErrorMessage
        {
            get { return _errorMessage; }
            set { _errorMessage = value; }
        }

        /// <summary>
        /// Gets the error message style.
        /// </summary>
        /// <value>The error message style.</value>
        public Style ErrorStyle
        {
            get { return _errorStyle; }
        }

        /// <summary>
        /// Gets or sets the Error Message displayed inline with the control.
        /// </summary>
        /// <value>The Error Message text.</value>
        public string Text
        {
            get
            {
                if (_text != String.Empty)
                    return _text;
                else
                    return _errorMessage;
            }
            set { _text = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether control displays quota stats.
        /// </summary>
        /// <value><c>true</c> if quota stats are displayed; otherwise, <c>false</c>.</value>
        public bool DisplayStatistics
        {
            get { return _displayStatistics; }
            set { _displayStatistics = value; }
        }

        /// <summary>
        /// When implemented by a class, gets or sets a value indicating whether the user-entered content in the specified control passes validation.
        /// </summary>
        /// <value></value>
        /// <returns>true if the content is valid; otherwise, false.</returns>
        public bool IsValid
        {
            get { return _isValid; }
            set { _isValid = value; }
        }

        /// <summary>
        /// When implemented by a class, evaluates the condition it checks and updates the <see cref="IsValid" /> property.
        /// </summary>
        public void Validate()
        {
        }

        /// <summary>
        /// Raises the <see cref="OnInit" /> event to the page's validators collection.
        /// </summary>
        /// <param name="e">An <see cref="EventArgs" /> object that contains the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            Page.Validators.Add(this);

            // assign some default styles
            if (_displayStatistics)
            {
                BorderStyle = BorderStyle.Solid;
                BorderWidth = new Unit("1px");
                BorderColor = Color.Black;
                BackColor = Color.WhiteSmoke;
                Font.Size = FontUnit.Parse("12px");
            }
        }

        //*********************************************************************
        //
        // OnUnload Method
        //
        // Remove this control from the Validators collection.
        //
        //*********************************************************************

        /// <summary>
        /// Raises the <see cref="OnUnload" /> event from the Validators collection.
        /// </summary>
        /// <param name="e">An <see cref="EventArgs" /> object that contains event data.</param>
        protected override void OnUnload(EventArgs e)
        {
            if (Page != null)
            {
                Page.Validators.Remove(this);
            }
            base.OnUnload(e);
        }

        /// <summary>
        /// Evaluates that the quota is valid.
        /// </summary>
        /// <returns>true, if the quota has passed the limit.</returns>
        protected bool EvaluateIsValid()
        {
// TODO Fix Quota information
#if !DEBUG
            if (_quotaInfo == null)
                return false;

            if (_quotaInfo.CommunityQuotaUsed > _quotaInfo.CommunityQuota)
                return false;

            if (_quotaInfo.UserQuotaUsed > _quotaInfo.UserQuota)
                return false;
#endif
            return true;
        }

        /// <summary>
        /// Renders the control to the specified HTML writer.
        /// </summary>
        /// <param name="writer">The <see cref="HtmlTextWriter" /> object that receives the control content.</param>
        override protected void Render(HtmlTextWriter writer)
        {
            if (_quotaInfo != null)
                base.Render(writer);
        }

        /// <summary>
        /// Renders the quota stats and error message (if appropriate) to the specified writer. This method is used primarily by control developers.
        /// </summary>
        /// <param name="writer">A <see cref="HtmlTextWriter" /> that represents the output stream to render HTML content on the client.</param>
        override protected void RenderContents(HtmlTextWriter writer)
        {
            // Display Statistics
            if (_displayStatistics)
            {
                // Display user quota
                RenderUserQuota(writer);

                // Display community quota (if administrator)
                // We have to use the Identity object here instead of UserInfo because
                // of the admin menu.
                if (Context.User.IsInRole(CommunityRole.Administrators.ToString()))
                    RenderCommunityQuota(writer);
            }

            // Display Error Message            
            if (_isValid == false)
                RenderErrorMessage(writer);
        }

        /// <summary>
        /// Renders the user quota statistics.
        /// </summary>
        /// <param name="writer">The writer.</param>
        private void RenderUserQuota(HtmlTextWriter writer)
        {
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);

            // Display user quota
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write(String.Format(_userQuotaText, _quotaInfo.UserQuota / 1024));
            writer.RenderEndTag();


            // Create a spacer cell
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write("&nbsp;");
            writer.RenderEndTag();


            // Display user quota used
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write(String.Format(_userQuotaUsedText, _quotaInfo.UserQuotaUsed / 1024));
            writer.RenderEndTag();


            writer.RenderEndTag();
        }

        /// <summary>
        /// Renders the community quota statistics.
        /// </summary>
        /// <param name="writer">The writer.</param>
        private void RenderCommunityQuota(HtmlTextWriter writer)
        {
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);

            // Display community quota
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write(String.Format(_communityQuotaText, _quotaInfo.CommunityQuota / 1024));
            writer.RenderEndTag();


            // Create a spacer cell
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write("&nbsp;");
            writer.RenderEndTag();

            // Display community quota used
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write(String.Format(_communityQuotaUsedText, _quotaInfo.CommunityQuotaUsed / 1024));
            writer.RenderEndTag();


            writer.RenderEndTag();
        }

        /// <summary>
        /// Renders the error message.
        /// </summary>
        /// <param name="writer">The writer.</param>
        private void RenderErrorMessage(HtmlTextWriter writer)
        {
            // if display stats, then open TR and TD
            if (_displayStatistics)
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Tr);
                writer.AddAttribute(HtmlTextWriterAttribute.Colspan, "3");
                writer.RenderBeginTag(HtmlTextWriterTag.Td);
            }

            // write error message
            _errorStyle.AddAttributesToRender(writer);
            writer.RenderBeginTag(HtmlTextWriterTag.Span);
            writer.Write(_text);
            writer.RenderEndTag();

            // if display stats, close TR and TD
            if (_displayStatistics)
            {
                writer.RenderEndTag();
                writer.RenderEndTag();
            }
        }

        /// <summary>
        /// If displaying quota stats render a table, otherwise render span.
        /// This property is used primarily by control developers.
        /// </summary>
        /// <value>The <see cref="HtmlTextWriterTag" /> value that corresponds to this Web server control.</value>
        /// <returns>One of the <see cref="HtmlTextWriterTag" /> enumeration values.</returns>
        override protected HtmlTextWriterTag TagKey
        {
            get
            {
                if (_displayStatistics)
                    return HtmlTextWriterTag.Table;
                else
                    return HtmlTextWriterTag.Span;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DisplayQuota"/> class,
        /// assigning default CSS class and validating quota stats.
        /// </summary>
        public DisplayQuota()
            : base()
        {
            CssClass = "displayQuota";

            // assign a red font to error style by default
            _errorStyle.ForeColor = System.Drawing.Color.Red;


            // Get Quota Info
            if (Context != null)
            {
                if (!Context.User.Identity.IsAuthenticated)
                    throw new Exception("Can't calculate quota without a user");

                //CommunityProfile communityProfile = new CommunityProfile();
                //_quotaInfo = new QuotaInfo(
                //    communityProfile.CommunityQuota,
                //    communityProfile.CommunityQuotaUsed,
                //    communityProfile.UserQuota,
                //    communityProfile.UserQuotaUsed);

                _isValid = EvaluateIsValid();
            }
        }
    }
}
