// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Security;

using DotCommunities.Framework.Components;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Represents the Edit Profile page. The Edit Profile page
    /// enables users to modify their profile information. 
    /// </summary>
    public class EditProfile : CommunityControl
    {
        string _skinFileName = "Users_EditProfile.ascx";

        ProfileForm ctlProfileForm;
        Panel pnlProfileUpdateSuccessful;
        Button btnUpdate;

        /// <summary>
        /// Calls the base <see cref="CommunityControl"/> constructor
        /// and assigns the default page skin. Also checkes
        /// whether user is authenticated.
        /// Initializes a new instance of the <see cref="EditProfile"/> class.
        /// </summary>
        public EditProfile()
            : base()
        {
            // Check Security
            if (!CurrentUser.IsAuthenticated)
                CommunityContext.RedirectToLoginPage();

            // Assign a default skin file name
            if (SkinFileName == null)
                SkinFileName = _skinFileName;
        }

        /// <summary>
        /// Specifies the skins directory where this page's skin file is located.
        /// </summary>
        override protected string SkinType
        {
            get { return "ContentSkins"; }
        }

        /// <summary>
        /// Represents the user ID associated with the profile being edited.
        /// </summary>
        /// <value>The user ID.</value>
        private int UserID
        {
            get { return (int)ViewState["UserID"]; }
            set { ViewState["UserID"] = value; }
        }

        /// <summary>
        /// This method must be implemented by all derived controls. The
        /// inherited method retrieves all the controls that it needs from
        /// the skin.
        /// </summary>
        /// <param name="skin">The control skin.</param>
        override protected void InitializeSkin(Control skin)
        {
            // Find the Profile Form control
            ctlProfileForm = (ProfileForm)GetControl(skin, "ctlProfileForm");

            // Find the update sucessful panel
            pnlProfileUpdateSuccessful = (Panel)GetControl(skin, "pnlProfileUpdateSuccessful");

            // Find the Update button
            btnUpdate = (Button)GetControl(skin, "btnUpdate");
            btnUpdate.Click += new EventHandler(btnUpdate_Click);
        }

        /// <summary>
        /// Assigns values to the controls from the page skin.
        /// Raises the <see cref="E:System.Web.UI.Control.Load"></see> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"></see> object that contains the event data.</param>
        override protected void OnLoad(EventArgs e)
        {
            EnsureChildControls();

            if (!Page.IsPostBack)
            {
                //// Get Current User Profile information
                CommunityProfile objProfile = CommunityContext.CurrentUser.Profile;
                if (objProfile != null)
                {
                    ctlProfileForm.FirstName = objProfile.FirstName;
                    ctlProfileForm.LastName = objProfile.LastName;
                    ctlProfileForm.TimeZone = objProfile.TimeZone;
                    ctlProfileForm.Occupation = objProfile.Occupation;
                    ctlProfileForm.Location = objProfile.Location;
                    ctlProfileForm.Interests = objProfile.Interests;
                    ctlProfileForm.MSN = objProfile.MSN;
                    ctlProfileForm.Yahoo = objProfile.Yahoo;
                    ctlProfileForm.AIM = objProfile.AIM;
                    ctlProfileForm.ICQ = objProfile.ICQ;
                    ctlProfileForm.Url = objProfile.Website;
                    ctlProfileForm.FakeEmail = objProfile.PublicEMail;
                    ctlProfileForm.PublishEMail = objProfile.PublishEMail;
                    ctlProfileForm.PublishFullName = objProfile.PublishFullName;
                    ctlProfileForm.EnablePrivateMessages = objProfile.PrivateMessages;
                }
                // Get the user information that is specific to this community
                ctlProfileForm.EnableNewsletter = CurrentUser.LocalUser.EnableNewsletter;
                ctlProfileForm.EnableNotifications = CurrentUser.LocalUser.EnableNotifications;
            }
        }

        /// <summary>
        /// Updates user profile information or displays error message to user.
        /// </summary>
        /// <param name="s">The source of the event.</param>
        /// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
        private void btnUpdate_Click(Object s, EventArgs e)
        {
            if (Page.IsValid)
            {
                CommunityProfile objProfile = CommunityContext.CurrentUser.Profile;
                if (objProfile != null)
                {

                    objProfile.FirstName = ctlProfileForm.FirstName;
                    objProfile.LastName = ctlProfileForm.LastName;
                    objProfile.TimeZone = ctlProfileForm.TimeZone;
                    objProfile.Occupation = ctlProfileForm.Occupation;
                    objProfile.Location = ctlProfileForm.Location;
                    objProfile.Interests = ctlProfileForm.Interests;
                    objProfile.MSN = ctlProfileForm.MSN;
                    objProfile.Yahoo = ctlProfileForm.Yahoo;
                    objProfile.AIM = ctlProfileForm.AIM;
                    objProfile.ICQ = ctlProfileForm.ICQ;
                    objProfile.Website = ctlProfileForm.Url;
                    objProfile.PublicEMail = ctlProfileForm.FakeEmail;
                    objProfile.PublishEMail = ctlProfileForm.PublishEMail;
                    objProfile.PublishFullName = ctlProfileForm.PublishFullName;
                    objProfile.PrivateMessages = ctlProfileForm.EnablePrivateMessages;

                    objProfile.Save();

                }
                
                // Save the user information that is specific to this community
                CurrentUser.LocalUser.EnableNewsletter = ctlProfileForm.EnableNewsletter;
                CurrentUser.LocalUser.EnableNotifications = ctlProfileForm.EnableNotifications;

                CurrentUser.LocalUser.Save();

                pnlProfileUpdateSuccessful.Visible = true;
            }
        }
    }
}
