// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.ComponentModel;

using DotCommunities.Framework.Designers;
using DotCommunities.Framework.Components;

namespace DotCommunities.Framework.Controls
{
    /// <summary>
    /// Class for displaying a web part options.
    /// </summary>
    [Designer(typeof(CommunityDesigner))]
    public class WebPartOptionList : WebControl, INamingContainer
    {
        WebPartManager wpManager;

        LinkButton btnBrowseView;
        LinkButton btnDesignView;
        LinkButton btnEditView;
        LinkButton btnCatalogView;
        LinkButton btnPersonalizationModeToggle;

        /// <summary>
        /// Creates a new instance of <see cref="WebPartOptionList"/>.
        /// </summary>
        public WebPartOptionList()
            : base(HtmlTextWriterTag.Table)
        {
            btnBrowseView = new LinkButton();
            btnDesignView = new LinkButton();
            btnEditView = new LinkButton();
            btnCatalogView = new LinkButton();
            btnPersonalizationModeToggle = new LinkButton();

            btnBrowseView.Click += new EventHandler(btnBrowseView_Click);
            btnDesignView.Click += new EventHandler(btnDesignView_Click);
            btnEditView.Click += new EventHandler(btnEditView_Click);
            btnCatalogView.Click += new EventHandler(btnCatalogView_Click);
            btnPersonalizationModeToggle.Click += new EventHandler(btnPersonalizationModeToggle_Click);

            btnBrowseView.Enabled = false;
            btnDesignView.Enabled = false;
            btnEditView.Enabled = false;
            btnCatalogView.Visible = false;
            btnPersonalizationModeToggle.Visible = false;
        }

        /// <summary>
        /// Updates the display mode to <see cref="WebPartManager.BrowseDisplayMode"/>.
        /// </summary>
        /// <param name="sender">The sender of the event.</param>
        /// <param name="e">The event arguments.</param>
        protected void btnBrowseView_Click(object sender, EventArgs e)
        {
            wpManager.DisplayMode = WebPartManager.BrowseDisplayMode;
            UpdateUI();
        }

        /// <summary>
        /// Updates the display mode to <see cref="WebPartManager.DesignDisplayMode"/>.
        /// </summary>
        /// <param name="sender">The sender of the event.</param>
        /// <param name="e">The event arguments.</param>
        protected void btnDesignView_Click(object sender, EventArgs e)
        {
            wpManager.DisplayMode = WebPartManager.DesignDisplayMode;
            UpdateUI();
        }

        /// <summary>
        /// Updates the display mode to <see cref="WebPartManager.EditDisplayMode"/>.
        /// </summary>
        /// <param name="sender">The sender of the event.</param>
        /// <param name="e">The event arguments.</param>
        protected void btnEditView_Click(object sender, EventArgs e)
        {
            wpManager.DisplayMode = WebPartManager.EditDisplayMode;
            UpdateUI();
        }

        /// <summary>
        /// Updates the display mode to <see cref="WebPartManager.CatalogDisplayMode"/>.
        /// </summary>
        /// <param name="sender">The sender of the event.</param>
        /// <param name="e">The event arguments.</param>
        protected void btnCatalogView_Click(object sender, EventArgs e)
        {
            wpManager.DisplayMode = WebPartManager.CatalogDisplayMode;
            UpdateUI();
        }

        /// <summary>
        /// Toggles the scope of personalisation between User and Shared.
        /// </summary>
        /// <param name="sender">The sender of the event.</param>
        /// <param name="e">The event arguments.</param>
        protected void btnPersonalizationModeToggle_Click(object sender, EventArgs e)
        {
            wpManager.Personalization.ToggleScope();
            UpdateUI();
        }

        /// <summary>
        /// Event called when the page is loading.
        /// </summary>
        /// <param name="e">The event arguments.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            // Get the web part manager from the page
            wpManager = WebPartManager.GetCurrentWebPartManager(Page);

            // Update the UI on first load
            if (!Page.IsPostBack)
                UpdateUI();
        }

        /// <summary>
        /// Updates the UI buttons and visibility of the control.
        /// </summary>
        private void UpdateUI()
        {
            if (wpManager != null)
            {
                EnsureChildControls();

                btnBrowseView.Enabled = wpManager.SupportedDisplayModes.Contains(WebPartManager.BrowseDisplayMode);
                btnDesignView.Enabled = wpManager.SupportedDisplayModes.Contains(WebPartManager.DesignDisplayMode);
                btnEditView.Enabled = wpManager.SupportedDisplayModes.Contains(WebPartManager.EditDisplayMode);
                btnCatalogView.Visible = wpManager.SupportedDisplayModes.Contains(WebPartManager.CatalogDisplayMode);
                btnPersonalizationModeToggle.Visible = wpManager.Personalization.CanEnterSharedScope;
                btnPersonalizationModeToggle.Text = string.Format("Current Scope: {0}",
                   wpManager.Personalization.Scope.ToString());

                this.Visible = wpManager.Zones.Count > 0 &&
                btnBrowseView.Enabled &&
                btnDesignView.Enabled &&
                btnEditView.Enabled &&
                btnCatalogView.Visible &&
                btnPersonalizationModeToggle.Visible;
            }
            else
                this.Visible = false;
        }

        /// <summary>
        /// Creates the buttons to toggle between different views.
        /// </summary>
        override protected void CreateChildControls()
        {
            Controls.Clear();

            btnBrowseView.Text = "Browse View";
            btnDesignView.Text = "Design View";
            btnEditView.Text = "Edit View";
            btnCatalogView.Text = "Catalog View";

            Controls.Add(btnBrowseView);
            Controls.Add(btnDesignView);
            Controls.Add(btnEditView);
            Controls.Add(btnCatalogView);
            Controls.Add(btnPersonalizationModeToggle);
        }

        /// <summary>
        /// Renders the buttons to the response stream.
        /// </summary>
        /// <param name="tw">The <see cref="HtmlTextWriter"/> to render controls.</param>
        protected override void RenderContents(HtmlTextWriter tw)
        {
            if (this.Visible && CommunityContext.WebPartsEnabled)
            {
                tw.RenderBeginTag(HtmlTextWriterTag.Tr);

                tw.AddAttribute("class", this.CssClass);

                tw.RenderBeginTag(HtmlTextWriterTag.Td);
                btnBrowseView.RenderControl(tw);
                tw.Write(" | ");
                tw.RenderEndTag();

                tw.AddAttribute("class", this.CssClass);

                tw.RenderBeginTag(HtmlTextWriterTag.Td);
                btnDesignView.RenderControl(tw);
                tw.Write(" | ");
                tw.RenderEndTag();

                tw.AddAttribute("class", this.CssClass);

                tw.RenderBeginTag(HtmlTextWriterTag.Td);
                btnEditView.RenderControl(tw);
                tw.Write(" | ");
                tw.RenderEndTag();

                tw.AddAttribute("class", this.CssClass);

                tw.RenderBeginTag(HtmlTextWriterTag.Td);
                btnCatalogView.RenderControl(tw);
                tw.RenderEndTag();

                tw.AddAttribute("class", this.CssClass);

                tw.RenderBeginTag(HtmlTextWriterTag.Td);
                if (btnPersonalizationModeToggle.Visible)
                    tw.Write(" | ");
                btnPersonalizationModeToggle.RenderControl(tw);
                tw.RenderEndTag();

                tw.RenderEndTag(); // close tr
            }
        }
    }
}
