// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;

using DotCommunities.Framework.Managers;
using DotCommunities.Framework.Components;

namespace DotCommunities.Framework
{
    /// <summary>
    /// Holds user information specific to a community.
    /// </summary>
    [Serializable()]
    public class LocalUser
    {
        int userID;
        string userName;
        bool enableNewsletter = true;
        bool enableNotifications = true;
        int totalPosts;
        int databaseQuota;
        int databaseQuotaUsed;

        bool isDirty = false;

        /// <summary>
        /// The current users local User ID.
        /// </summary>
        public int UserID
        {
            get { return userID; }
        }

        /// <summary>
        /// The current users user name.
        /// </summary>
        public string UserName
        {
            get { return userName; }
        }

        /// <summary>
        /// The users newsletter preference for a community.
        /// </summary>
        public bool EnableNewsletter
        {
            get { return enableNewsletter; }
            set 
            {
                if (enableNewsletter != value)
                {
                    enableNewsletter = value;
                    isDirty = true;
                }
            }
        }

        /// <summary>
        /// The users notification preferences for a community.
        /// </summary>
        public bool EnableNotifications
        {
            get { return enableNotifications; }
            set
            {
                if (enableNotifications != value)
                {
                    enableNotifications = value;
                    isDirty = true;
                }
            }
        }

        /// <summary>
        /// The users total posts for a community.
        /// </summary>
        public int TotalPosts
        {
            get { return totalPosts; }
        }

        /// <summary>
        /// The users database quota for a community.
        /// </summary>
        public int DatabaseQuota
        {
            get { return databaseQuota; }
            set
            {
                if (databaseQuota != value)
                {
                    databaseQuota = value;
                    isDirty = true;
                }
            }
        }

        /// <summary>
        /// The users database quota used for a community.
        /// </summary>
        public int DatabaseQuotaUsed
        {
            get { return databaseQuotaUsed; }
        }

        /// <summary>
        /// Creates a new <see cref="LocalUser"/> instance with default values.
        /// </summary>
        /// <param name="userName">The user to create local data for.</param>
        public LocalUser(string userName)
        {
            this.userID = -1;
            this.userName = userName;
            this.databaseQuota = CommunityContext.CurrentCommunitySettings.UserDatabaseQuota;
            this.isDirty = true;
        }

        /// <summary>
        /// Creates a new <see cref="LocalUser"/> instance from a data row.
        /// </summary>
        /// <param name="reader">The <see cref="IDataReader"/> containing local data.</param>
        public LocalUser(IDataReader reader)
        {
            userID = (int)reader["User_ID"];
            userName = reader["User_UserName"].ToString();
            enableNewsletter = (bool)reader["User_EnableNewsletter"];
            enableNotifications = (bool)reader["User_EnableNotifications"];
            totalPosts = (int)reader["User_TotalPosts"];
            databaseQuota = (int)reader["User_DatabaseQuota"];
            databaseQuotaUsed = (int)reader["User_DatabaseQuotaUsed"];
        }

        /// <summary>
        /// Updates the <see cref="LocalData"/> objects information in the database (if changed).
        /// </summary>
        public void Save()
        {
            if (isDirty)
            {
                FrameworkManager.UpdateUser(userName, enableNewsletter, enableNotifications, databaseQuota);
                isDirty = false;
            }
        }
    }
}
