// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web;
using System.Data;
using System.Collections.Generic;

using DotCommunities.Framework.Properties;
using DotCommunities.Framework.Components;
using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Manages category data for the <see cref="T:FrameworkProvider"/>.
    /// </summary>
    public partial class FrameworkManager : FrameworkManagerBase
    {
        /// <summary>
        /// Removes category objects from the cache.
        /// </summary>
        private static void ClearCategories()
        {
            RemoveCachedItem(string.Format(Resources.CategoriesCacheKey, CommunityContext.CurrentCommunity.ID));
            RemoveCachedItem(string.Format(Resources.CategoryListCacheKey, CommunityContext.CurrentCommunity.ID));
        }

        /// <summary>
        /// Parses a list of categories into a string of Category IDs for saving to the database.
        /// </summary>
        /// <param name="categories">A dictionary list of categories to parse.</param>
        /// <returns>A formatted string of category IDs.</returns>
        public static string ParseCategoryIDs(Dictionary<int, Category> categories)
        {
            string s = string.Empty;
            // Loop through the keys and add them to the string
            foreach (int id in categories.Keys)
                s += string.Format("{0}|", id.ToString());
            // Trim the trailing '|' from the string
            return s.TrimEnd('|');
        }

        /// <summary>
        /// Retrieves the most popular categories from the database and caches them for the duration specified.
        /// </summary>
        /// <param name="cacheDuration">The duration to hold the categories in the cache for.</param>
        /// <returns>A </returns>
        public static List<object> GetLatestCategories(int cacheDuration)
        {
            string cacheKey = string.Format(Resources.LatestCategoriesCacheKey, CommunityContext.CurrentCommunity.ID);

            object item = GetCachedItem(cacheKey);

            if (item != null && item is List<object>)
                return (List<object>)item;

            return (List<object>)AddCachedItem(cacheKey, Provider.GetLatestCategories(CommunityContext.CurrentCommunity.ID), cacheDuration, false);
        }

        /// <summary>
        /// Gets the categories from the cache or the database if not cached.
        /// </summary>
        /// <returns>A generic dictionary containing <see cref="T:Category"/> objects.</returns>
        public static Dictionary<int, Category> GetCategories()
        {
            string cacheKey = string.Format(Resources.CategoriesCacheKey, CommunityContext.CurrentCommunity.ID);

            object item = GetCachedItem(cacheKey);

            if (item != null && item is Dictionary<int, Category>)
                return (Dictionary<int, Category>)item;

            return (Dictionary<int, Category>)AddCachedItem(cacheKey, Provider.GetCategories(CommunityContext.CurrentCommunity.ID));
        }

        /// <summary>
        /// Gets the categories as a <see cref="T:DataTable"/>.
        /// </summary>
        /// <returns>A <see cref="T:DataTable"/> of category data.</returns>
        public static DataSet GetCategoryList()
        {
            string cacheKey = string.Format(Resources.CategoryListCacheKey, CommunityContext.CurrentCommunity.ID);

            object item = GetCachedItem(cacheKey);

            if (item != null && item is DataSet)
                return (DataSet)item;

            return (DataSet)AddCachedItem(cacheKey, Provider.GetCategoryList(CommunityContext.CurrentCommunity.ID));
        }

        /// <summary>
        /// Retrieves a <see cref="T:Category"/> from the cache or the database.
        /// </summary>
        /// <param name="categoryName">The name of the category to retrieve.</param>
        /// <returns>The named <see cref="T:Category"/> or <c>null</c> if not found.</returns>
        public static Category GetCategory(string categoryName)
        {
            foreach (Category c in GetCategories().Values)
                if (c.Name.ToLowerInvariant() == categoryName.ToLowerInvariant())
                    return c;
            return null;
        }

        /// <summary>
        /// Retrieves a <see cref="T:Category"/> from the cache or the database.
        /// </summary>
        /// <param name="categoryID">The ID of the category to retrieve.</param>
        /// <returns>The named <see cref="T:Category"/> or <c>null</c> if not found.</returns>
        public static Category GetCategory(int categoryID)
        {
            Dictionary<int, Category> categories = GetCategories();
            if (categories.ContainsKey(categoryID))
                return categories[categoryID];
            return null;
        }

        /// <summary>
        /// Returns a generic list of content items assigned to the category.
        /// </summary>
        /// <param name="categoryID">The ID of the category to retrieve content for.</param>
        /// <param name="recordsPerPage">The number of records to display per page.</param>
        /// <param name="pageIndex">The current page index.</param>
        /// <param name="sortOrder">The sort order of the content items.</param>
        /// <returns>The paged collection of content items.</returns>
        public static List<object> GetCategoryContent(int categoryID, int recordsPerPage, int pageIndex, string sortOrder)
        {
            return Provider.GetCategoryContent(CommunityContext.CurrentCommunity.ID, categoryID, CommunityContext.CurrentUser.UserName, recordsPerPage, pageIndex, sortOrder);
        }

        /// <summary>
        /// Retrieves the count of total content items for a specified category.
        /// </summary>
        /// <param name="categoryID">The ID of the category to count.</param>
        /// <returns>The total number of content items.</returns>
        public static int GetTotalCategoryRecords(int categoryID)
        {
            return Provider.GetTotalCategoryRecords(CommunityContext.CurrentCommunity.ID, categoryID);
        }

        /// <summary>
        /// Creates a new category.
        /// </summary>
        /// <param name="sectionID">The section ID to associate the category with.</param>
        /// <param name="name">The name of the category.</param>
        /// <param name="description">A description of the category.</param>
        /// <param name="categoryImage">A category image.</param>
        /// <returns>The created category ID.</returns>
        public static int AddCategory(int sectionID, string name, string description, HttpPostedFile categoryImage)
        {
            try
            {
                // Add Category image
                string imageFileName = AddCommunityImage(ImageType.Category, categoryImage);
                // Check that a section is selected
                int? id = null;
                if (sectionID > 0)
                    id = sectionID;
                // Add the category
                return Provider.AddCategory(CommunityContext.CurrentCommunity.ID, id, name, imageFileName, description);
            }
            finally
            {
                ClearCategories();
            }
        }

        /// <summary>
        /// Edits an existing category.
        /// </summary>
        /// <param name="categoryID">The category ID to edit.</param>
        /// <param name="sectionID">The section ID to associate the category with.</param>
        /// <param name="name">The name of the category.</param>
        /// <param name="description">A description of the category.</param>
        /// <param name="categoryImage">A category image.</param>
        /// <param name="existingImageName">The name of the category image if it exists.</param>
        public static void EditCategory(int categoryID, int sectionID, string name, string description, HttpPostedFile categoryImage, string existingImageName)
        {
            try
            {
                // Delete any existing images
                if (!string.IsNullOrEmpty(existingImageName))
                    DeleteCommunityImage(existingImageName);
                // Add any new images to the database
                string imageFileName = AddCommunityImage(ImageType.Category, categoryImage);
                // Check that a section is selected
                int? id = null;
                if (sectionID > 0)
                    id = sectionID;
                // Edit the category
                Provider.EditCategory(CommunityContext.CurrentCommunity.ID, categoryID, id, name, imageFileName, description);
            }
            finally
            {
                ClearCategories();
            }
        }

        /// <summary>
        /// Removes a category and its associated links.
        /// </summary>
        /// <param name="category_ID">The ID of the category to remove.</param>
        public static void DeleteCategory(int category_ID)
        {
            try
            {
                // Delete the associated image
                Category c = GetCategory(category_ID);
                if (c != null && (!string.IsNullOrEmpty(c.ImagePath)))
                    DeleteCommunityImage(c.ImagePath);
                // Delete the category
                Provider.DeleteCategory(CommunityContext.CurrentCommunity.ID, category_ID);
            }
            finally
            {
                ClearCategories();
            }
        }
    }
}
