// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;

using DotCommunities.Framework.Collections;
using DotCommunities.Framework.Properties;
using DotCommunities.Framework.Components;
using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Manages comment data for the <see cref="T:DotCommunities.Framework.Providers.FrameworkProvider"/>.
    /// </summary>
    public partial class FrameworkManager : FrameworkManagerBase
    {
        /// <summary>
        /// Clears comment data from the cache.
        /// </summary>
        /// <param name="contentPageID">The current content page ID to clear.</param>
        private static void ClearComments(int contentPageID)
        {
            // Remove the single comment from the cache
            RemoveCachedItem(string.Format(Resources.CommentCacheKey, contentPageID));

            // Remove the comments from the cache
            RemoveCachedItemByPattern(string.Format(Resources.CommentsCacheRemovalPattern, contentPageID));
        }

        /// <summary>
        /// Gets a single comment.
        /// </summary>
        /// <param name="contentPageID">The current content page ID.</param>
        /// <param name="userName">The current username.</param>
        /// <returns>A <see cref="Comment"/> object containing the comment data.</returns>
        public static Comment GetComment(int contentPageID, string userName)
        {
            string cacheKey = string.Format(Resources.CommentCacheKey, contentPageID);

            object item = GetCachedItem(cacheKey);

            if (item != null)
                return (Comment)item;

            return (Comment)AddCachedItemFor60Seconds(cacheKey, Provider.GetComment(CommunityContext.CurrentCommunity.ID, contentPageID, userName));
        }

        /// <summary>
        /// Gets the comments for a content page.
        /// </summary>
        /// <param name="contentPageID">The current content page ID.</param>
        /// <param name="userName">The current username.</param>
        /// <param name="orderBy">The order in which to sort comments.</param>
        /// <returns>A <see cref="CommentCollection"/> object containing comment data.</returns>
        public static CommentCollection GetComments(int contentPageID, string userName, int orderBy)
        {
            string cacheKey = string.Format(Resources.CommentsCacheKey, contentPageID, orderBy);

            object item = GetCachedItem(cacheKey);

            if (item != null)
                return BuildCommentCollection((DataSet)item);

            // Configure the DataSet and add it to the cache
            item = AddCachedItemFor60Seconds(cacheKey, Provider.GetComments(CommunityContext.CurrentCommunity.ID, contentPageID, userName, orderBy));

            return BuildCommentCollection((DataSet)item);
        }

        /// <summary>
        /// Builds a <see cref="CommentCollection"/> from data in a <see cref="DataSet"/>.
        /// </summary>
        /// <param name="comments">The comment data set.</param>
        /// <returns>The comment collection.</returns>
        private static CommentCollection BuildCommentCollection(DataSet comments)
        {
            CommentCollection commentCollection = new CommentCollection();

            foreach (DataRow dr in comments.Tables[0].Rows)
                commentCollection.Add(new Comment(dr));

            return commentCollection;
        }

        /// <summary>
        /// Adds a new comment to the database.
        /// </summary>
        /// <param name="contentPageID">The content page ID for all comments.</param>
        /// <param name="parentCommentID">The parent content page ID to add a comment to.</param>
        /// <param name="moderationStatus">The <see cref="ModerationStatus"/> of the comment.</param>
        /// <param name="userName">The comment authors user name.</param>
        /// <param name="title">The comment title.</param>
        /// <param name="body">The comment body text.</param>
        /// <returns>The content page ID for the created comment.</returns>
        public static int AddComment(int contentPageID, int parentCommentID, ModerationStatus moderationStatus, string userName, string title, string body)
        {
            string metaDescription = CommunityContext.ParseMetaDescription(body);
            string metaKeys = CommunityContext.ParseMetaKeys(body);

            // Set the parent ID to the content page ID if it is not specified
            if (parentCommentID <= 0)
                parentCommentID = contentPageID;

            try
            {
                return Provider.AddComment(CommunityContext.CurrentCommunity.ID, parentCommentID, userName, title, metaDescription, metaKeys, (int)moderationStatus, body);
            }
            finally
            {
                ClearComments(contentPageID);
            }
        }

        /// <summary>
        /// Edits an existing comment in the database.
        /// </summary>
        /// <param name="contentPageID">The content page ID for all comments.</param>
        /// <param name="commentPageID">The content page ID of the comment to edit.</param>
        /// <param name="userName">The comment authors user name.</param>
        /// <param name="title">The comment title.</param>
        /// <param name="body">The comment body text.</param>
        public static void EditComment(int contentPageID, int commentPageID, string userName, string title, string metaDescription, string metaKeys, string body)
        {
            try
            {
                Provider.EditComment(CommunityContext.CurrentCommunity.ID, commentPageID, userName, title, metaDescription, metaKeys, body);
            }
            finally
            {
                ClearComments(contentPageID);
            }
        }
    }
}
