// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;

using DotCommunities.Framework.Providers;
using DotCommunities.Framework.Components;
using DotCommunities.Framework.Properties;
using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Manages community data for the <see cref="FrameworkProvider"/>.
    /// </summary>
    public partial class FrameworkManager : FrameworkManagerBase
    {
        /// <summary>
        /// Retrieves all the communities from the cache or the database if not found in the cache.
        /// </summary>
        /// <returns>A <see cref="DataSet"/> containing the communities table.</returns>
        public static DataSet GetAllCommunities()
        {
            object item = GetCachedItem(Resources.CommunitiesCacheKey);

            if (item != null)
                return (DataSet)item;

            // Delegate to the provider
            return (DataSet)AddCachedItem(Resources.CommunitiesCacheKey, Provider.GetAllCommunities());
        }

        /// <summary>
        ///  Returns a list of all communities, sorted by the sort order. 
        /// </summary>
        /// <returns>A <see cref="DataView"/> of sorted communities.</returns>
        public static DataView GetAllSortedCommunities()
        {
            return GetAllSortedCommunities("Community_SortOrder", true);
        }

        /// <summary>
        ///  Returns a list of all communities, sorted by the sort order. 
        /// </summary>
        /// <param name="sortExpression">The sort expression.</param>
        /// <param name="isAscending">The sort direction.</param>
        /// <returns>A <see cref="DataView"/> of sorted communities.</returns>
        public static DataView GetAllSortedCommunities(string sortExpression, bool isAscending)
        {
            DataView communities = GetAllCommunities().Tables[0].DefaultView;
            communities.Sort = sortExpression + (isAscending ? " ASC" : " DESC");
            return communities;
        }

        /// <summary>
        ///  Returns the current community by examining the subdomain, domain and IP. 
        /// </summary>
        /// <returns>The current <see cref="Community"/> object.</returns>
        public static Community GetCommunity()
        {
            // Calculate Domain and Subdomain
            string domain;
            string subDomain = "*";

            string host = CommunityContext.Host.ToLower();

            string[] segs = host.Split('.');

            switch (segs.Length)
            {
                case 2:
                    domain = segs[0] + "." + segs[1];
                    break;
                case 3:
                    domain = segs[1] + "." + segs[2];
                    subDomain = segs[0];
                    break;
                default:
                    domain = host;
                    break;
            }

            // TODO Community override & app path should be optional

            //subDomain = CommunityGlobals.AppPath;

            //// If an override was provided then use it
            //if (App.CommunityOverride != "")
            //    subDomain = App.CommunityOverride;


            // Retrieve IP
            string ip = CommunityContext.ServerIPAddress;

            // Find matching communities
            string strMatch = String.Format("(community_domain='{0}' OR community_domain='*') AND (community_subDomain='{1}' OR community_subDomain='*')", domain, subDomain);
            DataRow[] drowMatches = GetAllCommunities().Tables[0].Select(strMatch);
            if (drowMatches.Length == 0)
                return null;

            if (drowMatches.Length == 1)
                return new Community(drowMatches[0]);

            if ((string)drowMatches[0]["Community_subDomain"] != "*")
                return new Community(drowMatches[0]);
            else
                return new Community(drowMatches[1]);
        }

        /// <summary>
        /// Return a specified community.
        /// </summary>
        /// <param name="communityID">ID of community to return.</param>
        /// <returns>The <see cref="Community"/> object for the specified community.</returns>
        public static Community GetCommunity(int communityID)
        {
            DataRow[] matches = GetAllCommunities().Tables[0].Select("Community_ID=" + communityID.ToString());
            return new Community(matches[0]);
        }

        /// <summary>
        /// Adds a new community. Also creates the administrator account
        /// for that community.
        /// </summary>
        /// <param name="name">The name of the new community.</param>
        /// <param name="domainName">The domain name of the new community.</param>
        /// <param name="domain">The domain of the new community.</param>
        /// <param name="subDomain">The sub domain of the new community.</param>
        /// <param name="applicationPath">The application path of the new community.</param>
        /// <param name="databaseQuota">The database quota of the new community.</param>
        /// <param name="isEnabled">Is the community enabled.</param>
        /// <param name="userName">The administrator user name for the new community.</param>
        /// <param name="password">The administrator password for the new community.</param>
        /// <returns>The new community ID.</returns>
        public static int AddCommunity(
                            string name,
                            string domainName,
                            string domain,
                            string subDomain,
                            string applicationPath,
                            int databaseQuota,
                            bool isEnabled,
                            string userName,
                            string password)
        {
            int communityID = -1;
            
            try
            {
                communityID = Provider.AddCommunity(
                                    name,
                                    domainName,
                                    domain,
                                    subDomain,
                                    applicationPath,
                                    databaseQuota,
                                    isEnabled);


            }
            finally
            {
                ClearCommunityCache();
            }

            return communityID;
        }

        /// <summary>
        /// Edits an existing community.
        /// </summary>
        /// <param name="communityID">The ID of the community to edit.</param>
        /// <param name="name">The name of the community.</param>
        /// <param name="domainName">The domain name of the community.</param>
        /// <param name="domain">The domain of the community.</param>
        /// <param name="subDomain">The sub domain of the community.</param>
        /// <param name="applicationPath">The application path of the community.</param>
        /// <param name="databaseQuota">The database quota of the community.</param>
        /// <param name="isEnabled">Is the community enabled.</param>
        public static void EditCommunity(
                            int communityID,
                            string name,
                            string domainName,
                            string domain,
                            string subDomain,
                            string applicationPath,
                            int databaseQuota,
                            bool isEnabled)
        {
            try
            {
                Provider.EditCommunity(
                            communityID,
                            name,
                            domainName,
                            domain,
                            subDomain,
                            applicationPath,
                            databaseQuota,
                            isEnabled);
            }
            finally
            {
                ClearCommunityCache();
            }
        }

        /// <summary>
        /// Saves the community settings information as an XML file serialised
        /// from the appropriate <see cref="CommunitySettings"/> object.
        /// </summary>
        /// <param name="typeName">The name of the type of <see cref="CommunitySettings"/> object.</param>
        /// <param name="xml">The serialised XML of the <see cref="CommunitySettings"/> object.</param>
        public static void UpdateCommunitySettings(int communityID, string typeName, string xml)
        {
            try
            {
                Provider.UpdateCommunitySettings(
                            communityID,
                            typeName,
                            xml);
            }
            finally
            {
                ClearCommunityCache();
            }
        }

        /// <summary>
        /// Moves the sort order of the community up.
        /// </summary>
        /// <param name="communityID">The ID of the community to be sorted.</param>
        public static void MoveCommunityUp(int communityID)
        {
            try
            {
                Provider.MoveCommunityUp(communityID);
            }
            finally
            {
                ClearCommunityCache();
            }
        }

        /// <summary>
        /// Moves the sort order of the community down.
        /// </summary>
        /// <param name="communityID">The ID of the community to be sorted.</param>
        public static void MoveCommunityDown(int communityID)
        {
            try
            {
                Provider.MoveCommunityDown(communityID);
            }
            finally
            {
                ClearCommunityCache();
            }
        }

        /// <summary>
        /// Deletes the community from the database.
        /// </summary>
        /// <param name="communityID">The ID of the community to be deleted.</param>
        public static void DeleteCommunity(int communityID)
        {
            try
            {
                Provider.DeleteCommunity(communityID);
            }
            finally
            {
                ClearCommunityCache();
            }
        }

        /// <summary>
        /// Retrieves the <see cref="CommunityStatus"/> from the data store.
        /// </summary>
        /// <returns>The <see cref="CommunityStatus"/> for the current community.</returns>
        public static CommunityStatus GetCommunityStatus()
        {
            return (CommunityStatus)Provider.GetCommunityStatus(CommunityContext.CurrentCommunity.ID);
        }

        /// <summary>
        /// Sets the <see cref="CommunityStatus"/> in the data store.
        /// </summary>
        public static void SetCommunityStatus(CommunityStatus status)
        {
            Provider.SetCommunityStatus(CommunityContext.CurrentCommunity.ID, status);
        }

        /// <summary>
        /// Removes the cached community data from the cache.
        /// </summary>
        static void ClearCommunityCache()
        {
            RemoveCachedItem(Resources.CommunitiesCacheKey);
        }
    }
}
