// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;
using System.Collections.Generic;
using System.IO;

using DotCommunities.Framework.Enums;
using DotCommunities.Framework.Providers;
using DotCommunities.Framework.Properties;
using DotCommunities.Framework.Components;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Manages content page data for the <see cref="FrameworkProvider"/>.
    /// </summary>
    public partial class FrameworkManager : FrameworkManagerBase
    {
        /// <summary>
        /// <see cref="EventHandler"/> used by content pages of different types
        /// to be notified that the content has changed sort order.
        /// </summary>
        public static EventHandler ContentSortOrderChanged;

        /// <summary>
        /// Removed cached content page data.
        /// </summary>
        /// <param name="sectionID">The section to remove data from.</param>
        /// <param name="contentPageID">The optional content page ID to remove data from.</param>
        public static void ClearContentPages(int sectionID, int contentPageID)
        {
            // Remove the content page from the cache
            if (contentPageID > 0 && sectionID > 0)
                RemoveCachedItem(string.Format(Resources.ContentPageCacheKey, sectionID, contentPageID));
            else if (sectionID > 0) // Removes all cached content from the section
                RemoveCachedItemByPattern(string.Format(Resources.ContentPageCacheRemovalPattern, sectionID));
            // Remove the total content pages from the cache
            RemoveCachedItem(string.Format(Resources.SectionContentPageCountCacheKey, sectionID));
        }

        /// <summary>
        /// Returns a <see cref="GenericPage"/> object from a file name. Checks if the
        /// file name is a number, if it is, then a content page is assumed.
        /// </summary>
        /// <param name="requestFile">The page name of the content page (e.g. 1.aspx).</param>
        /// <returns>A <see cref="GenericPage"/> object containing the content page data.</returns>
        public static GenericPage GetContentPageFromPageName(string requestFile)
        {
            int contentID = 0;

            // Attempt to convert request path to integer
            string strContentID = Path.GetFileNameWithoutExtension(requestFile);
            if (!Int32.TryParse(strContentID, out contentID))
                return null;

            // Otherwise, we have a content page
            ContentPage contentPageInfo = GetContentPage(contentID, CommunityContext.CurrentSection.SectionID);

            // Check whether it exists
            if (contentPageInfo == null)
                return null;

            return new GenericPage
               (
                   contentPageInfo.ContentPageID,     // Page ID
                   contentPageInfo.ParentID,          // Page Parent ID
                   PageType.Content,                  // Page Type
                   String.Empty,                      // Page Content Type
                   contentPageInfo.ContentType,       // Page Name
                   contentPageInfo.Title,             // Page Title
                   contentPageInfo.Description,       // Page Description
                   contentPageInfo.MetaDescription,   // Page Meta Description
                   contentPageInfo.MetaKeys           // Page Meta Keys
              );

        }

        /// <summary>
        /// Returns a content page from the database given a particular ID. 
        /// </summary>
        /// <param name="contentID">The content page ID.</param>
        /// <param name="sectionID">The section ID containing the content page.</param>
        /// <returns>A <see cref="ContentPage"/> object containing the content page data.</returns>
        public static ContentPage GetContentPage(int contentID, int sectionID)
        {
            string cacheKey = string.Format(Resources.ContentPageCacheKey, sectionID, contentID);

            object item = GetCachedItem(cacheKey);

            if (item != null)
                return (ContentPage)item;

            return (ContentPage)AddCachedItemFor60Seconds(cacheKey, Provider.GetContentPage(contentID, sectionID));
        }

        /// <summary>
        /// Returns the total number of visible content pages in a 
        /// particular section. 
        /// </summary>
        /// <param name="sectionID">The section ID of the section that requires the content page count.</param>
        /// <returns>The number of content pages contained in a section.</returns>
        public static int GetTotalContentPages(int sectionID)
        {
            string cacheKey = string.Format(Resources.SectionContentPageCountCacheKey, sectionID);

            object item = GetCachedItem(cacheKey);

            if (item != null)
                return (int)item;

            return (int)AddCachedItemFor60Seconds(cacheKey, Provider.GetTotalContentPages(sectionID));
        }

        /// <summary>
        /// Retrieves the latest content added to this community.
        /// </summary>
        /// <returns>A list of new content.</returns>
        public static List<object> GetNewContent(int count)
        {
            return GetNewContent(string.Empty, count);
        }

        /// <summary>
        /// Retrieves the latest content added to this community
        /// of a particular type.
        /// </summary>
        /// <param name="pageTypeName">The type of page content we are after.</param>
        /// <returns>A list of new content for the selected page type.</returns>
        public static List<object> GetNewContent(string pageTypeName, int count)
        {
            string cacheKey = string.Format(Resources.NewContentCacheKey, CommunityContext.CurrentCommunity.ID, pageTypeName);

            List<object> content = (List<object>)GetCachedItem(cacheKey);

            if (content != null && content is List<object>)
                return (List<object>)content;

            content = new List<object>();

            DataSet newContent = Provider.GetNewContent(CommunityContext.CurrentCommunity.ID, pageTypeName, count);

            if (newContent != null && newContent.Tables.Count > 0)
                foreach (DataRow dr in newContent.Tables[0].Rows)
                    content.Add(new CommunityContent(dr));

            return (List<object>)AddCachedItem(cacheKey, content, 60, true);
        }

        /// <summary>
        /// Retrieves the most viewed content added to this community.
        /// </summary>
        /// <returns>A list of popular content.</returns>
        public static List<object> GetPopularContent(int count)
        {
            return GetPopularContent(string.Empty, count);
        }

        /// <summary>
        /// Retrieves the most viewed content added to this community
        /// of a particular type.
        /// </summary>
        /// <param name="pageTypeName">The type of page content we are after.</param>
        /// <returns>A list of popular content for the selected page type.</returns>
        public static List<object> GetPopularContent(string pageTypeName, int count)
        {
            string cacheKey = string.Format(Resources.PopularContentCacheKey, CommunityContext.CurrentCommunity.ID, pageTypeName);

            List<object> content = (List<object>)GetCachedItem(cacheKey);

            if (content != null && content is List<object>)
                return (List<object>)content;

            content = new List<object>();

            DataSet popularContent = Provider.GetPopularContent(CommunityContext.CurrentCommunity.ID, pageTypeName, count);

            if (popularContent != null && popularContent.Tables.Count > 0)
                foreach (DataRow dr in popularContent.Tables[0].Rows)
                    content.Add(new CommunityContent(dr));

            return (List<object>)AddCachedItem(cacheKey, content, 60, true);
        }

        /// <summary>
        /// Used when sorting content to move content up. 
        /// </summary>
        /// <param name="contentPageID">The content page ID of the content to move.</param>
        public static void MoveContentUp(int contentPageID)
        {
            try
            {
                Provider.MoveContentUp(contentPageID);
            }
            finally
            {
                // Fire the sort order changed event handler
                if (ContentSortOrderChanged != null)
                    ContentSortOrderChanged(CommunityContext.CurrentSection, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Used when sorting content to move content down. 
        /// </summary>
        /// <param name="contentPageID">The content page ID of the content to move.</param>
        public static void MoveContentDown(int contentPageID)
        {
            try
            {
                Provider.MoveContentDown(contentPageID);
            }
            finally
            {
                // Fire the sort order changed event handler
                if (ContentSortOrderChanged != null)
                    ContentSortOrderChanged(CommunityContext.CurrentSection, EventArgs.Empty);
            }
        }
    }
}
