// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web.Configuration;
using System.Collections.Generic;

using DotCommunities.Framework.Enums;
using DotCommunities.Framework.Components;
using DotCommunities.Framework.Configuration;
using DotCommunities.Framework.Properties;
using DotCommunities.Framework.Collections;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Contains static methods for working with content types like named pages and modules.
    /// </summary>
    public class ContentTypesManager : CommunityManagerBase
    {
        /// <summary>
        /// Gets the <see cref="GenericPageCollection"/> from the cache or loads them from the web
        /// configuration file.
        /// </summary>
        /// <returns>A <see cref="GenericPageCollection"/> containing named page data.</returns>
        public static GenericPageCollection GetNamedPages()
        {
            object namedPages = GetCachedItem(Resources.NamedPagesCacheKey);

            if (namedPages != null && namedPages is GenericPageCollection)
                return (GenericPageCollection)namedPages;

            return (GenericPageCollection)AddCachedItem(Resources.NamedPagesCacheKey, ConfigureNamedPages(), 60, true);
        }

        /// <summary>
        /// Gets the <see cref="GenericPageCollection"/> from the collection stored
        /// in the web configuration file.
        /// </summary>
        /// <returns>A <see cref="GenericPageCollection"/> containing named page data.</returns>
        private static GenericPageCollection ConfigureNamedPages()
        {
            ContentTypesSection contentTypes = GetContentTypesSection();

            if (contentTypes != null)
            {
                GenericPageCollection pages = new GenericPageCollection();

                foreach (NamedPageElement page in contentTypes.Pages)
                    pages.Add(page.Path, GetGenericPageFromNamedPage(page));

                return pages;
            }

            return null;
        }

        /// <summary>
        /// Gets the <see cref="GenericPageCollection"/> from the cache or loads them from the web
        /// configuration file.
        /// </summary>
        /// <returns>A <see cref="GenericPageCollection"/> containing module page data.</returns>
        public static GenericPageCollection GetModulePages()
        {
            object modulePages = GetCachedItem(Resources.ModulePagesCacheKey);

            if (modulePages != null && modulePages is GenericPageCollection)
                return (GenericPageCollection)modulePages;

            return (GenericPageCollection)AddCachedItem(Resources.ModulePagesCacheKey, ConfigureModulePages(), 60, true);
        }

        /// <summary>
        /// Gets the <see cref="GenericPageCollection"/> from the collection stored
        /// in the web configuration file.
        /// </summary>
        /// <returns>A <see cref="GenericPageCollection"/> containing module page data.</returns>
        private static GenericPageCollection ConfigureModulePages()
        {
            ContentTypesSection contentTypes = GetContentTypesSection();
                
            if (contentTypes != null)
            {
                GenericPageCollection modules = new GenericPageCollection();

                foreach (ModulePageElement module in contentTypes.Modules)
                    modules.Add(module.Name, GetGenericPageFromModulePage(module));

                return modules;
            }

            return null;
        }

        /// <summary>
        /// Gets the <see cref="GenericPageCollection"/> from the collection stored
        /// in the web configuration file and filtered by section type.
        /// </summary>
        /// <returns>A <see cref="GenericPageCollection"/> containing module page data.</returns>
        public static GenericPageCollection GetModuleSections()
        {
            return GetModuleSections(false);
        }

        /// <summary>
        /// Gets the <see cref="GenericPageCollection"/> from the collection stored
        /// in the web configuration file and filtered by section type.
        /// </summary>
        /// <param name="parentsOnly">Value indicating if only parent sections are returned.</param>
        /// <returns>A <see cref="GenericPageCollection"/> containing module page data.</returns>
        public static GenericPageCollection GetModuleSections(bool parentsOnly)
        {
            ContentTypesSection contentTypes = GetContentTypesSection();

            if (contentTypes != null)
            {
                GenericPageCollection modules = new GenericPageCollection();

                foreach (ModulePageElement module in contentTypes.Modules)
                    if ((module.IsSection && (!parentsOnly)) || (module.IsSection && parentsOnly && module.IsParent))
                        modules.Add(module.Name, GetGenericPageFromModulePage(module));

                return modules;
            }

            return null;
        }

        /// <summary>
        /// Gets the <see cref="GenericHandlerCollection"/> from the cache or loads them from the web
        /// configuration file.
        /// </summary>
        /// <returns>A <see cref="GenericHandlerCollection"/> containing handler page data.</returns>
        public static GenericHandlerCollection GetGenericHandlers()
        {
            object genericHandlers = GetCachedItem(Resources.GenericHandlersCacheKey);

            if (genericHandlers != null && genericHandlers is GenericHandlerCollection)
                return (GenericHandlerCollection)genericHandlers;

            return (GenericHandlerCollection)AddCachedItem(Resources.GenericHandlersCacheKey, ConfigureGenericHandlers(), 60, true);
        }

        /// <summary>
        /// Gets the <see cref="GenericHandlerCollection"/> from the collection stored
        /// in the web configuration file.
        /// </summary>
        /// <returns>A <see cref="GenericHandlerCollection"/> containing handler page data.</returns>
        private static GenericHandlerCollection ConfigureGenericHandlers()
        {
            ContentTypesSection contentTypes = GetContentTypesSection();

            if (contentTypes != null)
            {
                GenericHandlerCollection handlers = new GenericHandlerCollection();

                foreach (GenericHandlerElement handler in contentTypes.GenericHandlers)
                    handlers.Add(handler.Name, new GenericHandler(handler.Name, handler.Control, handler.ExtensionNames));

                return handlers;
            }

            return null;
        }

        /// <summary>
        /// Gets the <see cref="ContentTypesSection"/> from the web configuration file.
        /// </summary>
        /// <returns>The <see cref="ContentTypesSection"/> object.</returns>
        protected static ContentTypesSection GetContentTypesSection()
        {
            return (ContentTypesSection)WebConfigurationManager.GetSection(Resources.CommunityConfigurationGroupName + Resources.ContentTypesSectionName);
        }

        /// <summary>
        /// Gets the page information from a <see cref="NamedPageElement"/>.
        /// </summary>
        /// <param name="page">The <see cref="NamedPageElement"/> to extract information from.</param>
        /// <returns>A <see cref="GenericPage"/> containing data from the <see cref="NamedPageElement"/>.</returns>
        private static GenericPage GetGenericPageFromNamedPage(NamedPageElement page)
        {
            return new GenericPage
            (
                -1,               // Page ID
                -1,               // Page Parent ID
                PageType.Named,   // Page is a named page type
                page.TypeName,    // Page Type Name
                page.Name,        // Page Name
                page.Title,       // Page Title
                page.Description, // Page Description
                String.Empty,     // Page Meta Description
                String.Empty      // Page Meta Keys
            );
        }

        private static GenericPage GetGenericPageFromModulePage(ModulePageElement module)
        {
            PageType pageType = PageType.Content;

            if (module.IsSection && module.IsParent)
                pageType = PageType.ParentSection;
            else if (module.IsSection)
                pageType = PageType.Section;
            else if (module.IsParent)
                pageType = PageType.Parent;

            return new GenericPage
            (
                -1,                 // Module ID
                -1,                 // Module Parent ID
                pageType,           // Module Type
                module.TypeName,    // Module Type Name
                module.Name,        // Module Name
                module.Title,       // Module Title
                module.Description, // Module Description
                String.Empty,       // Module Meta Description
                String.Empty        // Module Meta Keys
            );
        }
    }
}
