// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using DotCommunities.Framework.Properties;
using DotCommunities.Framework.Components;
using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Manages content page data for the <see cref="FrameworkProvider"/>.
    /// </summary>
    public partial class FrameworkManager : FrameworkManagerBase
    {
        /// <summary>
        /// Gets the section image, performing an image resize if required.
        /// </summary>
        /// <param name="sectionID">The section ID.</param>
        /// <param name="imageID">The image ID.</param>
        /// <param name="width">The required image width.</param>
        /// <param name="height">The required image height.</param>
        /// <param name="isThumbnail">if set to <c>true</c>, image is returned as a thumbnail.</param>
        /// <returns>An <see cref="ImageData"/> object containing the image.</returns>
        public static ImageData GetSectionImage(int sectionID, int imageID, int width, int height, bool isThumbnail)
        {
            // Build the image cache key
            string cacheKey = string.Format(Resources.SectionImageCacheKey, CommunityContext.CurrentCommunity.ID, sectionID, imageID, width, height, isThumbnail);

            // Gets the image from the cache (if cached)
            object item = GetCachedItem(cacheKey);
            if (item != null && item is ImageData)
                return (ImageData)item;

            // Get the image from the database
            ImageData image = Provider.GetSectionImage(CommunityContext.CurrentCommunity.ID, sectionID, imageID, width, height, isThumbnail);

            // if no resize, just return after caching
            if (image.Width == width && image.Height == height && image.IsThumbnail == isThumbnail)
                return (ImageData)AddCachedItemFor60Seconds(cacheKey, image);

            // Perform resize
            image.Image = ModifyImage(image.Original, width, height, image.ContentType, isThumbnail);

            // Save resized image
            Provider.SaveSizedSectionImage(CommunityContext.CurrentCommunity.ID, imageID, image);

            // Cache image data
            return (ImageData)AddCachedItemFor60Seconds(cacheKey, image);
        }

        /// <summary>
        /// Removes the image data from the cache.
        /// </summary>
        /// <param name="fileName">A filename if a specific cached image is removed.</param>
        private static void ClearCommunityImages(string fileName)
        {
            // Build the images cache key
            string cacheKey = string.Format(Resources.CommunityImagesCacheKey, CommunityContext.CurrentCommunity.ID);
            // Remove the images from the cache
            RemoveCachedItem(cacheKey);

            // Check we have a filename
            if (!string.IsNullOrEmpty(fileName))
            {
                // Build the image cache key
                cacheKey = string.Format(Resources.CommunityImageCacheRemovalPattern, CommunityContext.CurrentCommunity.ID, fileName);
                // Remove the image from the cache
                RemoveCachedItemByPattern(cacheKey);
            }
        }

        /// <summary>
        /// Gets a community image from the cache or database.
        /// </summary>
        /// <param name="fileName">The image file name.</param>
        /// <param name="targetSize">The required image size.</param>
        /// <returns>An <see cref="ImageData"/> object containing the image.</returns>
        public static ImageData GetCommunityImage(string fileName, int targetSize)
        {
            // Build the image cache key
            string cacheKey = string.Format(Resources.CommunityImageCacheKey, CommunityContext.CurrentCommunity.ID, fileName, targetSize);

            // Gets the image from the cache (if cached)
            object item = GetCachedItem(cacheKey);
            if (item != null && item is ImageData)
                return (ImageData)item;

            // Get the image from the database
            ImageData image = Provider.GetCommunityImage(CommunityContext.CurrentCommunity.ID, fileName);

            // See if we need to perform a resize of the image
            if (targetSize > 0)
                image.Image = ResizeImage(image.Original, targetSize, image.ContentType);
            if (image.Image != image.Original)
                image.ContentType = "image/jpeg";

            // Cache image data
            return (ImageData)AddCachedItemFor60Seconds(cacheKey, image);
        }

        /// <summary>
        /// Retrieves all the community images from the cache or database.
        /// </summary>
        /// <returns>A <see cref="DataSet"/> containing community images.</returns>
        public static DataSet GetCommunityImages()
        {
            // Build the images cache key
            string cacheKey = string.Format(Resources.CommunityImagesCacheKey, CommunityContext.CurrentCommunity.ID);

            // Gets the images from the cache (if cached)
            object item = GetCachedItem(cacheKey);
            if (item != null && item is DataSet)
                return (DataSet)item;

            // Cache images
            return (DataSet)AddCachedItemFor60Seconds(cacheKey, Provider.GetCommunityImages(CommunityContext.CurrentCommunity.ID));
        }

        /// <summary>
        /// Adds a new community image to the database.
        /// </summary>
        /// <param name="imageType">Type of the image.</param>
        /// <param name="proposedFile">The proposed file.</param>
        /// <returns>The image filename.</returns>
        public static string AddCommunityImage(ImageType imageType, HttpPostedFile proposedFile)
        {
            // Make sure that the posted file has content
            if (proposedFile == null || proposedFile.ContentLength == 0)
                return null;

            string imgFileName = proposedFile.FileName.Remove(0, proposedFile.FileName.LastIndexOf("\\") + 1);
            string imgContentType = proposedFile.ContentType;

            if (imgContentType == "image/jpeg")
                imgContentType = "image/pjpeg";

            // Make sure that the posted file is actually an image
            if (!IsImageFile(imgFileName))
                return null;

            // Process the image into byte array
            Stream imgStream = proposedFile.InputStream;
            int imgLen = proposedFile.ContentLength;
            byte[] imgBinaryData = new byte[imgLen];
            int n = imgStream.Read(imgBinaryData, 0, imgLen);

            try
            {
                // Add the image to the database
                Provider.AddCommunityImage(CommunityContext.CurrentCommunity.ID, imgFileName, imgContentType, (int)imageType, imgBinaryData);
            }
            finally
            {
                // Clear the images from the cache
                ClearCommunityImages(imgFileName);
            }

            // Return the image filename
            return imgFileName;
        }

        /// <summary>
        /// Removes the community image from the database.
        /// </summary>
        /// <param name="imageName">The name of the file to delete.</param>
        public static void DeleteCommunityImage(string imageName)
        {
            try
            {
                Provider.DeleteCommunityImage(CommunityContext.CurrentCommunity.ID, imageName);
            }
            finally
            {
                // Clear the images from the cache
                ClearCommunityImages(imageName);
            }
        }

        /// <summary>
        /// Returns <c>true</c> or <c>false</c> depending on whether the filename 
        /// ends with an image file extension. 
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <returns>
        /// 	<c>true</c> if the specified file name is an image file; otherwise, <c>false</c>.
        /// </returns>
        private static bool IsImageFile(string fileName)
        {
            // Convert the filename to lowercase
            fileName = fileName.ToLowerInvariant();
            // Check if the extension is valid
            return (fileName.EndsWith(".gif") || fileName.EndsWith(".jpg") || fileName.EndsWith(".jpeg"));
        } 

        #region Image Manipulation Methods
        /// <summary>
        /// Modifies the size (width or height) of an image.
        /// </summary>
        /// <param name="imageFile">The original byte array of the image.</param>
        /// <param name="targetSize">The target width/height of the image.</param>
        /// <param name="contentType">The type of image (this should be either jpeg or gif).</param>
        /// <returns>A byte array containing the resized image.</returns>
        public static byte[] ResizeImage(byte[] imageFile, int targetSize, string contentType)
        {
            System.Drawing.Image original = System.Drawing.Image.FromStream(new MemoryStream(imageFile));
            try
            {
                int targetH = original.Height;
                int targetW = original.Width;

                if (original.Height <= targetSize && original.Width <= targetSize)
                {
                    // We do not need to perform a resize
                    return imageFile;
                }
                else if (original.Height >= original.Width)
                {
                    targetH = targetSize;
                    targetW = (int)(original.Width * ((float)targetSize / (float)original.Height));
                }
                else if (original.Height <= original.Width)
                {
                    targetW = targetSize;
                    targetH = (int)(original.Height * ((float)targetSize / (float)original.Width));
                }

                System.Drawing.Image imgPhoto = System.Drawing.Image.FromStream(new MemoryStream(imageFile));
                try
                {
                    // Create a new blank canvas.  The resized image will be drawn on this canvas.
                    Bitmap bmPhoto = new Bitmap(targetW, targetH, PixelFormat.Format24bppRgb);
                    try
                    {
                        bmPhoto.SetResolution(72, 72);

                        Graphics grPhoto = Graphics.FromImage(bmPhoto);
                        try
                        {
                            grPhoto.SmoothingMode = SmoothingMode.AntiAlias;
                            grPhoto.InterpolationMode = InterpolationMode.HighQualityBicubic;
                            grPhoto.PixelOffsetMode = PixelOffsetMode.HighQuality;

                            // Fill the background with white
                            grPhoto.FillRectangle(new SolidBrush(Color.White), new Rectangle(0, 0, targetW, targetH));

                            // Draw the resized image
                            grPhoto.DrawImage(imgPhoto, new Rectangle(0, 0, targetW, targetH), 0, 0, original.Width, original.Height, GraphicsUnit.Pixel);

                            // Save out to memory.  We dispose of all objects to make sure the files don't stay locked.
                            MemoryStream mm = new MemoryStream();
                            bmPhoto.Save(mm, System.Drawing.Imaging.ImageFormat.Jpeg);

                            return mm.GetBuffer();
                        }
                        finally
                        {
                            grPhoto.Dispose();
                        }
                    }
                    finally
                    {
                        bmPhoto.Dispose();
                    }
                }
                finally
                {
                    imgPhoto.Dispose();
                }
            }
            finally
            {
                original.Dispose();
            }            
        }

        /// <summary>
        /// Modifies the width or height of an image.
        /// </summary>
        /// <param name="original">The original byte array of the image.</param>
        /// <param name="width">The new image width.</param>
        /// <param name="height">The new image height.</param>
        /// <param name="contentType">The type of image (this should be either jpeg or gif).</param>
        /// <param name="isThumbNail">if set to <c>true</c>, image is redrawn to suit a thumbnail image.</param>
        /// <returns>A byte array containing the modified image.</returns>
        public static byte[] ModifyImage(byte[] original, int width, int height, string contentType, bool isThumbNail)
        {
            MemoryStream stream = null;
            Bitmap bmp = null;
            Bitmap thumbBmp = null;

            try
            {
                // convert byte array to image
                stream = new MemoryStream(original);

                // convert size to new dimensions
                bmp = (Bitmap)System.Drawing.Image.FromStream(stream);

                // Calculate image width or height
                if (width == -1)
                    width = (int)(((double)height / (double)bmp.Height) * (double)bmp.Width);

                if (height == -1)
                    height = (int)(((double)width / (double)bmp.Width) * (double)bmp.Height);

                thumbBmp = new Bitmap(bmp, width, height);
                thumbBmp.Palette = GetTransparentColorPalette(bmp);

                // convert to stream in preparation to convert to byte array
                stream = new MemoryStream();

                // Save the stream - first we need to determine if image is JPG or GIF
                // to determine the proper encoder type
                if (contentType.ToLower().EndsWith("pjpeg"))
                {
                    // Is this a thumbnail?
                    if (isThumbNail)
                        stream = SaveJpgWithCompression(thumbBmp, 40);
                    else
                        thumbBmp.Save(stream, ImageFormat.Jpeg);
                }
                else
                    thumbBmp.Save(stream, ImageFormat.Gif);
            }
            finally
            {
                // Cleanup objects from memory
                if (thumbBmp != null)
                    thumbBmp.Dispose();
                if (bmp != null)
                    bmp.Dispose();
            }

            // Return the image from the stream
            if (stream != null)
                return stream.GetBuffer();

            // Return null if the method failed
            return null;
        }

        /// <summary>
        /// Saves the JPG with compression.
        /// </summary>
        /// <remarks>
        /// JPG's upload by user will likely have some compression
        /// Therefore a value of 40 or more will be needed to see some 
        /// improvements in file size.
        /// </remarks>
        /// <param name="original">The image to compress.</param>
        /// <param name="fileName">The filename to save the JPG to.</param>
        /// <param name="compression">A value from 1 to 100. A value of 100 is no compression. 
        /// A value of 1 is maximum compression.</param>
        private static void SaveJpgWithCompression(System.Drawing.Image original, string fileName, long compression)
        {
            EncoderParameters eps = new EncoderParameters(1);
            eps.Param[0] = new EncoderParameter(Encoder.Quality, compression);
            ImageCodecInfo ici = GetEncoderInfo("image/jpeg");
            original.Save(fileName, ici, eps);
        }

        /// <summary>
        /// Saves the JPG with compression.
        /// </summary>
        /// <remarks>
        /// JPG's upload by user will likely have some compression
        /// Therefore a value of 40 or more will be needed to see some 
        /// improvements in file size.
        /// </remarks>
        /// <param name="original">The image to compress.</param>
        /// <param name="compression">A value from 1 to 100. A value of 100 is no compression. 
        //	A value of 1 is maximum compression.</param>
        /// <returns>A <see cref="MemoryStream"/> containing the compressed image.</returns>
        private static MemoryStream SaveJpgWithCompression(System.Drawing.Image original, long compression)
        {
            MemoryStream ms = new MemoryStream();
            EncoderParameters eps = new EncoderParameters(1);
            eps.Param[0] = new EncoderParameter(Encoder.Quality, compression);

            ImageCodecInfo ici = GetEncoderInfo("image/jpeg");
            original.Save(ms, ici, eps);
            return ms;
        }

        /// <summary>
        /// Finds the transparent colors in a GIF image.
        /// </summary>
        /// <param name="original">The original image.</param>
        /// <returns>A <see cref="ColorPalette"/> containing the transparent colours.</returns>
        private static ColorPalette GetTransparentColorPalette(Bitmap original)
        {
            Color testColor;
            Color newColor;
            ColorPalette pal = original.Palette;

            for (int i = 0; i < pal.Entries.Length - 1; i++)
            {
                testColor = pal.Entries[i];

                if (testColor.A == 0)
                {
                    newColor = Color.FromArgb(0, testColor);
                    pal.Entries[i] = newColor;

                }
            }

            return pal;
        }

        /// <summary>
        /// Retrieves the the Image Codec for a particular image MIME type.
        /// </summary>
        /// <remarks>
        /// From KB article Q324790.
        /// </remarks>
        /// <param name="mimeType">The MIME Type of the encoder.</param>
        /// <returns>An <see cref="ImageCodecInfo"/> object containing the 
        /// codec information for the specified MIME type.</returns>
        private static ImageCodecInfo GetEncoderInfo(String mimeType)
        {
            int j;
            ImageCodecInfo[] encoders;
            encoders = ImageCodecInfo.GetImageEncoders();
            for (j = 0; j < encoders.Length; ++j)
            {
                if (encoders[j].MimeType == mimeType)
                    return encoders[j];
            }
            return null;
        }
        #endregion
    }
}
