// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections.Generic;
using System.Diagnostics;

using DotCommunities.Framework.Enums;

using Microsoft.Practices.EnterpriseLibrary.Logging;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Logs information and exceptions to the Enterprise Library Logging Application Block.
    /// </summary>
    public sealed class LogManager
    {
        /// <summary>
        /// Defines the categories that can be assigned to the log manager.
        /// </summary>
        public enum LogCategory
        {
            /// <summary>
            /// The General category.
            /// </summary>
            General,
            /// <summary>
            /// The Exceptions category.
            /// </summary>
            Exceptions,
            /// <summary>
            /// The Providers category.
            /// </summary>
            Providers,
            /// <summary>
            /// The Tasks category.
            /// </summary>
            Tasks
        }

        /// <summary>
        /// The string representation of the <see cref="LogCategory"/> enumeration.
        /// </summary>
        private static readonly string[] CategoryNames = 
            {
                "General",
                "Exceptions",
                "Providers",
                "Tasks"
            };

        /// <summary>
        /// Creates a list of category strings.
        /// </summary>
        /// <param name="categories">A collection of <see cref="LogCategory"/> objects.</param>
        /// <returns>A list of category strings.</returns>
        private static List<string> GetCategoryNames(ICollection<LogCategory> categories)
        {
            List<string> categoryNames = new List<string>(categories.Count);
            foreach (LogCategory c in categories)
                categoryNames.Add(CategoryNames[(int)c]);
            return categoryNames;
        }

        /// <summary>
        /// Parses a list of categories into a string.
        /// </summary>
        /// <param name="categories">A collection of category strings.</param>
        /// <returns>A formatted string of categories.</returns>
        private static string ParseCategories(ICollection<string> categories)
        {
            string categoryNames = string.Empty;
            foreach (string c in categories)
                categoryNames += string.Format("{0}, ", c);
            categoryNames = categoryNames.Trim();
            categoryNames = categoryNames.TrimEnd(',');
            if (string.IsNullOrEmpty(categoryNames))
                categoryNames = "No Category";
            return categoryNames;
        }

        /// <summary>
        /// Builds a list of <see cref="LogCategory"/> objects.
        /// </summary>
        /// <param name="categories">A list of <see cref="LogCategory"/> paramters.</param>
        /// <returns>A list of <see cref="LogCategory"/> objects.</returns>
        public static List<LogCategory> BuildCategories(params LogCategory[] categories)
        {
            List<LogCategory> logCategories = new List<LogCategory>();
            foreach (LogCategory c in categories)
                logCategories.Add(c);
            return logCategories;
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Information"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        public static void WriteInformation(string title, string message)
        {
            Write(TraceEventType.Information, title, message);
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Information"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        public static void WriteInformation(EventCode eventId, string title, string message)
        {
            Write(eventId, TraceEventType.Information, title, message);
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Information"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="category">The <see cref="LogCategory"/> to which this event belongs.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        public static void WriteInformation(EventCode eventId, LogCategory category, string title, string message)
        {
            Write(eventId, TraceEventType.Information, BuildCategories(category), title, message, false);
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Warning"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        public static void WriteWarning(string title, string message)
        {
            Write(TraceEventType.Warning, title, message);
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Warning"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        public static void WriteWarning(EventCode eventId, string title, string message)
        {
            Write(eventId, TraceEventType.Warning, title, message);
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Warning"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="category">The <see cref="LogCategory"/> to which this event belongs.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        public static void WriteWarning(EventCode eventId, LogCategory category, string title, string message)
        {
            Write(eventId, TraceEventType.Warning, BuildCategories(category), title, message, false);
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Verbose"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        public static void WriteDebug(string title, string message)
        {
            Write(TraceEventType.Verbose, title, message);
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Verbose"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        public static void WriteDebug(string title, string message, bool consoleOnly)
        {
            Write(TraceEventType.Verbose, title, message, consoleOnly);
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Verbose"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        public static void WriteDebug(EventCode eventId, string title, string message)
        {
            Write(eventId, TraceEventType.Verbose, title, message);
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Verbose"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        public static void WriteDebug(EventCode eventId, string title, string message, bool consoleOnly)
        {
            Write(eventId, TraceEventType.Verbose, title, message, consoleOnly);
        }

        /// <summary>
        /// Writes a <see cref="TraceEventType.Verbose"/> event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="category">The <see cref="LogCategory"/> to which this event belongs.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        public static void WriteDebug(EventCode eventId, LogCategory category, string title, string message, bool consoleOnly)
        {
            Write(eventId, TraceEventType.Verbose, BuildCategories(category), title, message, consoleOnly);
        }

        /// <summary>
        /// Writes an event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="severity">The <see cref="TraceEventType"/> of the event.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        public static void Write(TraceEventType severity, string title, string message)
        {
            Write(EventCode.General, severity, BuildCategories(LogCategory.General), title, message, false);
        }

        /// <summary>
        /// Writes an event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="severity">The <see cref="TraceEventType"/> of the event.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        public static void Write(TraceEventType severity, string title, string message, bool consoleOnly)
        {
            Write(EventCode.General, severity, BuildCategories(LogCategory.General), title, message, consoleOnly);
        }

        /// <summary>
        /// Writes an event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="severity">The <see cref="TraceEventType"/> of the event.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        public static void Write(EventCode eventId, TraceEventType severity, string title, string message)
        {
            Write(eventId, severity, BuildCategories(LogCategory.General), title, message, false);
        }

        /// <summary>
        /// Writes an event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="severity">The <see cref="TraceEventType"/> of the event.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        public static void Write(EventCode eventId, TraceEventType severity, string title, string message, bool consoleOnly)
        {
            Write(eventId, severity, BuildCategories(LogCategory.General), title, message, consoleOnly);
        }

        /// <summary>
        /// Writes an event to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="severity">The <see cref="TraceEventType"/> of the event.</param>
        /// <param name="categories">The collection of <see cref="LogCategory"/> categories that the event belongs.</param>
        /// <param name="title">The title of the event.</param>
        /// <param name="message">The description of the event.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        public static void Write(EventCode eventId, TraceEventType severity, ICollection<LogCategory> categories, string title, string message, bool consoleOnly)
        {
            LogEntry entry = new LogEntry();

            entry.EventId = (int)eventId;
            entry.Severity = severity;
            entry.Categories = GetCategoryNames(categories);
            entry.Title = title;
            entry.Message = message;

            Log(entry, consoleOnly);
        }

        /// <summary>
        /// Writes an exception to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="e">The <see cref="Exception"/> to log.</param>
        public static void LogException(Exception e)
        {
            LogException(EventCode.General, BuildCategories(LogCategory.Exceptions), e, false);
        }

        /// <summary>
        /// Writes an exception to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="e">The <see cref="Exception"/> to log.</param>
        public static void LogException(EventCode eventId, Exception e)
        {
            LogException(eventId, BuildCategories(LogCategory.Exceptions), e, false);
        }

        /// <summary>
        /// Writes an exception to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="category">The <see cref="LogCategory"/> to which this event belongs.</param>
        /// <param name="e">The <see cref="Exception"/> to log.</param>
        public static void LogException(EventCode eventId, LogCategory category, Exception e)
        {
            LogException(eventId, BuildCategories(category), e, false);
        }

        /// <summary>
        /// Writes an exception to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="categories">The collection of <see cref="LogCategory"/> categories that the event belongs.</param>
        /// <param name="e">The <see cref="Exception"/> to log.</param>
        public static void LogException(EventCode eventId, ICollection<LogCategory> categories, Exception e)
        {
            LogException(eventId, categories, e, false);
        }

        /// <summary>
        /// Writes an exception to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="e">The <see cref="Exception"/> to log.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        public static void LogException(Exception e, bool consoleOnly)
        {
            LogException(EventCode.General, BuildCategories(LogCategory.Exceptions), e, consoleOnly);
        }

        /// <summary>
        /// Writes an exception to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="e">The <see cref="Exception"/> to log.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        public static void LogException(EventCode eventId, Exception e, bool consoleOnly)
        {
            LogException(eventId, BuildCategories(LogCategory.Exceptions), e, consoleOnly);
        }

        /// <summary>
        /// Writes an exception to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="category">The <see cref="LogCategory"/> to which this event belongs.</param>
        /// <param name="e">The <see cref="Exception"/> to log.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        public static void LogException(EventCode eventId, LogCategory category, Exception e, bool consoleOnly)
        {
            LogException(eventId, BuildCategories(category), e, consoleOnly);
        }

        /// <summary>
        /// Writes an exception to the Enterprise Library Logging Application Block.
        /// </summary>
        /// <param name="eventId">The <see cref="EventCode"/> of the logged event.</param>
        /// <param name="categories">The collection of <see cref="LogCategory"/> categories that the event belongs.</param>
        /// <param name="e">The <see cref="Exception"/> to log.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        public static void LogException(EventCode eventId, ICollection<LogCategory> categories, Exception e, bool consoleOnly)
        {
            LogEntry entry = new LogEntry();

            entry.EventId = (int)eventId;
            entry.Severity = TraceEventType.Error;
            entry.Categories = GetCategoryNames(categories);
            entry.Title = e.GetType().FullName;
            entry.Message = e.Message;

            entry.ExtendedProperties.Add(new KeyValuePair<string,object>("StackTrace", e.StackTrace));

            Log(entry, consoleOnly);
        }

        /// <summary>
        /// Logs the event to the Enterprise Library and the console (if a debug build).
        /// </summary>
        /// <param name="entry">The <see cref="LogEntry"/> to log.</param>
        /// <param name="consoleOnly">If <c>true</c>, the entry will only be logged to the console. 
        /// If <c>false</c> entry will be logged to both Enterprise Library and the console.</param>
        private static void Log(LogEntry entry, bool consoleOnly)
        {
            if (!consoleOnly)
                Logger.Write(entry);
#if DEBUG
            string stackTrace = "None";
            if (entry.ExtendedProperties.ContainsKey("StackTrace") && entry.ExtendedProperties["StackTrace"] != null)
                stackTrace = entry.ExtendedProperties["StackTrace"].ToString();

            Debug.WriteLine(string.Format(
                "--------------------\n{0}\n--------------------\nEvent: {1} ({2})\nTitle: {3}\nSeverity: {4}\n--------------------\n{5}\nStack Trace:\n{6}\n--------------------", 
                DateTime.Now,
                (EventCode)entry.EventId, 
                ParseCategories(entry.Categories),
                entry.Title,
                entry.Severity,
                entry.Message,
                stackTrace));
#endif
        }
    }
}
