// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections.Generic;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Properties;

using Microsoft.Practices.EnterpriseLibrary.Caching;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Manages message data for the <see cref="T:DotCommunities.Framework.Providers.FrameworkProvider"/>.
    /// </summary>
    public partial class FrameworkManager : FrameworkManagerBase
    {
        /// <summary>
        /// Removes message objects from the cache.
        /// </summary>
        /// <param name="culture">The message culture to remove from the cache.</param>
        private static void ClearMessages(string culture)
        {
            RemoveCachedItem(string.Format(Resources.MessagesCacheKey, CommunityContext.CurrentCommunity.ID, culture));
            RemoveCachedItemByPattern(Resources.MessageCacheRemovalPattern);
        }

        /// <summary>
        /// Returns a dictionary list of messages based on the invariant culture.
        /// </summary>
        /// <returns>A dictionary list of messages.</returns>
        public static Dictionary<int, Message> GetMessages()
        {
            return GetMessages(null);
        }

        /// <summary>
        /// Returns a dictionary list of messages based on the message culture.
        /// </summary>
        /// <param name="culture">The culture of the messages.</param>
        /// <returns>A dictionary list of messages.</returns>
        public static Dictionary<int, Message> GetMessages(string culture)
        {
            string cacheKey = string.Format(Resources.MessagesCacheKey, CommunityContext.CurrentCommunity.ID, culture);

            object item = GetCachedItem(cacheKey);

            if (item != null && item is Dictionary<int, Message>)
                return (Dictionary<int, Message>)item;

            return (Dictionary<int, Message>)AddCachedItemForLifetime(cacheKey, Provider.GetMessages(CommunityContext.CurrentCommunity.ID, culture));
        }

        /// <summary>
        /// Gets the <see cref="T:Message"/> object from the cached list of messages. The message is returned
        /// for the invariant culture.
        /// </summary>
        /// <param name="messageID">The ID of the message to retrieve.</param>
        /// <returns>A <see cref="T:Message"/> object containing required message data.</returns>
        public static Message GetMessage(int messageID)
        {
            return GetMessage(messageID, null);
        }

        /// <summary>
        /// Gets the <see cref="T:Message"/> object from the cached list of messages.
        /// </summary>
        /// <param name="messageID">The ID of the message to retrieve.</param>
        /// <param name="culture">The culture of the messages.</param>
        /// <returns>A <see cref="T:Message"/> object containing required message data.</returns>
        public static Message GetMessage(int messageID, string culture)
        {
            Dictionary<int, Message> messsages = GetMessages(culture);

            if (messsages.ContainsKey(messageID))
                return messsages[messageID];

            return null;
        }

        /// <summary>
        /// Gets the <see cref="T:Message"/> from the cache or database.
        /// </summary>
        /// <remarks>
        /// Messages are cached at high priority for up to 24 hours to improve performance.
        /// </remarks>
        /// <param name="name">The name of the message to retrieve.</param>
        /// <returns>A <see cref="T:Message"/> object containing message data.</returns>
        public static Message GetMessage(string name, string culture)
        {
            string cacheKey = string.Format(Resources.MessageCacheKey, CommunityContext.CurrentCommunity.ID, name, culture);

            object item = GetCachedItem(cacheKey);

            if (item != null && item is Message)
                return (Message)item;

            return (Message)AddCachedItem(cacheKey, Provider.GetMessage(CommunityContext.CurrentCommunity.ID, name, culture), 60 * 24, CacheItemPriority.High, true);
        }

        /// <summary>
        /// Edits an existing message.
        /// </summary>
        /// <param name="messageID">The ID of the message.</param>
        /// <param name="culture">The language culture for the message.</param>
        /// <param name="subject">The message subject header.</param>
        /// <param name="body">The message body text.</param>
        public static void EditMessage(int messageID, string culture, string subject, string body)
        {
            try
            {
                Provider.EditMessage(CommunityContext.CurrentCommunity.ID, messageID, culture, subject, body);
            }
            finally
            {
                ClearMessages(culture);
            }
        }
    }
}
