// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web.Security;
using System.Collections.Generic;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Properties;

using Microsoft.Practices.EnterpriseLibrary.Caching;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Manages message data for the <see cref="T:DotCommunities.Framework.Providers.FrameworkProvider"/>.
    /// </summary>
    public partial class FrameworkManager : FrameworkManagerBase
    {
        /// <summary>
        /// Clears the associated notification from the cache.
        /// </summary>
        /// <param name="contentPageID">The associated content page ID for notification.</param>
        /// <param name="sectionID">The associated section ID for notification.</param>
        /// <param name="userName">The name of the user to notify.</param>
        private static void ClearNotificationStatus(int pageID, int sectionID, string userName)
        {
            RemoveCachedItem(string.Format(Resources.NotificationStatusCacheKey, CommunityContext.CurrentCommunity.ID, pageID, sectionID, userName));
        }

        /// <summary>
        /// Gets the status of the notifications for the current user.
        /// </summary>
        /// <param name="contentPageID">The associated content page ID for notification.</param>
        /// <param name="sectionID">The associated section ID for notification.</param>
        /// <param name="userName">The name of the user to notify.</param>
        /// <returns><c>true</c> if the user should be notified; Otherwise <c>false</c>.</returns>
        public static bool GetNotificationStatus(int contentPageID, int sectionID, string userName)
        {
            string cacheKey = string.Format(Resources.NotificationStatusCacheKey, CommunityContext.CurrentCommunity.ID, contentPageID, sectionID, userName);

            object item = GetCachedItem(cacheKey);

            if (item != null && item is bool)
                return (bool)item;

            // Check that a page is selected
            int? id = null;
            if (contentPageID > 0)
                id = contentPageID;

            return (bool)AddCachedItem(cacheKey, Provider.GetNotificationStatus(CommunityContext.CurrentCommunity.ID, id, sectionID, userName));
        }

        /// <summary>
        /// Updates the status of the notifications for the current user.
        /// </summary>
        /// <param name="pageID">The associated page ID for notification.</param>
        /// <param name="sectionID">The associated section ID for notification.</param>
        /// <param name="userName">The name of the user to notify.</param>
        /// <param name="notify">Value indicating if the notification is set or not.</param>
        public static void UpdateNotificationStatus(int contentPageID, int sectionID, string userName, bool notify)
        {
            try
            {
                // Check that a page is selected
                int? id = null;
                if (contentPageID > 0)
                    id = contentPageID;

                Provider.UpdateNotificationStatus(CommunityContext.CurrentCommunity.ID, id, sectionID, userName, notify);
            }
            finally
            {
                ClearNotificationStatus(contentPageID, sectionID, userName);
            }
        }

        /// <summary>
        /// Sends e-mail notifications, ends the response and redirects the user back to the originating page.
        /// </summary>
        /// <param name="sectionID">The section ID containing the content.</param>
        /// <param name="contentPageID">The content page ID of the content.</param>
        /// <param name="userName">The user who created/updated the content.</param>
        /// <param name="title">The title of the content.</param>
        /// <param name="description">The description of the content.</param>
        /// <param name="returnUrl">The url to redirect the user to while notifications are being sent.</param>
        public static void SendNotifications(int sectionID, int contentPageID, string userName, string title, string description, string returnUrl)
        {
            SendNotifications(sectionID, contentPageID, userName, title, description, returnUrl, false);
        }

        /// <summary>
        /// Sends e-mail notifications, ends the response and redirects the user back to the originating page.
        /// </summary>
        /// <param name="sectionID">The section ID containing the content.</param>
        /// <param name="contentPageID">The content page ID of the content.</param>
        /// <param name="userName">The user who created/updated the content.</param>
        /// <param name="title">The title of the content.</param>
        /// <param name="description">The description of the content.</param>
        /// <param name="returnUrl">The url to redirect the user to while notifications are being sent.</param>
        /// <param name="isNewContent">Set to <c>true</c> if the content has just been added and we want notifications for the section.</param>
        public static void SendNotifications(int sectionID, int contentPageID, string userName, string title, string description, string returnUrl, bool isNewContent)
        {
            // Set the timeout to 10 minutes
            CommunityContext.Server.ScriptTimeout = 10 * 60;

            // Return the user to the originating url
            CommunityContext.Response.Redirect(returnUrl, false);

            // Complete the response
            CommunityContext.Response.Flush();
            CommunityContext.Response.Close();

            try
            {
                // Send the notifications
                SendNotifications(sectionID, contentPageID, userName, title, description, isNewContent);
            }
            finally
            {
                // End the response
                CommunityContext.Response.End();
            }
        }

        /// <summary>
        /// Enqueues e-mail notifications for sending.
        /// </summary>
        /// <param name="sectionID">The section ID containing the content.</param>
        /// <param name="contentPageID">The content page ID of the content.</param>
        /// <param name="userName">The user who created/updated the content.</param>
        /// <param name="title">The title of the content.</param>
        /// <param name="description">The description of the content.</param>
        /// <param name="isNewContent">Set to <c>true</c> if the content has just been added and we want notifications for the section.</param>
        private static void SendNotifications(int sectionID, int contentPageID, string userName, string title, string description, bool isNewContent)
        {
            // Get base url
            string baseUrl = CommunityContext.ResolveAbsoluteUrl(CommunityContext.ApplicationPath);

            NotificationFormat format = new NotificationFormat();

            // Set common format links
            format.SectionName = FrameworkManager.GetSection(sectionID).Title;
            format.EditProfileLink = baseUrl + "/Users_EditProfile.aspx";
            format.ContentLink = CommunityContext.ResolveAbsoluteUrl(CommunityContext.CalculateContentPath(sectionID, contentPageID));

            // Set common format text
            format.ContentTitle = title;
            format.ContentDescription = description;

            // Find out if we want to get the notification for the section rather than the page
            int id = -1;
            if (!isNewContent)
                id = contentPageID;

            // HACK This could take a while with a lot of users tracking the same post

            // Get the list of users to be notified
            foreach (string s in GetNotificationUsers(sectionID, id))
            {
                // Get the users details
                CommunityProfile userProfile = CommunityUser.GetProfile(s);
                MembershipUser user = CommunityUser.GetUser(s);

                // Set the users details in the format object
                format.UserName = userName;
                format.EmailAddress = user.Email;
                format.FirstName = userProfile.FirstName.Trim();
                format.LastName = userProfile.LastName.Trim();

                // If the user has not filled in their profile yet, 
                // set their name as their user name instead
                if (string.IsNullOrEmpty(format.FullName))
                    format.FirstName = user.UserName;

                // TODO Format to users selected language and send options as HTML
                FrameworkManager.EnqueueEmail(format.CreateNotificationEmail(FrameworkManager.GetMessage("Notifications", null)), true);
            }
        }

        /// <summary>
        /// Gets a list of users who are tracking this section/content from the cache or the database if not cached.
        /// </summary>
        /// <remarks>
        /// User names are cached at high priority for up to 5 minutes to improve performance.
        /// </remarks>
        /// <param name="sectionID">The section ID that is being tracked.</param>
        /// <param name="contentPageID">The content page ID that is being tracked.</param>
        /// <returns>A list of user names that are tracking this section/content.</returns>
        private static List<string> GetNotificationUsers(int sectionID, int contentPageID)
        {
            string cacheKey = string.Format(Resources.NotificationUsersCacheKey, CommunityContext.CurrentCommunity.ID, sectionID, contentPageID);

            object item = GetCachedItem(cacheKey);

            if (item != null && item is List<string>)
                return (List<string>)item;

            // Check that a page is selected
            int? id = null;
            if (contentPageID > 0)
                id = contentPageID;

            return (List<string>)AddCachedItem(cacheKey, Provider.GetNotificationUsers(CommunityContext.CurrentCommunity.ID, sectionID, id), 5, CacheItemPriority.High, false);
        }
    }
}
