// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;
using System.Collections.Generic;

using DotCommunities.Framework.Collections;
using DotCommunities.Framework.Components;
using DotCommunities.Framework.Properties;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Manages sydication data for the <see cref="FrameworkProvider"/>.
    /// </summary>
    public partial class FrameworkManager : FrameworkManagerBase
    {
        /// <summary>
        /// Returns a list of <see cref="T:SyndicationItemInfo" /> objects that represent
        /// content pages.
        /// </summary>
        /// <param name="sectionID">The section ID to list.</param>
        /// <param name="maxCount">The maximum number of items to return.</param>
        /// <returns>A list of <see cref="T:SyndicationItemInfo" /> objects.</returns>
        public static List<SyndicationItem> GetSyndicationItems(int sectionID, int maxCount)
        {
            // Create the item list to return from this method
            List<SyndicationItem> colItems = new List<SyndicationItem>();

            // Get all the enabled sections for the site
            SectionCollection sections = GetAllEnabledSections();

            // Now get the content for the parent section from the database
            DataTable dtbSyndicationItems = GetInheritedSyndicationItemsFromProvider(sectionID);

            // Recursively load child content items stating with the current section ID
            dtbSyndicationItems.Merge(GetInheritedSyndicationItems(sectionID, sections));

            // Check that we have any rows returned
            if (dtbSyndicationItems.Rows.Count > 0)
            {
                // Sort the rows by the date the content was created
                dtbSyndicationItems.DefaultView.Sort = "ContentPage_DateCreated DESC";

                int feedCount = maxCount;

                // Loop through the table creating sycronisation item objects as we go
                foreach (DataRowView dr in dtbSyndicationItems.DefaultView)
                {
                    SyndicationItem responseItem = new SyndicationItem();

                    responseItem.ContentPageID = (int)dr["ContentPage_ID"];
                    responseItem.Title = (string)dr["ContentPage_Title"];
                    responseItem.Link = CommunityContext.CalculateContentPath((int)dr["ContentPage_SectionID"], (int)dr["ContentPage_ID"]);
                    responseItem.Description = (string)dr["ContentPage_Description"];
                    responseItem.DateCreated = (DateTime)dr["ContentPage_DateCreated"];
                    responseItem.EnableCategories = (bool)dr["Section_EnableCategories"];
                    responseItem.Categories = dr["ContentPage_Categories"].ToString().Split('|');
                    responseItem.EnableComments = (bool)dr["Section_EnableComments"];
                    responseItem.CommentCount = (int)dr["CommentCount"];

                    colItems.Add(responseItem);

                    // Decrement the feed count on each pass - once we get to 0 we don't need to
                    // get any more objects, so break out of the loop
                    if (--feedCount <= 0)
                        break;
                }
            }

            // Finally, return the list of sycronisation items
            return colItems;
        }

        /// <summary>
        /// Recursive method that searches the parent section and its child sections for syndication content.
        /// </summary>
        /// <param name="parentSectionID">The parent section ID to locate child sections with.</param>
        /// <param name="sections">The section collection of all enabled sections.</param>
        /// <returns>A data table of found syndication items.</returns>
        private static DataTable GetInheritedSyndicationItems(int parentSectionID, SectionCollection sections)
        {
            DataTable dtbSyndicationItems = new DataTable("SyndicationItems");

            // Loop through the sections collection
            foreach (Section s in sections.Values)
            {
                // Check if the current section is published as a feed and is a child section
                if (s.IsSectionSyndicated && (s.ParentID == parentSectionID))
                {
                    // Get the content for the current section from the database (max 20 items)
                    dtbSyndicationItems.Merge(GetInheritedSyndicationItemsFromProvider(s.SectionID));

                    // Recurisvely call this method, loading any results into the data table
                    dtbSyndicationItems.Merge(GetInheritedSyndicationItems(s.SectionID, sections));
                }
            }

            // Finally, return the data table of found syncronisation items
            return dtbSyndicationItems;
        }

        /// <summary>
        /// Retrieves the Syndication data from the cache, or the provider if not in the cache.
        /// </summary>
        /// <param name="sectionID">The section ID to syndicate.</param>
        /// <returns>A <see cref="DataTable"/> containing syndication data.</returns>
        private static DataTable GetInheritedSyndicationItemsFromProvider(int sectionID)
        {
            string cacheKey = string.Format(Resources.SyndicationItemsCacheKey, CommunityContext.CurrentCommunity.ID, sectionID);

            object syndicationItems = GetCachedItem(cacheKey);

            if (syndicationItems != null && syndicationItems is DataSet && (syndicationItems as DataSet).Tables.Count > 0)
                return (syndicationItems as DataSet).Tables[0];

            DataSet syndicationDataSet = (DataSet)AddCachedItem(cacheKey, Provider.GetInheritedSyndicationItems(CommunityContext.CurrentCommunity.ID, sectionID));
            if (syndicationDataSet != null && syndicationDataSet.Tables.Count > 0)
                return syndicationDataSet.Tables[0];

            return null;
        }
    }
}
