// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections.Generic;
using System.IO;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Properties;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Loads and configures the <see cref="PageTheme"/> and <see cref="PageStyle"/> objects.
    /// </summary>
    public class ThemeManager : CommunityManagerBase
    {
        /// <summary>
        /// Gets the themes from the cache.
        /// </summary>
        /// <returns>A generic dictionary containing <see cref="PageTheme"/> objects.</returns>
        public static Dictionary<string, PageTheme> GetThemes()
        {
            object item = GetCachedItem(Resources.PageThemesCacheKey);

            if (item != null)
                return (Dictionary<string, PageTheme>)item;

            return (Dictionary<string, PageTheme>)AddCachedItemForLifetime(Resources.PageThemesCacheKey, GetThemesFromFileSystem());
        }

        /// <summary>
        /// Gets the styles from the cache.
        /// </summary>
        /// <returns>A generic dictionary containing <see cref="PageStyle"/> objects.</returns>
        public static Dictionary<string, PageStyle> GetStyles()
        {
            object item = GetCachedItem(Resources.PageStylesCacheKey);

            if (item != null)
                return (Dictionary<string, PageStyle>)item;

            return (Dictionary<string, PageStyle>)AddCachedItemForLifetime(Resources.PageStylesCacheKey, GetStylesFromFileSystem());
        }

        /// <summary>
        /// Gets the themes from the file system.
        /// </summary>
        /// <returns>A generic dictionary containing <see cref="PageTheme"/> objects.</returns>
        private static Dictionary<string, PageTheme> GetThemesFromFileSystem()
        {
            Dictionary<string, PageTheme> colThemes = new Dictionary<string, PageTheme>();
            string[] themeDirectories;

            // Load List of Common Themes
            string themeDirPath = CommunityContext.Context.Server.MapPath(CommunityContext.ApplicationPath + "\\Communities\\");
            try
            {
                themeDirectories = Directory.GetDirectories(themeDirPath + "Common\\Themes");
            }
            catch (DirectoryNotFoundException)
            {
                throw new Exception("Themes directory is missing");
            }

            foreach (string theme in themeDirectories)
            {
                if (!Path.GetFileName(theme).StartsWith("_"))
                {
                    string themeName = Path.GetFileNameWithoutExtension(theme);
                    string themePath = theme.Remove(0, themeDirPath.Length);
                    themePath = ConvertUrl(themePath);
                    string themePage = string.Format("{0}{1}/Skins/PageSkins/{2}.master", CommunityContext.CommunitiesFolderPath, themePath, themeName);
                    colThemes.Add(themeName, new PageTheme(themeName, themePath, themePage));
                }
            }

            // Load List of Community Specific Themes
            themeDirPath = CommunityContext.Context.Server.MapPath(CommunityContext.ApplicationPath + "\\Communities\\");
            string fullThemeDirPath = themeDirPath + CommunityContext.CurrentCommunity.Name + "\\Themes";
            if (Directory.Exists(fullThemeDirPath))
            {
                themeDirectories = Directory.GetDirectories(fullThemeDirPath);

                foreach (string theme in themeDirectories)
                {
                    if (!Path.GetFileName(theme).StartsWith("_"))
                    {
                        string themeName = Path.GetFileNameWithoutExtension(theme);
                        string themePath = theme.Remove(0, themeDirPath.Length);
                        themeName = String.Format("{0} ({1})", themeName, CommunityContext.CurrentCommunity.Name);
                        themePath = ConvertUrl(themePath);
                        string themePage = string.Format("{0}{1}/Skins/PageSkins/{2}.master", CommunityContext.CommunitiesFolderPath, themePath, themeName);
                        colThemes.Add(themeName, new PageTheme(themeName, themePath, themePage));
                    }
                }
            }

            return colThemes;
        }

        /// <summary>
        /// Gets the styles from the file system.
        /// </summary>
        /// <returns>A generic dictionary containing <see cref="PageStyle"/> objects.</returns>
        private static Dictionary<string, PageStyle> GetStylesFromFileSystem()
        {
            Dictionary<string, PageStyle> colStyles = new Dictionary<string, PageStyle>();
            string[] themeDirectories;

            // Load List of Common Styles
            string themeDirPath = CommunityContext.Context.Server.MapPath(CommunityContext.ApplicationPath + "\\Communities\\");
            try
            {
                themeDirectories = Directory.GetDirectories(themeDirPath + "Common\\Themes");
            }
            catch (DirectoryNotFoundException)
            {
                throw new Exception("Themes directory is missing");
            }

            foreach (string theme in themeDirectories)
            {
                if (!Path.GetFileName(theme).StartsWith("_"))
                {
                    // Get Style Files
                    string[] styleFiles = Directory.GetFiles(Path.Combine(theme, "Styles"), "*.css");
                    foreach (string style in styleFiles)
                    {
                        string styleName = Path.GetFileNameWithoutExtension(style);
                        string stylePath = style.Remove(0, themeDirPath.Length);
                        stylePath = string.Format("{0}{1}", CommunityContext.CommunitiesFolderPath, ConvertUrl(stylePath));
                        // if not default theme show it
                        if (styleName.ToLower() == "default")
                            styleName = Path.GetFileNameWithoutExtension(theme);
                        else
                            styleName = String.Format("{0} {1}", Path.GetFileNameWithoutExtension(theme), styleName);
                        colStyles.Add(styleName, new PageStyle(styleName, stylePath));
                    }
                }
            }

            // Load List of Community Specific Styles
            themeDirPath = CommunityContext.Context.Server.MapPath(CommunityContext.ApplicationPath + "\\Communities\\");
            string fullThemeDirPath = themeDirPath + CommunityContext.CurrentCommunity.Name + "\\Themes";
            if (Directory.Exists(fullThemeDirPath))
            {
                themeDirectories = Directory.GetDirectories(fullThemeDirPath);

                foreach (string theme in themeDirectories)
                {
                    if (!Path.GetFileName(theme).StartsWith("_"))
                    {
                        // Get Style Files
                        string[] styleFiles = Directory.GetFiles(Path.Combine(theme, "Styles"), "*.css");
                        foreach (string style in styleFiles)
                        {
                            string styleName = Path.GetFileNameWithoutExtension(style);
                            string stylePath = style.Remove(0, themeDirPath.Length);
                            stylePath = string.Format("{0}{1}", CommunityContext.CommunitiesFolderPath, ConvertUrl(stylePath));
                            // if not default theme show it
                            if (styleName.ToLower() == "default")
                                styleName = String.Format("{0} ({1})", Path.GetFileNameWithoutExtension(theme), CommunityContext.CurrentCommunity.Name);
                            else
                                styleName = String.Format("{0} {1} ({2})", Path.GetFileNameWithoutExtension(theme), styleName, CommunityContext.CurrentCommunity.Name);
                            colStyles.Add(styleName, new PageStyle(styleName, stylePath));
                        }
                    }
                }
            }

            return colStyles;
        }

        /// <summary>
        /// Converts the URL.
        /// </summary>
        /// <param name="path">The path.</param>
        /// <returns>The converted URL.</returns>
        private static string ConvertUrl(string path)
        {
            return path.Replace("\\", "/");
        }
    }
}
