// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.IO;
using System.Xml.Serialization;

using DotCommunities.Framework.Components;
using DotCommunities.Framework.Properties;
using System.Xml;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Manages time zone data.
    /// </summary>
    public class TimeZoneManager
    {
        /// <summary>
        /// Retrieves a time zone from the cache or XML file.
        /// </summary>
        /// <param name="id">The time zone to retrieve (e.g. GMT).</param>
        /// <returns>A <see cref="Zone"/> object containing the time zone details.</returns>
        public static Zone GetTimeZone(string id)
        {
            TimeZoneData zones = GetTimeZones();
            if (zones != null && zones.Items != null)
                foreach (Zone zone in zones.Items)
                    if (zone.ID.ToUpperInvariant() == id.ToUpperInvariant())
                        return zone;
            return null;
        }

        /// <summary>
        /// Retrieves a list of time zones from the cache or XML file.
        /// </summary>
        /// <returns>A <see cref="TimeZoneData"/> object containing a list of <see cref="Zone"/> objects.</returns>
        public static TimeZoneData GetTimeZones()
        {
            object zone = FrameworkManager.GetCachedItem(Resources.TimeZonesCacheKey);

            if (zone != null && zone is TimeZoneData)
                return (TimeZoneData)zone;

            return (TimeZoneData)FrameworkManager.AddCachedItemForLifetime(Resources.TimeZonesCacheKey, GetTimeZoneData());
        }

        /// <summary>
        /// Retrieves a list of time zones from the XML file.
        /// </summary>
        /// <returns>A <see cref="TimeZoneData"/> object containing a list of <see cref="Zone"/> objects.</returns>
        private static TimeZoneData GetTimeZoneData()
        {
            // Create a serialiser
            XmlSerializer serializer = new XmlSerializer(typeof(TimeZoneData));

            XmlDocument xmlSettingsData = new XmlDocument();
            // Try to load the XML data from the default settings
            try
            {
                xmlSettingsData.Load(CommunityContext.AppSettingsPath + Resources.TimeZoneXmlFileName);
            }
            catch (XmlException e)
            {
                // Log it
                LogManager.WriteWarning("Time Zone File Error", string.Format("Couuld not load {0}, reason: {1}", CommunityContext.AppSettingsPath + Resources.TimeZoneXmlFileName, e.Message));
            }

            TextReader reader = new StringReader(xmlSettingsData.InnerXml);
            return (TimeZoneData)serializer.Deserialize(reader);
        }
    }
}
