// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;
using System.Web.Security;
using System.Collections.Generic;

using DotCommunities.Framework.Providers;
using DotCommunities.Framework.Components;
using DotCommunities.Framework.Properties;

namespace DotCommunities.Framework.Managers
{
    /// <summary>
    /// Manages community data for the <see cref="FrameworkProvider"/>.
    /// </summary>
    public partial class FrameworkManager : FrameworkManagerBase
    {
        /// <summary>
        /// Returns a collection of filtered users in the current community.
        /// </summary>
        /// <param name="filter">A filter to return users that have a username beginning with the filter string.</param>
        /// <returns>A <see cref="LocalUser"/> list object containing the filtered users in this community.</returns>
        public static List<LocalUser> GetFilteredUsers(string filter)
        {
            string cacheKey = string.Format(Resources.LocalFilteredUsersCacheKey, CommunityContext.CurrentCommunity.ID, filter);

            object item = GetCachedItem(cacheKey);

            if (item != null && item is List<LocalUser>)
                return (List<LocalUser>)item;

            // If we do not have a filter then just return all the users
            if (string.IsNullOrEmpty(filter) || filter.ToLowerInvariant() == "all")
                return GetUsers();

            // Filter and cache the users that have usernames beginning with the filter text
            List<LocalUser> filteredUsers = new List<LocalUser>();
            foreach (LocalUser u in GetUsers())
                if (u.UserName.ToLowerInvariant().StartsWith(filter.ToLowerInvariant()))
                    filteredUsers.Add(u);

            return (List<LocalUser>)AddCachedItemFor60Seconds(cacheKey, filteredUsers);
        }

        /// <summary>
        /// Returns a collection of all local users in the current community.
        /// </summary>
        /// <returns>A list of <see cref="LocalUser"/> objects containing all the local users in this community.</returns>
        public static List<LocalUser> GetUsers()
        {
            string cacheKey = string.Format(Resources.LocalUsersCacheKey, CommunityContext.CurrentCommunity.ID);

            object item = GetCachedItem(cacheKey);

            if (item != null && item is List<LocalUser>)
                return (List<LocalUser>)item;

            // Delegate to the provider
            return (List<LocalUser>)AddCachedItem(cacheKey, Provider.GetUsers(CommunityContext.CurrentCommunity.ID));
        }

        /// <summary>
        /// Gets the local user information from the cache or the database.
        /// </summary>
        /// <param name="userName">The user name for the local data.</param>
        /// <returns>A <see cref="LocalUser"/> object containing local data.</returns>
        public static LocalUser GetUser(string userName)
        {
            string cacheKey = string.Format(Resources.LocalUserCacheKey, CommunityContext.CurrentCommunity.ID, userName);

            object item = GetCachedItem(cacheKey);

            if (item != null)
                return (LocalUser)item;

            // Delegate to the provider
            return (LocalUser)AddCachedItem(cacheKey, Provider.GetUser(CommunityContext.CurrentCommunity.ID, userName));
        }

        /// <summary>
        /// Updates the local user data in the database.
        /// </summary>
        /// <param name="userName">The user name for the local data.</param>
        /// <param name="enableNewsletter">The users newsletter preference.</param>
        /// <param name="enableNotifications">The users notification preference.</param>
        /// <param name="databaseQuota">The users maximum database quota.</param>
        public static void UpdateUser(string userName, bool enableNewsletter, bool enableNotifications, int databaseQuota)
        {
            string cacheKey = string.Format(Resources.LocalUserCacheKey, CommunityContext.CurrentCommunity.ID, userName);

            Provider.UpdateUser(CommunityContext.CurrentCommunity.ID, userName, enableNewsletter, enableNotifications, databaseQuota);

            RemoveCachedItem(cacheKey);
        }
    }
}
