// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;

using DotCommunities.Framework.Components;

namespace DotCommunities.Framework
{
    /// <summary>
    /// Represents all the elements that will be replaced in 
    /// a notification email.
    /// </summary>
    public class NotificationFormat
    {
        private string sectionName = String.Empty;
        private string userName = String.Empty;
        private string firstName = String.Empty;
        private string lastName = String.Empty;
        private string emailAddress = String.Empty;
        private string contentTitle = String.Empty;
        private string contentDescription = String.Empty;
        private string contentLink = String.Empty;
        private string editProfileLink = String.Empty;

        #region Public Properties
        /// <summary>
        /// Gets or sets the name of the section replaced in the email.
        /// </summary>
        /// <value>The name of the section.</value>
        public string SectionName
        {
            get { return sectionName; }
            set { sectionName = value; }
        }

        /// <summary>
        /// Gets or sets the user name replaced in the email.
        /// </summary>
        /// <value>The user name.</value>
        public string UserName
        {
            get { return userName; }
            set { userName = value; }
        }

        /// <summary>
        /// Gets or sets the users first name replaced in the email.
        /// </summary>
        /// <value>The users first name.</value>
        public string FirstName
        {
            get { return firstName; }
            set { firstName = value; }
        }

        /// <summary>
        /// Gets or sets the users last name replaced in the email.
        /// </summary>
        /// <value>The users last name.</value>
        public string LastName
        {
            get { return lastName; }
            set { lastName = value; }
        }
        
        /// <summary>
        /// Gets the full users name replaced in the email (from first and last name).
        /// </summary>
        /// <value>The full users name.</value>
        public string FullName
        {
            get { return string.Format("{0} {1}", firstName, lastName).Trim(); }
        }


        /// <summary>
        /// Gets or sets the e-mail address replaced in the email.
        /// </summary>
        /// <value>The e-mail address.</value>
        public string EmailAddress
        {
            get { return emailAddress; }
            set { emailAddress = value; }
        }

        /// <summary>
        /// Gets or sets the title of the content item replaced in the email. 
        /// </summary>
        /// <value>The content title.</value>
        public string ContentTitle
        {
            get { return contentTitle; }
            set { contentTitle = value; }
        }

        /// <summary>
        /// Gets or sets the description of the content item replaced in the email. 
        /// </summary>
        /// <value>The content description.</value>
        public string ContentDescription
        {
            get { return contentDescription; }
            set { contentDescription = value; }
        }

        /// <summary>
        /// Gets or sets the link to the content page replaced in the email. 
        /// </summary>
        /// <value>The content link.</value>
        public string ContentLink
        {
            get { return contentLink; }
            set { contentLink = value; }
        }

        /// <summary>
        /// Gets or sets the link to the user profile replaced in the email.
        /// </summary>
        /// <value>The edit profile link.</value>
        public string EditProfileLink
        {
            get { return editProfileLink; }
            set { editProfileLink = value; }
        }
        #endregion

        /// <summary>
        /// Initializes a new instance of the <see cref="T:NotificationFormat"/> class.
        /// </summary>
        public NotificationFormat() {}

        /// <summary>
        /// Creates an <see cref="T:Email"/> object from a notification <see cref="T:Message"/>.
        /// </summary>
        /// <param name="notification">The notification <see cref="T:Message"/> to format.</param>
        /// <returns>The <see cref="T:Email"/> object with formatted text.</returns>
        public Email CreateNotificationEmail(Message notification)
        {
            // Check that we have a notification object first
            if (notification == null)
                throw new ArgumentNullException("CreateNotificationEmail", "Notification message cannot be null");

            // Create a new email object
            Email mail = new Email(string.Format("notify@{0}", CommunityContext.CurrentCommunity.DomainName), EmailAddress);
            // The body is always in HTML format
            mail.IsBodyHtml = true;
            // Set the formatted subject and body
            mail.Subject = Format(notification.Subject);
            mail.Body = Format(notification.Body);
            // Return the configured email
            return mail;
        }

        /// <summary>
        /// Formats a string replacing key text with <see cref="NotificationFormat"/> properties.
        /// </summary>
        /// <param name="text">The text to format.</param>
        /// <returns>The formatted text.</returns>
        private string Format(string text)
        {
            // Perform replacements
            text = text.Replace("[SectionName]", this.SectionName);
            text = text.Replace("[UserName]", this.UserName);
            text = text.Replace("[FirstName]", this.FirstName);
            text = text.Replace("[LastName]", this.LastName);
            text = text.Replace("[FullName]", this.FullName);
            text = text.Replace("[ContentLink]", this.ContentLink);
            text = text.Replace("[ContentTitle]", this.ContentTitle);
            text = text.Replace("[ContentBody]", this.ContentDescription);
            text = text.Replace("[EditProfileLink]", this.EditProfileLink);
            // return the formatted text
            return text;
        }
    }
}
