// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;
using System.Collections.Generic;

using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework
{
    /// <summary>
    /// Represents all the information about a section, including the section
    /// security information.
    /// </summary>
    [Serializable]
    public class Section
    {
        private int sectionID;
        private int communityID;
        private int parentID = -1;
        private string name;
        private string title;
        private string menuTitle;
        private string description;
        private string pageTypeName;
        private string skin;
        private string layout;
        private string style;
        private string defaultSkin;
        private string logo;
        private string pageHeader;
        private string pageFooter;
        private string footer;
        private string pageMetaKeys;
        private string pageMetaDescription;
        private int viewCount;
        private bool enableCategories;
        private bool enableComments;
        private bool enableModeration;
        private bool enableRatings;
        private bool enableCommentRatings;
        private bool enableNotifications;
        private bool enableIndexing;
        private bool inheritTransformations;
        private string transformations;
        private bool areWebBoxesInherited;
        private bool areWebServiceBoxesInherited;
        private int webBoxDisplayMode;
        private int webServiceBoxDisplayMode;
        private string webServicePassword;
        private AllowHtml allowHtmlInput;
        private AllowHtml allowCommentHtmlInput;
        private bool isSectionWebService;
        private bool isSectionSyndicated;
        private bool isSystem;
        private bool isEnabled;
        private int recordsPerPage = 10;
        private DateTime dateCreated;

        private string content;

        private List<string> webBoxes;
        private List<string> webServiceBoxes;

        private List<string> viewRoles;
        private List<string> addRoles;
        private List<string> editRoles;
        private List<string> deleteRoles;
        private List<string> commentRoles;
        private List<string> moderateRoles;
        private List<string> rateRoles;

        private string path;

        #region Public Properties

        /// <summary>
        /// Represents the section ID.
        /// </summary>
        public int SectionID 
        {
            get { return sectionID; }
        }

        /// <summary>
        /// Represents the community ID associated with the section. 
        /// </summary>
        public int CommunityID 
        {
            get { return communityID; }
        }

        /// <summary>
        /// Represents the section name which is used when building the section URL.
        /// </summary>
        public string Name 
        {
            get { return name; }
        }

        /// <summary>
        /// Represents the section URL.
        /// </summary>
        public string Path 
        {
            get { return path; }
        }

        /// <summary>
        /// Represents the full name of the section.
        /// </summary>
        public string Title 
        {
            get { return title; }
        }

        /// <summary>
        /// Represents the title of the section displayed in menus.
        /// </summary>
        public string MenuTitle 
        {
            get { return menuTitle; }
        }

        /// <summary>
        /// Represents the brief description of the section.
        /// </summary>
        public string Description 
        {
            get { return description; }
        }


        /// <summary>
        /// Gets a value indicating whether this instance is the root (home) section.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is a root section; otherwise, <c>false</c>.
        /// </value>
        public bool IsRoot
        {
            get { return parentID <= 0; }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is enabled.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is enabled; otherwise, <c>false</c>.
        /// </value>
        public bool IsEnabled 
        {
            get { return isEnabled;}
        }

        /// <summary>
        /// Gets a value indicating whether this instance is a system section (for example, the profiles section).
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is a system section; otherwise, <c>false</c>.
        /// </value>
        public bool IsSystem
        {
            get { return isSystem; }
        }

        /// <summary>
        /// Represents the type name of content contained in the section. 
        /// </summary>
        public string PageTypeName 
        {
            get { return pageTypeName; }
        }

        /// <summary>
        /// Gets a value indicating whether this instance of web boxes inherited from parent section.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance of web boxes inherited; otherwise, <c>false</c>.
        /// </value>
        public bool AreWebBoxesInherited 
        {
            get { return areWebBoxesInherited; }
        }

        /// <summary>
        /// Represents the Web boxes displayed in this section. 
        /// </summary>
        public List<string> WebBoxes 
        {
            get { return webBoxes; }
            set { webBoxes = value; }
        }

        /// <summary>
        /// Gets or sets the web box display mode.
        /// </summary>
        public int WebBoxDisplayMode 
        {
            get { return webBoxDisplayMode; }
			set { webBoxDisplayMode = value; }
        }

        /// <summary>
        /// Gets a value indicating whether this instance of web service boxes are inherited.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance of web service boxes are inherited; otherwise, <c>false</c>.
        /// </value>
        public bool AreWebServiceBoxesInherited 
        {
            get { return areWebServiceBoxesInherited; }
        }

        /// <summary>
        /// Represents the web service boxes for this section.
        /// </summary>
        public List<string> WebServiceBoxes 
        {
            get { return webServiceBoxes; }
            set { webServiceBoxes = value; }
        }

        /// <summary>
        /// Gets or sets the web service box display mode.
        /// </summary>
        /// <value>The web service box display mode.</value>
        public int WebServiceBoxDisplayMode 
        {
            get { return webServiceBoxDisplayMode; }
			set { webServiceBoxDisplayMode = value; }
        }

        /// <summary>
        /// Gets the page skin used for this section.
        /// </summary>
        public string Skin 
        {
            get { return skin; }
        }

        /// <summary>
        /// Gets the page layout used for this section.
        /// </summary>
        public string Layout
        {
            get { return layout; }
        }
        
        /// <summary>
        /// Gets the cascading style sheet applied to this section.
        /// </summary>
        public string Style 
        {
            get { return style; }
        }
        
        /// <summary>
        /// Gets the parent section ID of this section.
        /// </summary>
        public int ParentID 
        {
            get { return parentID; }
        }
 
        /// <summary>
        /// Gets the logo image for this section.
        /// </summary>
        public string Logo 
        {
            get { return logo; }
        }

        /// <summary>
        /// Gets the page header text.
        /// </summary>
        public string PageHeader 
        {
            get { return pageHeader; }
        }

        /// <summary>
        /// Gets the page footer text.
        /// </summary>
        public string PageFooter 
        {
            get { return pageFooter; }
        }

        /// <summary>
        /// Gets the page footer for the bottom of the page.
        /// </summary>
        public string Footer 
        {
            get { return footer; }
        }

        /// <summary>
        /// Gets the page meta description.
        /// </summary>
        public string PageMetaDescription 
        {
            get { return pageMetaDescription; }
        }

        /// <summary>
        /// Gets the page meta keys.
        /// </summary>
        public string PageMetaKeys 
        {
            get { return pageMetaKeys; }
        }

        /// <summary>
        /// Gets the roles of users who can view pages in this section.
        /// </summary>
        public List<string> ViewRoles 
        {
            get { return viewRoles; }
        }

        /// <summary>
        /// Gets the roles of users who can add pages to this section.
        /// </summary>
        public List<string> AddRoles 
        {
            get { return addRoles; }
        }

        /// <summary>
        /// Gets the roles of users who can edit pages in this section.
        /// </summary>
        public List<string> EditRoles 
        {
            get { return editRoles; }
        }

        /// <summary>
        /// Gets the roles of users who can delete pages in this section.
        /// </summary>
        public List<string> DeleteRoles 
        {
            get { return deleteRoles; }
        }
      
        /// <summary>
        /// Gets the number of content items to display on the page. 
        /// </summary>
        public int RecordsPerPage 
        {
            get { return recordsPerPage; }
        }

        /// <summary>
        /// Gets a value indicating whether content in this section should be categorised.
        /// </summary>
        /// <value>
        ///     <c>true</c> if categorised otherwise, <c>false</c>.
        /// </value>
        public bool EnableCategories 
        {
            get { return enableCategories; }
        }

        /// <summary>
        /// Gets a value indicating whether users should be allowed to add comments to content 
        /// items in this section.
        /// </summary>
        /// <value>
        ///     <c>true</c> if comments enabled otherwise, <c>false</c>.
        /// </value>
        public bool EnableComments 
        {
            get { return enableComments; }
        }

        /// <summary>
        /// Gets the roles of users who can comment on pages in this section.
        /// </summary>
        public List<string> CommentRoles 
        {
            get { return commentRoles; }
        }

        /// <summary>
        /// Gets a value indicating whether content must be moderated before it is displayed.
        /// </summary>
        /// <value>
        ///     <c>true</c> if [enable moderation]; otherwise, <c>false</c>.
        /// </value>
        public bool EnableModeration 
        {
            get { return enableModeration; }
        }

        /// <summary>
        /// Gets the roles of users who can moderate pages in this section.
        /// </summary>
        public List<string> ModerateRoles 
        {
            get { return moderateRoles; }
        }

        /// <summary>
        /// Gets a value indicating whether users can rate content in this section.
        /// </summary>
        /// <value>
        ///     <c>true</c> if users can rate content otherwise, <c>false</c>.
        /// </value>
        public bool EnableRatings 
        {
            get { return enableRatings; }
        }

        /// <summary>
        /// Gets a value indicating whether users can rate comments in this section.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if users can rate comments otherwise, <c>false</c>.
        /// </value>
        public bool EnableCommentRatings 
        {
            get { return enableCommentRatings; }
        }

        /// <summary>
        /// Gets the roles of users who can rate pages in this section.
        /// </summary>
        public List<string> RateRoles 
        {
            get { return rateRoles; }
        }

        /// <summary>
        /// Gets a value indicating whether users can receive email notifications when new content 
        /// is added to this section.
        /// </summary>
        /// <value>
        ///     <c>true</c> if users can receive email notifications otherwise, <c>false</c>.
        /// </value>
        public bool EnableNotifications 
        {
            get { return enableNotifications; }
        }    

        /// <summary>
        /// Gets a value indicating whether transformations are inherited from the parent section.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if transformations are inherited otherwise, <c>false</c>.
        /// </value>
        public bool InheritTransformations 
        {
            get { return inheritTransformations; }
        }
        
        /// <summary>
        /// Gets the transformations that are applied to text in this section.
        /// </summary>
        public string Transformations 
        {
            get { return transformations; }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is exposed as a Web service. 
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is a section web service; otherwise, <c>false</c>.
        /// </value>
        public bool IsSectionWebService 
        {
            get { return isSectionWebService; }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is sydicated as an RSS service. 
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is sydicated; otherwise, <c>false</c>.
        /// </value>
        public bool IsSectionSyndicated
        {
            get { return isSectionSyndicated; }
        }
        
        /// <summary>
        /// Represents the password needed to access this section's Web service.
        /// </summary>
        public string WebServicePassword 
        {
            get { return webServicePassword; }
        }

        /// <summary>
        /// Represents whether or not users are allowed to add HTML content to forms.
        /// </summary>
        public AllowHtml AllowHtmlInput 
        {
            get { return allowHtmlInput; }
        }

        /// <summary>
        /// Represents whether or not users are allowed to add HTML content to comments.
        /// </summary>
        public AllowHtml AllowCommentHtmlInput 
        {
            get { return allowCommentHtmlInput; }
        }
        
        /// <summary>
        /// Gets the fully qualified assembly name that represents the content displayed by this section.
        /// </summary>
        public string Content 
        {
            get { return content; }
        }

        /// <summary>
        /// Gets the fail over skin file name.
        /// </summary>
		public string DefaultSkin 
		{
			get { return defaultSkin; }
		}
        #endregion

        /// <summary>
        /// Initializes a new instance of the <see cref="Section"/> class.
        /// </summary>
        /// <param name="drowSection">The drow section.</param>
        /// <param name="drowSectionSecurity">The drow section security.</param>
        /// <param name="drowWebBoxes">The drow web boxes.</param>
        /// <param name="drowWebServiceBoxes">The drow web service boxes.</param>
        public Section(DataRow drowSection, DataRow[] drowSectionSecurity, DataRow[] drowWebBoxes, DataRow[] drowWebServiceBoxes) 
        {
            if (drowSection["Section_ParentID"] != DBNull.Value)
                parentID = (int)drowSection["Section_ParentID"];

            sectionID = (int)drowSection[ "Section_ID" ];
            communityID = (int)drowSection[ "Section_CommunityID" ];
            name = (string)drowSection["Section_Name"];
            title = (string)drowSection[ "Section_Title" ];
            menuTitle = (string)drowSection[ "Section_MenuTitle" ];
            description = (string)drowSection[ "Section_Description" ];
            pageTypeName = (string)drowSection["Section_PageTypeName"];
            skin = (string)drowSection["Section_PageSkin"];
            layout = (string)drowSection["Section_PageLayout"];
            defaultSkin = (string)drowSection["Section_DefaultPageSkin"];
            style = (string)drowSection["Section_PageStyle"];
            logo = (string)drowSection["Section_Logo"];
            pageHeader = (string)drowSection["Section_PageHeader"];
            pageFooter = (string)drowSection["Section_PageFooter"];
            footer = (string)drowSection["Section_Footer"];
            pageMetaKeys = (string)drowSection["Section_PageMetaKeys"];
            pageMetaDescription = (string)drowSection["Section_PageMetaDescription"];
            viewCount = (int)drowSection["Section_ViewCount"];
            enableCategories = (bool)drowSection["Section_EnableCategories"];
            enableComments = (bool)drowSection["Section_EnableComments"];
            enableModeration = (bool)drowSection["Section_EnableModeration"];
            enableRatings = (bool)drowSection["Section_EnableRatings"];
            enableCommentRatings = (bool)drowSection["Section_EnableCommentRatings"];
            enableNotifications = (bool)drowSection["Section_EnableNotifications"];
            enableIndexing = (bool)drowSection["Section_EnableIndexing"];
            inheritTransformations = (bool)drowSection["Section_InheritTransformations"];
            transformations = (string)drowSection["Section_Transformations"];
            areWebBoxesInherited = (bool)drowSection["Section_AreWebBoxesInherited"];
            webBoxes = SplitWebBoxes(drowWebBoxes);
            webBoxDisplayMode = (int)drowSection["Section_WebBoxDisplayMode"];
            areWebServiceBoxesInherited = (bool)drowSection["Section_AreWebServiceBoxesInherited"];
            webServiceBoxes = SplitServiceBoxes(drowWebServiceBoxes);
            webServiceBoxDisplayMode = (int)drowSection["Section_WebServiceBoxDisplayMode"];
            webServicePassword = (string)drowSection["Section_WebServicePassword"];
            allowHtmlInput = (AllowHtml)drowSection["Section_AllowHtmlInput"];
            allowCommentHtmlInput = (AllowHtml)drowSection["Section_AllowCommentHtmlInput"];
            isSectionWebService = (bool)drowSection["Section_IsSectionWebService"];
            isSectionSyndicated = (bool)drowSection["Section_IsSectionSyndicated"];
            isSystem = (bool)drowSection["Section_IsSystem"];
            isEnabled = (bool)drowSection["Section_IsEnabled"];
            recordsPerPage = ((int)drowSection["Section_RecordsPerPage"]);
            dateCreated = (DateTime)drowSection["Section_DateCreated"];
            content = (string)drowSection["Section_PageTypeName"];
            path = (string)drowSection["Section_Path"];

            viewRoles = SplitRoles(drowSectionSecurity, RoleType.ViewRole);
            addRoles = SplitRoles(drowSectionSecurity, RoleType.AddRole);
            editRoles = SplitRoles(drowSectionSecurity, RoleType.EditRole);
            deleteRoles = SplitRoles(drowSectionSecurity, RoleType.DeleteRole);
            commentRoles = SplitRoles(drowSectionSecurity, RoleType.CommentRole);
            moderateRoles = SplitRoles(drowSectionSecurity, RoleType.ModerateRole);
            rateRoles = SplitRoles(drowSectionSecurity, RoleType.RateRole);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Section"/> class.
        /// </summary>
        public Section() { }

        /// <summary>
        /// Splits the roles and loads them into a <see cref="List<string>"/> collection.
        /// </summary>
        /// <param name="allRoles">A <see cref="DataRow[]"/> containing a sting of roles.</param>
        /// <param name="sectionRoleType">The <see cref="RoleType"/> of the section role.</param>
        /// <returns>An <see cref="List<string>"/> of roles.</returns>
        private List<string> SplitRoles(DataRow[] allRoles, RoleType sectionRoleType) 
        {
            List<string> colRoles = new List<string>();

            foreach (DataRow row in allRoles)
                if ((RoleType)row["Security_RoleType"] == sectionRoleType)
                    colRoles.Add( row["Security_Role"].ToString() );

            return colRoles;
        }

        /// <summary>
        /// Splits the boxes and loads them into a <see cref="List<string>"/> collection.
        /// </summary>
        /// <param name="allBoxes">A <see cref="DataRow[]"/> containing a sting of boxes.</param>
        /// <returns>An <see cref="List<string>"/> of boxes.</returns>
        private List<string> SplitWebBoxes(DataRow[] allBoxes) 
        {
            List<string> colBoxes = new List<string>();

            foreach (DataRow row in allBoxes)
                colBoxes.Add(row["WebBox_Name"].ToString());
                
            return colBoxes;
        }

        /// <summary>
        /// Splits the service boxes and loads them into a <see cref="List<string>"/> collection.
        /// </summary>
        /// <param name="allBoxes">A <see cref="DataRow[]"/> containing a sting of service boxes.</param>
        /// <returns>An <see cref="List<string>"/> of boxes.</returns>
        private List<string> SplitServiceBoxes(DataRow[] allBoxes) 
        {
            List<string> colBoxes = new List<string>();

            foreach (DataRow row in allBoxes)
                colBoxes.Add(row["ServiceBox_Name"].ToString());

            return colBoxes;
        }
    }
}
