// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Data;
using System.Data.Common;

using Microsoft.Practices.EnterpriseLibrary.Data;

using DotCommunities.Framework.Interfaces;
using DotCommunities.Framework.Managers;
using DotCommunities.Framework.Enums;

namespace DotCommunities.Framework.Providers
{
    /// <summary>
    /// Helper object used to manage access to the Enterprise Library Data Access Block.
    /// </summary>
    public sealed class SqlProviderHelper : IProviderHelper
    {
        private string connectionStringName;

        /// <summary>
        /// Gets the connection string.
        /// </summary>
        /// <exception cref="ArgumentException">
        /// Thrown if the connection string is not specified.
        /// </exception>
        public string ConnectionStringName
        {
            get
            {
                if (string.IsNullOrEmpty(connectionStringName))
                    throw new ArgumentException("Provider connection string not specified");
                return connectionStringName;
            }
        }

        /// <summary>
        /// Creates a new instance of the <see cref="SqlProviderHelper"/> class.
        /// </summary>
        /// <param name="connectionStringName">The connection string to use for database access.</param>
        public SqlProviderHelper(string connectionStringName)
        {
            this.connectionStringName = connectionStringName;
        }

        /// <summary>
        /// Creates the database connection to use with the provider.
        /// </summary>
        public Database CreateDatabase()
        {
            return DatabaseFactory.CreateDatabase(ConnectionStringName);
        }

        /// <summary>
        /// Returns a <see cref="DbCommand"/> object containing the discovered parameters of a stored procedure.
        /// </summary>
        /// <param name="name">The name of the stored procedure to invoke.</param>
        /// <returns>A <see cref="DbCommand"/> object containing the correct parameters for the stored procedure.</returns>
        public DbCommand CreateCommand(string name)
        {
            return CreateDatabase().GetStoredProcCommand(name);
        }

        /// <summary>
        /// Returns a <see cref="DbCommand"/> object containing the discovered parameters of a stored procedure.
        /// </summary>
        /// <param name="name">The name of the stored procedure to invoke.</param>
        /// <param name="values">An parameterised list of values required by the stored procedure.</param>
        /// <returns>A <see cref="DbCommand"/> object containing the correct parameters for the stored procedure.</returns>
        public DbCommand CreateCommand(string name, params object[] values)
        {
            return CreateDatabase().GetStoredProcCommand(name, values);
        }

        /// <summary>
        /// Returns a <see cref="DataSet"/> of results from the database.
        /// </summary>
        /// <param name="command">The <see cref="DbCommand"/> to execute.</param>
        /// <returns>A <see cref="DataSet"/> containing the retrieved data.</returns>
        public DataSet ExecuteDataSet(DbCommand command)
        {
            // Log the command
            LogCommand(command);
            // Run the SQL
            return CreateDatabase().ExecuteDataSet(command);
        }

        /// <summary>
        /// Returns an <see cref="IDataReader"/> for the command.
        /// </summary>
        /// <remarks>
        /// Connection needs to be closed manually (can be manually disposed by wrapping call around a <c>using</c> statement).
        /// </remarks>
        /// <param name="command">The <see cref="DbCommand"/> to execute.</param>
        /// <returns>An <see cref="IDataReader"/> containing the retrieved data.</returns>
        public IDataReader CreateReader(DbCommand command)
        {
            // Log the command
            LogCommand(command);
            // Run the SQL
            return CreateDatabase().ExecuteReader(command);
        }

        /// <summary>
        /// Executes a command.
        /// </summary>
        /// <param name="command">The <see cref="DbCommand"/> to execute.</param>
        /// <returns>The return value of the command.</returns>
        public int Execute(DbCommand command)
        {
            // Log the command
            LogCommand(command);
            // Execute the query
            CreateDatabase().ExecuteNonQuery(command);
            // Finally, get the return parameter value
            return (int)command.Parameters["@RETURN_VALUE"].Value;
        }

        /// <summary>
        /// Writes the command text and parameters to the debug log.
        /// </summary>
        /// <param name="command">The <see cref="DbCommand"/> to log.</param>
        private void LogCommand(DbCommand command)
        {
            LogManager.WriteDebug(EventCode.Database, string.Format("Executing Command: {0}", command.CommandText), ParseCommandParameters(command.Parameters));
        }

        /// <summary>
        /// Parses the <see cref="DbParameterCollection"/> into a string for logging.
        /// </summary>
        /// <param name="parameters">The parameters to parse.</param>
        /// <returns>A string of formatted parameter information.</returns>
        private string ParseCommandParameters(DbParameterCollection parameters)
        {
            string result = "No Parameters";

            if (parameters.Count > 1)
            {
                result = "Parameters\n--------------------\n";
                foreach (DbParameter param in parameters)
                    if (param.ParameterName.ToUpper() != "@RETURN_VALUE")
                        result += string.Format("Name: {0} | Type: {1} | Direction: {2} | Value: {3}\n", 
                                    param.ParameterName, 
                                    param.DbType, 
                                    param.Direction, 
                                    param.Value);
                result += "--------------------";
            }

            return result;
        }
    }
}
