// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web.UI;
using System.Web.UI.WebControls;

using DotCommunities.Framework.Controls;
using DotCommunities.Framework.Components;
using DotCommunities.Framework.Managers;

using DotCommunities.Modules.Managers;

namespace DotCommunities.Modules.Controls
{
    /// <summary>
    /// Represents the Add Article page. Enables users to add new articles.
    /// </summary>
    public class ArticleAddPage : ContentAddPage
    {
        string _skinFileName = "Articles_AddArticle.ascx";

        string _sectionContent = "DotCommunities.Modules.Controls.ArticleListPage";

        TextBox txtTitle;
        Panel pnlCategories;
        CategoryCheckBoxList cblCategories;
        TextBox txtBriefDescription;
        TextEditor txtBodyText;

        Title lblPreviewTitle;
        DisplayCategories categoryPreview;
        BriefDescription lblPreviewBriefDescription;
        ArticleBodyText lblPreviewBodyText;

        /// <summary>
        ///  Calls the base <see cref="ContentAddPage"/> constructor
        /// and assigns the default page skin. 
        /// </summary>
        public ArticleAddPage()
            : base()
        {
            // Assign a default skin file name
            SkinFileName = _skinFileName;

            // Assign section content
            SectionContent = _sectionContent;

            // Wire-up event handlers
            this.SkinLoad += new SkinLoadEventHandler(SkinLoadArticle);
            this.Preview += new PreviewEventHandler(PreviewArticle);
            this.Submit += new SubmitEventHandler(SubmitArticle);
        }

        /// <summary>
        /// The skin load event happens after a page skin has been loaded.
        /// Here, we grab the necessary controls from the page skin.
        /// </summary>
        /// <param name="s">The sender of the event.</param>
        /// <param name="e">The <see cref="SkinLoadEventArgs"/> of the event.</param>
        void SkinLoadArticle(Object s, SkinLoadEventArgs e)
        {
            // Find Title
            txtTitle = (TextBox)GetControl(e.Skin, "txtTitle");

            // Find the Category panel
            pnlCategories = (Panel)GetControl(e.Skin, "pnlCategories");

            // Find Category Picker
            cblCategories = (CategoryCheckBoxList)GetControl(e.Skin, "cblCategories");

            // Set the visiblity of the category picker
            pnlCategories.Visible = 
                CurrentSection.EnableCategories &&
                cblCategories.Items.Count > 0;

            // Find Intro Text 
            txtBriefDescription = (TextBox)GetControl(e.Skin, "txtBriefDescription");

            // Find Body Text
            txtBodyText = (TextEditor)GetControl(e.Skin, "txtBodyText");

            // Find Preview Title
            lblPreviewTitle = (Title)GetControl(e.Skin, "lblPreviewTitle");

            // Find the category preview
            categoryPreview = (DisplayCategories)GetControl(e.Skin, "categoryPreview");

            // Find Preview Intro Text
            lblPreviewBriefDescription = (BriefDescription)GetControl(e.Skin, "lblPreviewBriefDescription");

            // Find Preview Body Text
            lblPreviewBodyText = (ArticleBodyText)GetControl(e.Skin, "lblPreviewBodyText");
        }

        /// <summary>
        /// When previewing an article, we want to transfer everything from 
        /// the text boxes to the labels.
        /// </summary>
        /// <param name="s">The sender of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> of the event.</param>
        void PreviewArticle(Object s, EventArgs e)
        {
            // Set the title text
            lblPreviewTitle.Text = txtTitle.Text;

            if (CurrentSection.EnableCategories)
                categoryPreview.SelectedCategories = cblCategories.SelectedCategories;

            // Set the description text
            lblPreviewBriefDescription.Text = txtBriefDescription.Text;

            // Set the body text
            lblPreviewBodyText.Text = txtBodyText.Text;
        }

        /// <summary>
        ///  This method is raised by clicking the Add button in the Add 
        /// Article form. It adds the article to the database.
        /// </summary>
        /// <param name="s">The sender of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> of the event.</param>
        void SubmitArticle(Object s, EventArgs e)
        {
            if (Page.IsValid)
            {
                // Check moderation status
                int moderationStatus = 1;
                if (CurrentSection.EnableModeration && !CurrentUser.MayModerate)
                    moderationStatus = 0;

                // Add to Content Pages
                int contentPageID = ModuleManager.AddArticle
                (
                    CurrentSection.SectionID,
                    CurrentUser.UserName,
                    cblCategories.SelectedCategories,
                    txtTitle.Text,
                    txtBriefDescription.Text,
                    moderationStatus,
                    txtBodyText.Text
                );

                // Show warning message if moderation enabled
                if (CurrentSection.EnableModeration && !CurrentUser.MayModerate)
                    Context.Response.Redirect(CommunityContext.CalculatePath("Messages_Message.aspx?message=moderation"));

                // Otherwise, redirect to default page and send notifications
                FrameworkManager.SendNotifications(CurrentSection.SectionID, contentPageID, CurrentUser.UserName, txtTitle.Text, txtBriefDescription.Text, CommunityContext.CalculatePath("Default.aspx"), true);
            }
        }
    }
}
