// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Web.UI;
using System.Web.UI.WebControls;

using DotCommunities.Framework.Controls;
using DotCommunities.Framework.Components;
using DotCommunities.Framework.Managers;

using DotCommunities.Modules.Managers;

namespace DotCommunities.Modules.Controls
{
    /// <summary>
    /// Represents the Edit Article page. Enables users to edit articles.
    /// </summary>
    public class ArticleEditPage : ContentEditPage
    {
        string _skinFileName = "Articles_AddArticle.ascx";

        string _sectionContent = "DotCommunities.Modules.Controls.ArticleListPage";

        TextBox txtTitle;
        Panel pnlCategories;
        CategoryCheckBoxList cblCategories;
        TextBox txtBriefDescription;
        TextEditor txtBodyText;

        Title lblPreviewTitle;
        DisplayCategories categoryPreview;
        BriefDescription lblPreviewBriefDescription;
        ArticleBodyText lblPreviewBodyText;

        /// <summary>
        /// Gets or sets the content page ID. Stores the ID of the content 
        /// being edited in View State.
        /// </summary>
        /// <value>The content page ID.</value>
        int ContentPageID
        {
            get { return (int)ViewState["ContentPageID"]; }
            set { ViewState["ContentPageID"] = value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ArticleEditPage"/> class.
        /// </summary>
        /// <remarks>
        /// Calls the base ContentEditPage constructor
        /// and assigns the default page skin. Also checks whether
        /// current user has perIssions to edit.
        /// </remarks>
        public ArticleEditPage()
            : base()
        {
            // Assign a default skin file name
            SkinFileName = _skinFileName;
            
            // Specify Section Content
            SectionContent = _sectionContent;
            
            // Wire-up event handlers
            this.SkinLoad += new SkinLoadEventHandler(SkinLoadArticle);
            this.Preview += new PreviewEventHandler(PreviewArticle);
            this.Submit += new SubmitEventHandler(SubmitArticle);
        }

        /// <summary>
        /// This method is called when the skin is being initialized.
        /// </summary>
        /// <param name="s">The sender of the event.</param>
        /// <param name="e">The <see cref="SkinLoadEventArgs"/> of the event.</param>
        void SkinLoadArticle(Object s, SkinLoadEventArgs e)
        {
            // Find Title
            txtTitle = (TextBox)GetControl(e.Skin, "txtTitle");

            // Find the Category panel
            pnlCategories = (Panel)GetControl(e.Skin, "pnlCategories");

            // Find Category Picker
            cblCategories = (CategoryCheckBoxList)GetControl(e.Skin, "cblCategories");

            // Set the visiblity of the category picker
            pnlCategories.Visible =
                CurrentSection.EnableCategories &&
                cblCategories.Items.Count > 0;

            // Find Intro Text 
            txtBriefDescription = (TextBox)GetControl(e.Skin, "txtBriefDescription");

            // Find Body Text
            txtBodyText = (TextEditor)GetControl(e.Skin, "txtBodyText");

            // Find Preview Title
            lblPreviewTitle = (Title)GetControl(e.Skin, "lblPreviewTitle");

            // Find the topic preview
            categoryPreview = (DisplayCategories)GetControl(e.Skin, "categoryPreview");

            // Find Preview Intro Text
            lblPreviewBriefDescription = (BriefDescription)GetControl(e.Skin, "lblPreviewBriefDescription");

            // Find Preview Body Text
            lblPreviewBodyText = (ArticleBodyText)GetControl(e.Skin, "lblPreviewBodyText");
        }

        /// <summary>
        /// Raises the <see cref="Control.Load"/> event, assigning previous content values to edit form.
        /// </summary>
        /// <param name="e">The <see cref="EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                ContentPageID = Int32.Parse(Context.Request.QueryString["id"]);

                // Get the article info
                Article _articleInfo = (Article)ModuleManager.GetArticle(ContentPageID, CurrentUser.UserName);

                // Assign Values
                EnsureChildControls();

                txtTitle.Text = _articleInfo.Title;
                cblCategories.SelectedCategories = _articleInfo.Categories;
                txtBriefDescription.Text = _articleInfo.BriefDescription;
                txtBodyText.Text = _articleInfo.BodyText;
            }
        }

        /// <summary>
        /// Previews the article. This method is raised by clicking the Preview button in the Edit form.
        /// </summary>
        /// <param name="s">The sender of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> of the event.</param>
        void PreviewArticle(Object s, EventArgs e)
        {
            // Set the title text
            lblPreviewTitle.Text = txtTitle.Text;

            if (CurrentSection.EnableCategories)
                categoryPreview.SelectedCategories = cblCategories.SelectedCategories;

            // Set the description text
            lblPreviewBriefDescription.Text = txtBriefDescription.Text;

            // Set the body text
            lblPreviewBodyText.Text = txtBodyText.Text;
        }

        /// <summary>
        /// Submits the article. This method is raised by clicking the Edit button in the Edit form.
        /// </summary>
        /// <param name="s">The sender of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> of the event.</param>
        void SubmitArticle(Object s, EventArgs e)
        {
            if (Page.IsValid)
            {
                // TODO Allow for moderators to adjust meta keys and description

                // Edit Content Page
                ModuleManager.EditArticle
                (
                    CurrentSection.SectionID,
                    ContentPageID,
                    CurrentUser.UserName,
                    cblCategories.SelectedCategories,
                    txtTitle.Text,
                    txtBriefDescription.Text,
                    CommunityContext.ParseMetaDescription(txtBriefDescription.Text),
                    CommunityContext.ParseMetaKeys(txtBriefDescription.Text),
                    txtBodyText.Text
                );

                // Send notification and redirect back to Article Page
                FrameworkManager.SendNotifications(CurrentSection.SectionID, ContentPageID, CurrentUser.UserName, txtTitle.Text, txtBriefDescription.Text, CommunityContext.CalculatePath(String.Format("{0}.aspx", ContentPageID)));
            }
        }
    }
}
