// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections.Generic;

using DotCommunities.Framework;
using DotCommunities.Framework.Managers;
using DotCommunities.Framework.Components;
using DotCommunities.Modules.Properties;

namespace DotCommunities.Modules.Managers
{
    /// <summary>
    /// The <see cref="ModuleManager"/> configures the <see cref="ModuleProvider"/>.
    /// </summary>
    public partial class ModuleManager : FrameworkManagerBase
    {
        /// <summary>
        /// Event fired when the sort order changes.  Removes the articles from the cache.
        /// </summary>
        /// <param name="sender">The sender is the current section.</param>
        /// <param name="e">The EventArgs are empty.</param>
        static void Article_SortOrderChanged(object sender, EventArgs e)
        {
            Section section = (Section)sender;
            if (section != null)
                ClearArticles(section.SectionID, -1);
        }

        /// <summary>
        /// Removes Article objects from the cache.
        /// </summary>
        private static void ClearArticles(int sectionID, int contentPageID)
        {
            // Remove the specific article from the cache
            if (contentPageID > 0)
                RemoveCachedItem(string.Format(Resources.ArticleCacheKey, CommunityContext.CurrentCommunity.ID, contentPageID));
            // Remove the list of articles from the cache for all users
            RemoveCachedItemByPattern(string.Format(Resources.ArticlesCacheRemovalPattern, sectionID));
            // Remove any content page specific items from the cache
            FrameworkManager.ClearContentPages(sectionID, contentPageID);
        }

        /// <summary>
        /// Gets a particular article from the database.
        /// </summary>
        /// <param name="contentPageID">The content page ID.</param>
        /// <param name="userName">The username.</param>
        /// <returns>An <see cref="Article"/> object cast as a <see cref="CommunityContext"/> object.</returns>
        public static CommunityContent GetArticle(int contentPageID, string userName)
        {
            string cacheKey = string.Format(Resources.ArticleCacheKey, CommunityContext.CurrentCommunity.ID, contentPageID);

            object article = GetCachedItem(cacheKey);

            if (article != null && article is Article)
                return (Article)article;

            return (Article)AddCachedItem(cacheKey, Provider.GetArticle(CommunityContext.CurrentCommunity.ID, contentPageID, userName));
        }

        /// <summary>
        /// Gets the articles for this section. 
        /// </summary>
        /// <param name="sectionID">The section ID.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <param name="pageIndex">Index of the page.</param>
        /// <param name="sortOrder">The sort order.</param>
        /// <returns>A list of <see cref="Article"/> objects.</returns>
        public static List<object> GetArticles(int sectionID, int pageSize, int pageIndex, string sortOrder)
        {
            string cacheKey = string.Format(Resources.ArticlesCacheKey, sectionID, CommunityContext.CurrentUser.UserName, pageSize, pageIndex, sortOrder);

            object articles = GetCachedItem(cacheKey);

            if (articles != null && articles is List<object>)
                return (List<object>)articles;

            return (List<object>)AddCachedItemFor60Seconds(cacheKey, Provider.GetArticles(CommunityContext.CurrentCommunity.ID, sectionID, CommunityContext.CurrentUser.UserName, pageSize, pageIndex, sortOrder));
        }

        /// <summary>
        /// Adds a new <see cref="Article"/> to the database.
        /// </summary>
        /// <param name="sectionID">The current Section ID.</param>
        /// <param name="userName">The name of the user adding the content.</param>
        /// <param name="categories">A dictionary list of associated categories.</param>
        /// <param name="title">The article title.</param>
        /// <param name="briefDescription">The article description.</param>
        /// <param name="moderationStatus">The moderation status of the article.</param>
        /// <param name="body">The body of the article.</param>
        /// <returns>The content page ID of the article returned when added to the database.</returns>
        public static int AddArticle(int sectionID, string userName, Dictionary<int, Category> categories, string title, string briefDescription, int moderationStatus, string body)
        {
            try
            {
                string categoryIDs = FrameworkManager.ParseCategoryIDs(categories);
                string metaDescription = CommunityContext.ParseMetaDescription(briefDescription);
                string metaKeys = CommunityContext.ParseMetaKeys(briefDescription);

                // Add the article
                return Provider.AddArticle(
                    CommunityContext.CurrentCommunity.ID, 
                    sectionID, 
                    userName,
                    categoryIDs, 
                    title, 
                    briefDescription, 
                    metaDescription, 
                    metaKeys, 
                    moderationStatus, 
                    body);
            }
            finally
            {
                ClearArticles(sectionID, -1);
            }
        }

        /// <summary>
        /// Edits an exisiting article.
        /// </summary>
        /// <param name="sectionID">The current Section ID.</param>
        /// <param name="contentPageID">The content page ID of the article.</param>
        /// <param name="userName">The name of the user adding the content.</param>
        /// <param name="categories">A dictionary list of associated categories.</param>
        /// <param name="title">The article title.</param>
        /// <param name="briefDescription">The article description.</param>
        /// <param name="body">The body of the article.</param>
        public static void EditArticle(int sectionID, int contentPageID, string userName, Dictionary<int, Category> categories, string title, string briefDescription, string metaDescription, string metaKeys, string body)
        {
            try
            {
                string categoryIDs = FrameworkManager.ParseCategoryIDs(categories);

                // Edit the article
                Provider.EditArticle(
                    CommunityContext.CurrentCommunity.ID,
                    contentPageID,
                    userName,
                    categoryIDs,
                    title,
                    briefDescription,
                    metaDescription,
                    metaKeys,
                    body);
            }
            finally
            {
                ClearArticles(sectionID, contentPageID);
            }
        }
    }
}
