// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

using ManagementInstrumentation = System.Management.Instrumentation.Instrumentation;

using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;

using DotCommunities.Blocks.Tasks.Properties;

namespace DotCommunities.Blocks.Tasks.Instrumentation
{
    /// <summary>
    /// The instrumentation gateway for the task block when no instances of the objects are involved.
    /// </summary>
    [EventLogDefinition("Application", EventLogSourceName)]
    [CustomFactory(typeof(TaskEventLoggerCustomFactory))]
    public class TaskEventLogger : InstrumentationListener
    {
		private IEventLogEntryFormatter eventLogEntryFormatter;

        /// <summary>
        /// Made public for testing purposes.
        /// </summary>
        public const string EventLogSourceName = ".communities Tasks";

        /// <summary>
        /// Initializes a new instance of the <see cref="TaskEventLogger"/> class, specifying whether 
        /// logging to the event log and firing WMI events is allowed.
        /// </summary>
        /// <param name="eventLoggingEnabled"><code>true</code> if writing to the event log is allowed, <code>false</code> otherwise.</param>
        /// <param name="wmiEnabled"><code>true</code> if firing WMI events is allowed, <code>false</code> otherwise.</param>
        public TaskEventLogger(bool eventLoggingEnabled, bool wmiEnabled)
			: base((string)null, false, eventLoggingEnabled, wmiEnabled, null)
        {
			eventLogEntryFormatter = new EventLogEntryFormatter(Resources.BlockName);
        }

        /// <summary>
        /// Logs the occurrence of a configuration error for the Tasks Application Block through the 
        /// available instrumentation mechanisms.
        /// </summary>
        /// <param name="instanceName">The name of the instance this errors applies to.</param>
        /// <param name="messageTemplate">The format of the message that describes the error, with as parameter ({0}) the <paramref name="instanceName"/>.</param>
        /// <param name="exception">The exception raised for the configuration error.</param>
        /// <exception cref="FormatException"><paramref name="messageTemplate"/> could not be formatted by <see cref="String.Format(System.IFormatProvider, string, object[])"/> given the parameter <paramref name="instanceName"/>.</exception>
        public void LogConfigurationError(string instanceName, string messageTemplate, Exception exception)
        {
			if (WmiEnabled) FireManagementInstrumentation(new TaskConfigurationFailureEvent(instanceName, exception.ToString()));
            if (EventLoggingEnabled)
            {
				string errorMessage
					= string.Format(
						Resources.Culture,
						messageTemplate,
						instanceName);
				string entryText = eventLogEntryFormatter.GetEntryText(errorMessage, exception);

				EventLog.WriteEntry(GetEventSourceName(), entryText, EventLogEntryType.Error);
			}
        }
    }
}
