// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Diagnostics;

namespace DotCommunities.Blocks.Tasks
{
    /// <summary>
    /// Event handler for task-based events.
    /// </summary>
    /// <param name="sender">The sender of the event.</param>
    /// <param name="e">The <see cref="TaskEventArgs"/> of the task.</param>
    public delegate void TaskEventHandler(object sender, TaskEventArgs e);

    /// <summary>
    /// Task is an abtract class which contains all the base
    /// properties and methods of a task object.
    /// </summary>
    public abstract class Task : ITask
    {
        private string name;
        private int minutes;

        private DateTime lastExecuted = DateTime.MinValue;
        private TimeSpan lastExecutionTime = TimeSpan.Zero;

        /// <summary>
        /// Event fired when a task has started executing.
        /// </summary>
        public event TaskEventHandler TaskStarted;

        /// <summary>
        /// Event fired when a task has completed executing.
        /// </summary>
        public event TaskEventHandler TaskCompleted;

        /// <summary>
        /// Event fired when a task has failed.
        /// </summary>
        public event TaskEventHandler TaskFailed;

        /// <summary>
        /// The name of task.
        /// </summary>
        public string Name
        {
            get { return this.name; }
            set { this.name = value; }
        }

        /// <summary>
        /// The number of minutes between runs.
        /// </summary>
        public int Minutes
        {
            get { return minutes; }
            set { minutes = value; }
        }

        /// <summary>
        /// The time the task was last executed.
        /// </summary>
        public DateTime LastExecuted
        {
            get { return lastExecuted; }
        }

        /// <summary>
        /// The <see cref="TimeSpan"/> between the current time and the last execution time.
        /// </summary>
        public TimeSpan ExecutionDelay
        {
            get 
            {
                if (lastExecuted == DateTime.MinValue)
                    return TimeSpan.Zero;
                else
                    return new TimeSpan(DateTime.Now.Ticks - lastExecuted.Ticks);
            }
        }

        /// <summary>
        /// The time the task took to complete.
        /// </summary>
        public TimeSpan LastExecutionTime
        {
            get { return lastExecutionTime; }
        }

        /// <summary>
        /// Invokes the <see cref="TaskStarted"/> event.
        /// </summary>
        /// <param name="e">The <see cref="TaskEventArgs"/> of the event.</param>
        protected virtual void OnTaskStarted(TaskEventArgs e)
        {
            // Fire the event if it is available
            if (TaskStarted != null)
                TaskStarted(this, e);
        }

        /// <summary>
        /// Invokes the <see cref="TaskCompleted"/> event.
        /// </summary>
        /// <param name="e">The <see cref="TaskEventArgs"/> of the event.</param>
        protected virtual void OnTaskCompleted(TaskEventArgs e)
        {
            // Fire the event if it is available
            if (TaskCompleted != null)
                TaskCompleted(this, e);
        }

        /// <summary>
        /// Invokes the <see cref="TaskFailed"/> event.
        /// </summary>
        /// <param name="e">The <see cref="TaskEventArgs"/> of the event.</param>
        protected virtual void OnTaskFailed(TaskEventArgs e)
        {
            // Fire the event if it is available
            if (TaskFailed != null)
                TaskFailed(this, e);
        }

        /// <summary>
        /// When implemented in a derived class, Executes the task.
        /// </summary>
        /// <returns><c>true</c> if the task is completed, otherwise <c>false</c>.</returns>
        public abstract bool Execute();

        /// <summary>
        /// Method used by the <see cref="BackgroundScheduler"/> to execute a task.
        /// This method should not be called directly.
        /// </summary>
        /// <returns>
        /// <c>true</c>, if the task executed sucessfully; otherwise <c>false</c>.
        /// </returns>
        internal bool ExecuteTask()
        {
            bool result = false;

            // Set the start time for this task
            DateTime taskStarted = DateTime.Now;

            // Fire the starting events
            OnTaskStarted(new TaskEventArgs(Name, taskStarted));

            try
            {
                // Execute the task
                result = Execute();

                // Set the last executed time
                SetLastExecutionTime(taskStarted);

                // Fire task events after the task has completed
                if (result)
                    OnTaskCompleted(new TaskEventArgs(Name, taskStarted, lastExecuted));
                else
                    OnTaskFailed(new TaskEventArgs(Name, taskStarted, lastExecuted));
            }
            catch
            {
                // Reset the last executed time
                SetLastExecutionTime(taskStarted);
                // If we get an error then fire the task failed event
                OnTaskFailed(new TaskEventArgs(Name, taskStarted, lastExecuted));
            }

            // Return the result of the task
            return result;
        }

        /// <summary>
        /// Method used to set the last executed times.
        /// </summary>
        /// <param name="taskStarted">The time the task was started.</param>
        internal void SetLastExecutionTime(DateTime taskStarted)
        {
            // Set the last executed time to now
            lastExecuted = DateTime.Now;
            // Set the last executed timespan
            lastExecutionTime = new TimeSpan(lastExecuted.Ticks - taskStarted.Ticks);
        }
    }
}
