// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections.Generic;
using System.Threading;

using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;

using DotCommunities.Blocks.Tasks.Configuration;

namespace DotCommunities.Blocks.Tasks
{
    /// <summary>
    /// This class represents the interface to tasks as shown to the user. 
    /// All task operations are performed through this class.
    /// </summary>
    [ConfigurationNameMapper(typeof(TaskManagerDataRetriever))]
    [CustomFactory(typeof(TaskManagerCustomFactory))]
    public class TaskManager : IDisposable
	{
		private BackgroundScheduler scheduler;
        private Dictionary<string, Task> taskList;
        private Timer singleTimer = null;
        private int minutes = 15;
        private bool running = false;

        /// <summary>
        /// Initialises the task queue with the default delay of 15 minutes between runs.
        /// </summary>
        /// <param name="scheduler">The <see cref="BackgroundScheduler"/> to use.</param>
        /// <param name="taskList">A list of <see cref="Task"/>s to perform.</param>
        internal TaskManager(BackgroundScheduler scheduler, Dictionary<string, Task> taskList)
            : this(scheduler, taskList, 15) {}

        /// <summary>
        /// Initialises the task queue.
        /// </summary>
        /// <param name="scheduler">The <see cref="BackgroundScheduler"/> to use.</param>
        /// <param name="taskList">A list of <see cref="Task"/>s to perform.</param>
        /// <param name="minutes">The number of minutes between runs.</param>
        internal TaskManager(BackgroundScheduler scheduler, Dictionary<string, Task> taskList, int minutes)
		{
			this.scheduler = scheduler;
            this.taskList = taskList;
            this.minutes = minutes;

            // Create a new timer to add tasks to the queue
            singleTimer = new Timer(new TimerCallback(TaskTimer_CallBack), null, Timeout.Infinite, Interval);
		}

        /// <summary>
        /// Starts the <see cref="TaskManager"/>.
        /// </summary>
        public void Start()
        {
            Start(0);
        }

        /// <summary>
        /// Starts the <see cref="TaskManager"/>.
        /// </summary>
        /// <param name="startupDelay">The number of milliseconds to wait between startup and processing.</param>
        public void Start(int startupDelay)
        {
            if (!running)
                singleTimer.Change(startupDelay, Interval);
            running = true;
        }

        /// <summary>
        /// Stops the <see cref="TaskManager"/>.
        /// </summary>
        public void Stop()
        {
            if (running)
                singleTimer.Change(Timeout.Infinite, Timeout.Infinite);
            running = false;
        }

        /// <summary>
        /// Gets the value indicating if the tasks are currently being executed.
        /// </summary>
        /// <value>
        /// Returns <c>true</c>, if running; otherwise <c>false</c>.
        /// </value>
        public bool Running
        {
            get { return running; }
        }

        /// <summary>
        /// Returns the number of minutes between processing tasks.
        /// </summary>
        public int Interval
        {
            get { return minutes * 60000; }
        }

        /// <summary>
		/// Returns the number of items currently in the task list.
		/// </summary>
		public int Count
		{
			get { return taskList.Count; }
		}

		/// <summary>
		/// Returns true if the key refers to a task in the task list.
		/// </summary>
		/// <param name="key">Key of item to check for.</param>
        /// <returns><c>true</c> if item referenced by key is in the task list.</returns>
		public bool Contains(string key)
		{
			return taskList.ContainsKey(key);
		}

		/// <summary>
		/// Returns the item identified by the provided key.
		/// </summary>
		/// <param name="key">Key to retrieve from task list.</param>
		/// <exception cref="ArgumentNullException">Provided key is null.</exception>
		/// <exception cref="ArgumentException">Provided key is an empty string.</exception>
		/// <remarks>
        /// The <see cref="TaskManager"/> can be configured to use custom code to perform tasks.
		/// Each of these tasks can throw exceptions particular to their own implementations.
        /// </remarks>
		public Task this[string key]
		{
			get { return taskList[key]; }
		}

		/// <summary>
		/// Adds new <see cref="Task"/> to task list.
		/// </summary>
		/// <param name="key">Identifier for this <see cref="Task"/>.</param>
        /// <param name="value">The <see cref="Task"/> to be added.</param>
		/// <exception cref="ArgumentNullException">Provided key is null.</exception>
		/// <exception cref="ArgumentException">Provided key is an empty string.</exception>
		/// <remarks>
        /// The <see cref="TaskManager"/> can be configured to use custom code to perform tasks.
        /// Each of these tasks can throw exceptions particular to their own implementations.
        /// </remarks>
		public void Add(string key, Task value)
		{
			taskList.Add(key, value);
		}

		/// <summary>
		/// Removes the given item from the cache. If no item exists with that key, this method does nothing.
		/// </summary>
		/// <param name="key">Key of item to remove from cache.</param>
        /// <exception cref="ArgumentNullException">Provided key is null.</exception>
        /// <exception cref="ArgumentException">Provided key is an empty string.</exception>
        /// <remarks>
        /// The <see cref="TaskManager"/> can be configured to use custom code to perform tasks.
        /// Each of these tasks can throw exceptions particular to their own implementations.
        /// </remarks>
        public void Remove(string key)
		{
			taskList.Remove(key);
		}

		/// <summary>
		/// Returns the value associated with the given key.
		/// </summary>
		/// <param name="key">Key of item to return from cache.</param>
		/// <returns>Value stored in cache</returns>
        /// <exception cref="ArgumentNullException">Provided key is null.</exception>
        /// <exception cref="ArgumentException">Provided key is an empty string.</exception>
        /// <remarks>
        /// The <see cref="TaskManager"/> can be configured to use custom code to perform tasks.
        /// Each of these tasks can throw exceptions particular to their own implementations.
        /// </remarks>
        public Task GetTask(string key)
		{
			return taskList[key];
		}

		/// <summary>
		/// Removes all items from the cache. If an error occurs during the removal, the cache is left unchanged.
		/// </summary>
        /// <remarks>
        /// The <see cref="TaskManager"/> can be configured to use custom code to perform tasks.
        /// Each of these tasks can throw exceptions particular to their own implementations.
        /// </remarks>
        public void Flush()
		{
			taskList.Clear();
		}

		/// <summary>
		/// Not intended for public use. Only public due to requirements of IDisposable. 
        /// If you call this method, your task scheduler will be unusable.
		/// </summary>
		public void Dispose()
		{
			GC.SuppressFinalize(this);

			if (scheduler != null)
			{
				scheduler.Stop();
				scheduler = null;
			}

            taskList = null;
		}

        /// <summary>
        /// Callback from the internal timer. This method is where tasks are added to the queue.
        /// </summary>
        /// <param name="state">A custom value that can be returned from the timer (not used).</param>
        private void TaskTimer_CallBack(object state)
        {
            // Loop through the task list and add the tasks to the scheduler
            foreach (Task t in taskList.Values)
                if (t != null && (!scheduler.IsInQueue(t)) && 
                    (t.ExecutionDelay == TimeSpan.Zero || 
                     t.ExecutionDelay.Minutes >= t.Minutes))
                    scheduler.Add(t);
        }
    }
}
