// =========================================================================
// .communities - The Free Community Content Management Framework
// Copyright  2007 by Ed Quinn (http://edquinn.net)
// =========================================================================
//
// For more information about this software, visit:
// http://www.DotCommunities.org (official site)
// http://www.codeplex.com/DotCommunities (source code)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Shared Source License displayed here:
// http://www.codeplex.com/DotCommunities/Project/License.aspx
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Shared Source License for more details.
//
// Please do not remove any copyright or other notices from this Software.
//
// =========================================================================

using System;
using System.Collections.Generic;
using System.Configuration;
using System.Threading;

using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.ObjectBuilder;

using DotCommunities.Blocks.Tasks.Configuration;
using DotCommunities.Blocks.Tasks.Instrumentation;

namespace DotCommunities.Blocks.Tasks
{
    /// <summary>
    /// This type supports the Enterprise Library infrastructure and is not intended to be used directly from your code.
    /// Represents the process used to build an instance of <see cref="TaskManagerData"/> described by the <see cref="TaskManagerSettings"/> configuration section.
    /// </summary>
    /// <remarks>
    /// This is used by the <see cref="ConfiguredObjectStrategy"/> when an instance of the <see cref="TaskManagerData"/> class is requested to 
    /// a properly configured <see cref="IBuilder{TStageEnum}"/> instance.
    /// </remarks>
    public class TaskManagerCustomFactory : ICustomFactory
	{
        /// <summary>
        /// This method supports the Enterprise Library infrastructure and is not intended to be used directly from your code.
        /// Builds a <see cref="TaskManagerData"/> described by the <see cref="TaskManagerSettings"/> configuration section.
        /// </summary>
        /// <param name="context">The <see cref="IBuilderContext"/> that represents the current building process.</param>
        /// <param name="name">The name of the instance to build. It is part of the <see cref="ICustomFactory.CreateObject(IBuilderContext, string, IConfigurationSource, ConfigurationReflectionCache)"/> method, but it is not used in this implementation.</param>
        /// <param name="configurationSource">The source for configuration objects.</param>
        /// <param name="reflectionCache">The cache to use retrieving reflection information.</param>
        /// <returns>A fully initialized instance of <see cref="TaskManager"/>.</returns>
        public object CreateObject(IBuilderContext context, string name, IConfigurationSource configurationSource, ConfigurationReflectionCache reflectionCache)
		{
			TaskManagerData objectConfiguration = GetConfiguration(name, configurationSource);
			TaskInstrumentationProvider instrumentationProvider = CreateInstrumentationProvider(objectConfiguration.Name, configurationSource, reflectionCache);

            TaskManager createdObject
                = new TaskManagerFactoryHelper().BuildTaskManager(
                    objectConfiguration.Name,
                    objectConfiguration.Minutes,
                    objectConfiguration.Tasks,
                    instrumentationProvider);

            RegisterObject(context, name, createdObject);

            return createdObject;
		}

        private static TaskInstrumentationProvider CreateInstrumentationProvider(string name, IConfigurationSource configurationSource, ConfigurationReflectionCache reflectionCache)
        {
            TaskInstrumentationProvider instrumentationProvider = new TaskInstrumentationProvider();
            new InstrumentationAttachmentStrategy().AttachInstrumentation(name, instrumentationProvider, configurationSource, reflectionCache);

            return instrumentationProvider;
        }

		private TaskManagerData GetConfiguration(string id, IConfigurationSource configurationSource)
		{
            TaskConfigurationView view = new TaskConfigurationView(configurationSource);
            return view.GetTaskManagerData(id);
		}

        private void RegisterObject(IBuilderContext context, string name, TaskManager createdObject)
		{
			if (context.Locator != null)
			{
				ILifetimeContainer lifetime = context.Locator.Get<ILifetimeContainer>(typeof(ILifetimeContainer), SearchMode.Local);

				if (lifetime != null)
				{
					context.Locator.Add(new DependencyResolutionLocatorKey(typeof(TaskManager), name), createdObject);
					lifetime.Add(createdObject);
				}
			}
		}
	}

	/// <summary>
	/// Public for thesting purposes
	/// </summary>
    internal class TaskManagerFactoryHelper
    {
        /// <summary>
        /// Made public for testing purposes.
        /// </summary>
        public TaskManager BuildTaskManager(
            string taskManagerName,
            int minutes,
            NamedElementCollection<TaskData> tasks,
            TaskInstrumentationProvider instrumentationProvider)
        {
            BackgroundScheduler scheduler = new BackgroundScheduler(instrumentationProvider);

            Dictionary<string, Task> taskList = new Dictionary<string, Task>();
            foreach (TaskData data in tasks)
            {
                Task task = (Task)Activator.CreateInstance(data.Type);
                task.Name = data.Name;
                task.Minutes = data.Minutes;
                taskList.Add(task.Name, task);
            }


            scheduler.Start();

            return new TaskManager(scheduler, taskList, minutes);
        }
    }
}
